@@ -5,7 +5,6 @@
 package mozilla.components.browser.engine.system
 
 import android.content.Context
-import android.net.Uri
 import android.os.Bundle
 import android.webkit.WebChromeClient
 import android.webkit.WebResourceRequest
@@ -14,8 +13,8 @@ import android.webkit.WebStorage
 import android.webkit.WebView
 import android.webkit.WebViewClient
 import android.webkit.WebViewDatabase
+import androidx.core.net.toUri
 import androidx.test.ext.junit.runners.AndroidJUnit4
-import kotlinx.coroutines.ExperimentalCoroutinesApi
 import kotlinx.coroutines.test.runTest
 import mozilla.components.browser.engine.system.matcher.UrlMatcher
 import mozilla.components.browser.errorpages.ErrorType
@@ -102,15 +101,15 @@ class SystemEngineSessionTest {
 
         assertNotNull(loadHeaders)
         assertEquals(1, loadHeaders!!.size)
-        assertTrue(loadHeaders!!.containsKey("X-Requested-With"))
-        assertEquals("", loadHeaders!!["X-Requested-With"])
+        assertTrue(loadHeaders.containsKey("X-Requested-With"))
+        assertEquals("", loadHeaders["X-Requested-With"])
 
         val extraHeaders = mapOf("X-Extra-Header" to "true")
         engineSession.loadUrl("http://mozilla.org", additionalHeaders = extraHeaders)
         assertNotNull(loadHeaders)
-        assertEquals(2, loadHeaders!!.size)
-        assertTrue(loadHeaders!!.containsKey("X-Extra-Header"))
-        assertEquals("true", loadHeaders!!["X-Extra-Header"])
+        assertEquals(2, loadHeaders.size)
+        assertTrue(loadHeaders.containsKey("X-Extra-Header"))
+        assertEquals("true", loadHeaders["X-Extra-Header"])
     }
 
     @Test
@@ -306,7 +305,7 @@ class SystemEngineSessionTest {
         var onGotoHistoryIndexTriggered = false
         val engineSession = spy(SystemEngineSession(testContext))
         val settings = mock<WebSettings>()
-        val webView = mock<WebView>() {
+        val webView = mock<WebView> {
             whenever(this.settings).thenReturn(settings)
             whenever(copyBackForwardList()).thenReturn(mock())
         }
@@ -332,7 +331,9 @@ class SystemEngineSessionTest {
         try {
             engineSession.restoreState(mock())
             fail("Expected IllegalArgumentException")
-        } catch (e: IllegalArgumentException) {}
+        } catch (e: IllegalArgumentException) {
+            // Expected
+        }
         assertFalse(engineSession.restoreState(SystemEngineSessionState(Bundle())))
         verify(webView, never()).restoreState(mockitoAny(Bundle::class.java))
 
@@ -350,10 +351,9 @@ class SystemEngineSessionTest {
         verify(webView).restoreState(bundle)
     }
 
-    @ExperimentalCoroutinesApi
     @Test
     fun enableTrackingProtection() = runTest {
-        SystemEngineView.URL_MATCHER = UrlMatcher(arrayOf(""))
+        SystemEngineView.urlMatcher = UrlMatcher(arrayOf(""))
 
         val engineSession = spy(SystemEngineSession(testContext))
         val webView = mock<WebView>()
@@ -590,7 +590,7 @@ class SystemEngineSessionTest {
         engineView.render(engineSession)
 
         val request: WebResourceRequest = mock()
-        doReturn(Uri.parse("sample:about")).`when`(request).url
+        doReturn("sample:about".toUri()).`when`(request).url
 
         val response = engineSession.webView.webViewClient.shouldInterceptRequest(
             engineSession.webView,
@@ -612,7 +612,7 @@ class SystemEngineSessionTest {
         val request: WebResourceRequest = mock()
         doReturn(true).`when`(request).isForMainFrame
         doReturn(true).`when`(request).hasGesture()
-        doReturn(Uri.parse(url)).`when`(request).url
+        doReturn(url.toUri()).`when`(request).url
 
         val engineSession = SystemEngineSession(testContext)
         engineSession.webView = spy(engineSession.webView)
@@ -629,7 +629,7 @@ class SystemEngineSessionTest {
         val redirect: WebResourceRequest = mock()
         doReturn(true).`when`(redirect).isForMainFrame
         doReturn(false).`when`(redirect).hasGesture()
-        doReturn(Uri.parse("sample:about")).`when`(redirect).url
+        doReturn("sample:about".toUri()).`when`(redirect).url
 
         engineSession.webView.webViewClient.shouldInterceptRequest(engineSession.webView, redirect)
 
@@ -641,7 +641,7 @@ class SystemEngineSessionTest {
         val request: WebResourceRequest = mock()
         doReturn(true).`when`(request).isForMainFrame
         doReturn(true).`when`(request).hasGesture()
-        doReturn(Uri.parse("sample:about")).`when`(request).url
+        doReturn("sample:about".toUri()).`when`(request).url
 
         val interceptor = object : RequestInterceptor {
             override fun onLoadRequest(
@@ -704,7 +704,7 @@ class SystemEngineSessionTest {
         engineView.render(engineSession)
 
         val request: WebResourceRequest = mock()
-        doReturn(Uri.parse("sample:about")).`when`(request).url
+        doReturn("sample:about".toUri()).`when`(request).url
 
         val response = engineSession.webView.webViewClient.shouldInterceptRequest(
             engineSession.webView,
@@ -726,7 +726,7 @@ class SystemEngineSessionTest {
         engineView.render(engineSession)
 
         val request: WebResourceRequest = mock()
-        doReturn(Uri.parse("sample:about")).`when`(request).url
+        doReturn("sample:about".toUri()).`when`(request).url
 
         val response = engineSession.webView.webViewClient.shouldInterceptRequest(
             engineSession.webView,
@@ -764,7 +764,7 @@ class SystemEngineSessionTest {
         engineView.render(engineSession)
 
         val request: WebResourceRequest = mock()
-        doReturn(Uri.parse("sample:about")).`when`(request).url
+        doReturn("sample:about".toUri()).`when`(request).url
 
         val response = engineSession.webView.webViewClient.shouldInterceptRequest(
             engineSession.webView,
