@@ -4,10 +4,11 @@
 
 package mozilla.components.browser.state.reducer
 
-import android.net.Uri
+import androidx.core.net.toUri
 import mozilla.components.browser.state.action.ContentAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction
 import mozilla.components.browser.state.ext.containsPermission
+import mozilla.components.browser.state.ext.mergePermissions
 import mozilla.components.browser.state.state.BrowserState
 import mozilla.components.browser.state.state.ContentState
 import mozilla.components.browser.state.state.SessionState
@@ -181,6 +182,7 @@ internal object ContentStateReducer {
                         permissionRequestsList = it.permissionRequestsList + action.permissionRequest,
                     )
                 } else {
+                    it.permissionRequestsList.mergePermissions(action.permissionRequest)
                     it
                 }
             }
@@ -205,6 +207,7 @@ internal object ContentStateReducer {
                         appPermissionRequestsList = it.appPermissionRequestsList + action.appPermissionRequest,
                     )
                 } else {
+                    it.appPermissionRequestsList.mergePermissions(action.appPermissionRequest)
                     it
                 }
             }
@@ -238,7 +241,7 @@ internal object ContentStateReducer {
             is ContentAction.SetRecordingDevices -> updateContentState(state, action.sessionId) {
                 it.copy(recordingDevices = action.devices)
             }
-            is ContentAction.UpdateDesktopModeAction -> updateContentState(state, action.sessionId) {
+            is ContentAction.UpdateTabDesktopMode -> updateContentState(state, action.sessionId) {
                 it.copy(desktopMode = action.enabled)
             }
             is UpdatePermissionHighlightsStateAction.NotificationChangedAction -> {
@@ -271,6 +274,16 @@ internal object ContentStateReducer {
                     it.copy(persistentStorageChanged = action.value)
                 }
             }
+            is UpdatePermissionHighlightsStateAction.LocalDeviceAccessChangedAction -> {
+                updatePermissionHighlightsState(state, action.tabId) {
+                    it.copy(localDeviceAccessChanged = action.value)
+                }
+            }
+            is UpdatePermissionHighlightsStateAction.LocalNetworkAccessChangedAction -> {
+                updatePermissionHighlightsState(state, action.tabId) {
+                    it.copy(localNetworkAccessChanged = action.value)
+                }
+            }
             is UpdatePermissionHighlightsStateAction.AutoPlayAudibleBlockingAction -> {
                 updatePermissionHighlightsState(state, action.tabId) {
                     it.copy(autoPlayAudibleBlocking = action.value)
@@ -312,17 +325,19 @@ internal object ContentStateReducer {
             -> {
                 throw IllegalStateException("You need to add SessionPrioritizationMiddleware. ($action)")
             }
-            is ContentAction.UpdateProductUrlStateAction -> {
+            is ContentAction.EnteredPdfViewer -> {
                 updateContentState(state, action.tabId) {
-                    if (it.private) {
-                        it
-                    } else {
-                        it.copy(isProductUrl = action.isProductUrl)
-                    }
+                    it.copy(isPdf = true)
+                }
+            }
+            is ContentAction.ExitedPdfViewer -> {
+                updateContentState(state, action.tabId) {
+                    it.copy(isPdf = false)
                 }
             }
         }
     }
+
     private fun consumeDownload(state: BrowserState, sessionId: String, downloadId: String): BrowserState {
         return updateContentState(state, sessionId) {
             if (it.download != null && it.download.id == downloadId) {
@@ -344,7 +359,7 @@ private inline fun updatePermissionHighlightsState(
     }
 }
 
-private inline fun updateContentState(
+internal inline fun updateContentState(
     state: BrowserState,
     tabId: String,
     crossinline update: (ContentState) -> ContentState,
@@ -355,15 +370,15 @@ private inline fun updateContentState(
 }
 
 private fun isHostEquals(sessionUrl: String, newUrl: String): Boolean {
-    val sessionUri = Uri.parse(sessionUrl)
-    val newUri = Uri.parse(newUrl)
+    val sessionUri = sessionUrl.toUri()
+    val newUri = newUrl.toUri()
 
     return sessionUri.sameSchemeAndHostAs(newUri)
 }
 
 private fun isUrlSame(originalUrl: String, newUrl: String): Boolean {
-    val originalUri = Uri.parse(originalUrl)
-    val uri = Uri.parse(newUrl)
+    val originalUri = originalUrl.toUri()
+    val uri = newUrl.toUri()
 
     return uri.port == originalUri.port &&
         uri.host == originalUri.host &&
@@ -378,8 +393,8 @@ private fun isUrlSame(originalUrl: String, newUrl: String): Boolean {
  */
 private fun isInScope(manifest: WebAppManifest?, newUrl: String): Boolean {
     val scope = manifest?.scope ?: manifest?.startUrl ?: return false
-    val scopeUri = Uri.parse(scope)
-    val newUri = Uri.parse(newUrl)
+    val scopeUri = scope.toUri()
+    val newUri = newUrl.toUri()
 
     return newUri.isInScope(listOf(scopeUri))
 }
