@@ -601,7 +601,7 @@ class PlacesHistoryStorageTest {
     }
 
     @Test
-    @Config(sdk = [Build.VERSION_CODES.M])
+    @Config(sdk = [Build.VERSION_CODES.O])
     fun `When periodicStorageWorkRequest is called, worker with input specs is created`() {
         val request = history.periodicStorageWorkRequest<PlacesHistoryStorageWorker>(
             tag = PlacesHistoryStorageWorker.UNIQUE_NAME,
@@ -619,7 +619,7 @@ class PlacesHistoryStorageTest {
     }
 
     @Test
-    @Config(sdk = [Build.VERSION_CODES.M])
+    @Config(sdk = [Build.VERSION_CODES.O])
     fun `When storage maintenance work request is registered, the worker is enqueued`() {
         val config = Configuration.Builder().build()
         WorkManagerTestInitHelper.initializeTestWorkManager(testContext, config)
@@ -692,10 +692,10 @@ class PlacesHistoryStorageTest {
         storage.sync(SyncAuthInfo("kid", "token", 123L, "key", "serverUrl"))
 
         assertEquals("kid", passedAuthInfo!!.kid)
-        assertEquals("serverUrl", passedAuthInfo!!.tokenServerUrl)
-        assertEquals("token", passedAuthInfo!!.fxaAccessToken)
-        assertEquals(123L, passedAuthInfo!!.fxaAccessTokenExpiresAt)
-        assertEquals("key", passedAuthInfo!!.syncKey)
+        assertEquals("serverUrl", passedAuthInfo.tokenServerUrl)
+        assertEquals("token", passedAuthInfo.fxaAccessToken)
+        assertEquals(123L, passedAuthInfo.fxaAccessTokenExpiresAt)
+        assertEquals("key", passedAuthInfo.syncKey)
     }
 
     @Test
