@@ -27,7 +27,7 @@ import mozilla.components.browser.toolbar.internal.ActionContainer
 import mozilla.components.concept.menu.MenuController
 import mozilla.components.concept.toolbar.Toolbar
 import mozilla.components.support.ktx.android.content.isScreenReaderEnabled
-import mozilla.components.ui.colors.R.color as photonColors
+import mozilla.components.ui.colors.R as colorsR
 
 /**
  * Sub-component of the browser toolbar responsible for displaying the URL and related controls ("display mode").
@@ -86,8 +86,9 @@ class DisplayToolbar internal constructor(
     /**
      * Data class holding the customizable colors in "display mode".
      *
-     * @property securityIconSecure Color tint for the "secure connection" icon (lock).
-     * @property securityIconInsecure Color tint for the "insecure connection" icon (broken lock).
+     * @property siteInfoIconSecure Color tint for the "secure connection" icon (lock).
+     * @property siteInfoIconInsecure Color tint for the "insecure connection" icon (broken lock).
+     * @property siteInfoIconLocalPdf Color tint for the "local pdf" icon (vertical page).
      * @property emptyIcon Color tint for the icon shown when the URL is empty.
      * @property menu Color tint for the menu icon.
      * @property hint Text color of the hint shown when the URL is empty.
@@ -95,24 +96,23 @@ class DisplayToolbar internal constructor(
      * @property text Text color of the URL.
      * @property trackingProtection Color tint for the tracking protection icons.
      * @property separator Color tint for the separator shown between indicators.
-     * @property pageActionSeparator Color tint of separator dividing url and page actions.
      * @property highlight Color tint for the highlight icon.
      *
      * Set/Get the site security icon colours. It uses a pair of color integers which represent the
      * insecure and secure colours respectively.
      */
     data class Colors(
-        @ColorInt val securityIconSecure: Int,
-        @ColorInt val securityIconInsecure: Int,
-        @ColorInt val emptyIcon: Int,
-        @ColorInt val menu: Int,
-        @ColorInt val hint: Int,
-        @ColorInt val title: Int,
-        @ColorInt val text: Int,
-        @ColorInt val trackingProtection: Int?,
-        @ColorInt val separator: Int,
-        @ColorInt val pageActionSeparator: Int,
-        @ColorInt val highlight: Int?,
+        @param:ColorInt val siteInfoIconSecure: Int,
+        @param:ColorInt val siteInfoIconInsecure: Int,
+        @param:ColorInt val siteInfoIconLocalPdf: Int,
+        @param:ColorInt val emptyIcon: Int,
+        @param:ColorInt val menu: Int,
+        @param:ColorInt val hint: Int,
+        @param:ColorInt val title: Int,
+        @param:ColorInt val text: Int,
+        @param:ColorInt val trackingProtection: Int?,
+        @param:ColorInt val separator: Int,
+        @param:ColorInt val highlight: Int?,
     )
 
     /**
@@ -144,16 +144,26 @@ class DisplayToolbar internal constructor(
         BOTTOM,
     }
 
+    /**
+     * Data class holding the customizable margins for views in "display mode".
+     *
+     * @property goneStartMargin The start margin to be applied when the constraint target is gone.
+     * @property goneEndMargin The end margin to be applied when the constraint target is gone.
+     */
+    data class DisplayMargins(
+        val goneStartMargin: Int?,
+        val goneEndMargin: Int?,
+    )
+
     internal val views = DisplayToolbarViews(
         browserActions = rootView.findViewById(R.id.mozac_browser_toolbar_browser_actions),
         pageActions = rootView.findViewById(R.id.mozac_browser_toolbar_page_actions),
         navigationActions = rootView.findViewById(R.id.mozac_browser_toolbar_navigation_actions),
         background = rootView.findViewById(R.id.mozac_browser_toolbar_background),
         separator = rootView.findViewById(R.id.mozac_browser_toolbar_separator),
-        pageActionSeparator = rootView.findViewById(R.id.mozac_browser_toolbar_action_separator),
         emptyIndicator = rootView.findViewById(R.id.mozac_browser_toolbar_empty_indicator),
         menu = MenuButton(rootView.findViewById(R.id.mozac_browser_toolbar_menu)),
-        securityIndicator = rootView.findViewById(R.id.mozac_browser_toolbar_security_indicator),
+        siteInfoIndicator = rootView.findViewById(R.id.mozac_browser_toolbar_site_info_indicator),
         trackingProtectionIndicator = rootView.findViewById(
             R.id.mozac_browser_toolbar_tracking_protection_indicator,
         ),
@@ -168,29 +178,28 @@ class DisplayToolbar internal constructor(
      * Customizable colors in "display mode".
      */
     var colors: Colors = Colors(
-        securityIconSecure = ContextCompat.getColor(context, photonColors.photonWhite),
-        securityIconInsecure = ContextCompat.getColor(context, photonColors.photonWhite),
-        emptyIcon = ContextCompat.getColor(context, photonColors.photonWhite),
-        menu = ContextCompat.getColor(context, photonColors.photonWhite),
+        siteInfoIconSecure = ContextCompat.getColor(context, colorsR.color.photonWhite),
+        siteInfoIconInsecure = ContextCompat.getColor(context, colorsR.color.photonWhite),
+        siteInfoIconLocalPdf = ContextCompat.getColor(context, colorsR.color.photonWhite),
+        emptyIcon = ContextCompat.getColor(context, colorsR.color.photonWhite),
+        menu = ContextCompat.getColor(context, colorsR.color.photonWhite),
         hint = views.origin.hintColor,
         title = views.origin.titleColor,
         text = views.origin.textColor,
         trackingProtection = null,
-        separator = ContextCompat.getColor(context, photonColors.photonGrey80),
-        pageActionSeparator = ContextCompat.getColor(context, photonColors.photonGrey80),
+        separator = ContextCompat.getColor(context, colorsR.color.photonGrey80),
         highlight = null,
     )
         set(value) {
             field = value
 
-            updateSiteSecurityIcon()
+            updateSiteInfoIcon()
             views.emptyIndicator.setColorFilter(value.emptyIcon)
             views.menu.setColorFilter(value.menu)
             views.origin.hintColor = value.hint
             views.origin.titleColor = value.title
             views.origin.textColor = value.text
             views.separator.setColorFilter(value.separator)
-            views.pageActionSeparator.setBackgroundColor(value.pageActionSeparator)
 
             if (value.trackingProtection != null) {
                 views.trackingProtectionIndicator.setTint(value.trackingProtection)
@@ -239,14 +248,14 @@ class DisplayToolbar internal constructor(
     var urlFormatter: ((CharSequence) -> CharSequence)? = null
 
     /**
-     * Sets a listener to be invoked when the site security indicator icon is clicked.
+     * Sets a listener to be invoked when the site info indicator icon is clicked.
      */
-    fun setOnSiteSecurityClickedListener(listener: (() -> Unit)?) {
+    fun setOnSiteInfoClickedListener(listener: (() -> Unit)?) {
         if (listener == null) {
-            views.securityIndicator.setOnClickListener(null)
-            views.securityIndicator.background = null
+            views.siteInfoIndicator.setOnClickListener(null)
+            views.siteInfoIndicator.background = null
         } else {
-            views.securityIndicator.setOnClickListener {
+            views.siteInfoIndicator.setOnClickListener {
                 listener.invoke()
             }
 
@@ -257,7 +266,7 @@ class DisplayToolbar internal constructor(
                 true,
             )
 
-            views.securityIndicator.setBackgroundResource(outValue.resourceId)
+            views.siteInfoIndicator.setBackgroundResource(outValue.resourceId)
         }
     }
 
@@ -307,6 +316,24 @@ class DisplayToolbar internal constructor(
             updateIndicatorVisibility()
         }
 
+    /**
+     * Adds the security indicator if the list does not contain it.
+     */
+    fun addSecurityIndicator() {
+        if (!indicators.contains(Indicators.SECURITY)) {
+            indicators += Indicators.SECURITY
+        }
+    }
+
+    /**
+     * Removes the security indicator if the list contains it.
+     */
+    fun removeSecurityIndicator() {
+        if (indicators.contains(Indicators.SECURITY)) {
+            indicators -= Indicators.SECURITY
+        }
+    }
+
     /**
      * Sets the background that should be drawn behind the URL, page actions an indicators.
      */
@@ -314,6 +341,24 @@ class DisplayToolbar internal constructor(
         views.background.setImageDrawable(background)
     }
 
+    /**
+     * Sets the margins for the background view using the provided DisplayMargins.
+     *
+     * @param margins The DisplayMargins containing the start and end margins to be applied.
+     */
+    fun setUrlBackgroundMargins(margins: DisplayMargins) {
+        val layoutParams = views.background.layoutParams as? ConstraintLayout.LayoutParams
+        layoutParams?.let {
+            margins.goneStartMargin?.let { goneStartMargin ->
+                it.goneStartMargin = goneStartMargin
+            }
+
+            margins.goneEndMargin?.let { goneEndMargin ->
+                it.goneEndMargin = goneEndMargin
+            }
+        }
+    }
+
     /**
      * Whether the progress bar should be drawn at the top or bottom of the toolbar.
      */
@@ -413,7 +458,7 @@ class DisplayToolbar internal constructor(
     private fun updateIndicatorVisibility() {
         val urlEmpty = url.isEmpty()
 
-        views.securityIndicator.visibility = if (!urlEmpty && indicators.contains(Indicators.SECURITY)) {
+        views.siteInfoIndicator.visibility = if (!urlEmpty && indicators.contains(Indicators.SECURITY)) {
             View.VISIBLE
         } else {
             View.GONE
@@ -446,7 +491,7 @@ class DisplayToolbar internal constructor(
         views.separator.visibility = if (
             displayIndicatorSeparator &&
             views.trackingProtectionIndicator.isVisible &&
-            views.securityIndicator.isVisible
+            views.siteInfoIndicator.isVisible
         ) {
             View.VISIBLE
         } else {
@@ -481,26 +526,28 @@ class DisplayToolbar internal constructor(
         }
 
     /**
-     * Sets the site's security icon as secure if true, else the regular globe.
+     * Sets the site's info icon as a local PDF if true. If web content is being displayed,
+     * sets the site's info as secure if true, or insecure otherwise.
      */
-    internal var siteSecurity: Toolbar.SiteSecurity = Toolbar.SiteSecurity.INSECURE
+    internal var siteInfo: Toolbar.SiteInfo = Toolbar.SiteInfo.INSECURE
         set(value) {
             field = value
-            updateSiteSecurityIcon()
+            updateSiteInfoIcon()
         }
 
-    private fun updateSiteSecurityIcon() {
-        @ColorInt val color = when (siteSecurity) {
-            Toolbar.SiteSecurity.INSECURE -> colors.securityIconInsecure
-            Toolbar.SiteSecurity.SECURE -> colors.securityIconSecure
+    private fun updateSiteInfoIcon() {
+        @ColorInt val color = when (siteInfo) {
+            Toolbar.SiteInfo.INSECURE -> colors.siteInfoIconInsecure
+            Toolbar.SiteInfo.SECURE -> colors.siteInfoIconSecure
+            Toolbar.SiteInfo.LOCAL_PDF -> colors.siteInfoIconLocalPdf
         }
-        if (color == Color.TRANSPARENT && Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
-            views.securityIndicator.clearColorFilter()
+        if (color == Color.TRANSPARENT) {
+            views.siteInfoIndicator.clearColorFilter()
         } else {
-            views.securityIndicator.setColorFilter(color)
+            views.siteInfoIndicator.setColorFilter(color)
         }
 
-        views.securityIndicator.siteSecurity = siteSecurity
+        views.siteInfoIndicator.siteInfo = siteInfo
     }
 
     internal fun setTrackingProtectionState(state: Toolbar.SiteTrackingProtection) {
@@ -553,6 +600,7 @@ class DisplayToolbar internal constructor(
 
             // Announce "loading" for accessibility if it has not been completed
             if (progress < views.progress.max) {
+                @Suppress("DEPRECATION")
                 views.progress.announceForAccessibility(
                     context.getString(R.string.mozac_browser_toolbar_progress_loading),
                 )
@@ -672,21 +720,12 @@ class DisplayToolbar internal constructor(
      * Sets the horizontal padding.
      */
     fun setHorizontalPadding(horizontalPadding: Int) {
-        rootView.setPadding(horizontalPadding, 0, horizontalPadding, 0)
-    }
-
-    /**
-     * Hides the page action separator in display mode.
-     */
-    fun hidePageActionSeparator() {
-        views.pageActionSeparator.isVisible = false
-    }
-
-    /**
-     * Shows the page action separator in display mode.
-     */
-    internal fun showPageActionSeparator() {
-        views.pageActionSeparator.isVisible = true
+        val background = views.background
+        (background.layoutParams as? ConstraintLayout.LayoutParams)?.apply {
+            goneStartMargin = horizontalPadding
+            goneEndMargin = horizontalPadding
+            background.layoutParams = this
+        }
     }
 }
 
@@ -700,10 +739,9 @@ internal class DisplayToolbarViews(
     val navigationActions: ActionContainer,
     val background: ImageView,
     val separator: ImageView,
-    val pageActionSeparator: View,
     val emptyIndicator: ImageView,
     val menu: MenuButton,
-    val securityIndicator: SiteSecurityIconView,
+    val siteInfoIndicator: SiteInfoIconView,
     val trackingProtectionIndicator: TrackingProtectionIconView,
     val origin: OriginView,
     val progress: ProgressBar,
