@@ -19,7 +19,6 @@ import androidx.compose.foundation.layout.width
 import androidx.compose.foundation.text.BasicTextField
 import androidx.compose.foundation.text.KeyboardActions
 import androidx.compose.foundation.text.KeyboardOptions
-import androidx.compose.material3.ExperimentalMaterial3Api
 import androidx.compose.material3.Icon
 import androidx.compose.material3.Text
 import androidx.compose.material3.TextFieldDefaults
@@ -49,7 +48,7 @@ import androidx.compose.ui.unit.Dp
 import androidx.compose.ui.unit.dp
 import mozilla.components.compose.base.modifier.thenConditional
 import mozilla.components.compose.base.theme.AcornTheme
-import mozilla.components.ui.icons.R
+import mozilla.components.ui.icons.R as iconsR
 
 private val FocusedIndicatorLineThickness = 2.dp
 private val UnfocusedIndicatorLineThickness = 1.dp
@@ -86,7 +85,6 @@ private val TrailingIconHeight = 24.dp
  * called. Note that this IME action may be different from what you specified in
  * [KeyboardOptions.imeAction].
  */
-@OptIn(ExperimentalMaterial3Api::class)
 @Suppress("LongMethod")
 @Composable
 fun TextField(
@@ -204,7 +202,7 @@ fun TextField(
                             Spacer(modifier = Modifier.width(12.dp))
 
                             Icon(
-                                painter = painterResource(R.drawable.mozac_ic_warning_fill_24),
+                                painter = painterResource(iconsR.drawable.mozac_ic_warning_fill_24),
                                 contentDescription = null,
                                 tint = colors.errorTrailingIconColor,
                             )
