@@ -4,6 +4,8 @@
 
 package mozilla.components.feature.sitepermissions
 
+import android.Manifest.permission.ACCESS_COARSE_LOCATION
+import android.Manifest.permission.ACCESS_FINE_LOCATION
 import android.content.pm.PackageManager.PERMISSION_DENIED
 import android.content.pm.PackageManager.PERMISSION_GRANTED
 import androidx.fragment.app.FragmentManager
@@ -16,6 +18,8 @@ import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHig
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.AutoPlayInAudibleBlockingAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.AutoPlayInAudibleChangedAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.CameraChangedAction
+import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.LocalDeviceAccessChangedAction
+import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.LocalNetworkAccessChangedAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.LocationChangedAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.MediaKeySystemAccesChangedAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.MicrophoneChangedAction
@@ -35,6 +39,8 @@ import mozilla.components.concept.engine.permission.Permission.ContentAutoPlayAu
 import mozilla.components.concept.engine.permission.Permission.ContentAutoPlayInaudible
 import mozilla.components.concept.engine.permission.Permission.ContentCrossOriginStorageAccess
 import mozilla.components.concept.engine.permission.Permission.ContentGeoLocation
+import mozilla.components.concept.engine.permission.Permission.ContentLocalDeviceAccess
+import mozilla.components.concept.engine.permission.Permission.ContentLocalNetworkAccess
 import mozilla.components.concept.engine.permission.Permission.ContentMediaKeySystemAccess
 import mozilla.components.concept.engine.permission.Permission.ContentNotification
 import mozilla.components.concept.engine.permission.Permission.ContentPersistentStorage
@@ -53,7 +59,6 @@ import mozilla.components.support.base.Component.FEATURE_SITEPERMISSIONS
 import mozilla.components.support.base.facts.Action
 import mozilla.components.support.base.facts.processor.CollectionProcessor
 import mozilla.components.support.base.feature.OnNeedToRequestPermissions
-import mozilla.components.support.ktx.kotlin.stripDefaultPort
 import mozilla.components.support.test.any
 import mozilla.components.support.test.eq
 import mozilla.components.support.test.ext.joinBlocking
@@ -80,6 +85,7 @@ import org.mockito.Mockito.never
 import org.mockito.Mockito.spy
 import org.mockito.Mockito.times
 import org.mockito.Mockito.verify
+import java.net.URL
 import java.security.InvalidParameterException
 import java.util.UUID
 import mozilla.components.ui.icons.R as iconsR
@@ -137,6 +143,29 @@ class SitePermissionsFeatureTest {
         )
     }
 
+    @Test
+    fun `GIVEN an url WHEN calling trimOriginHttpsSchemeAndPort THEN default https scheme and default port are trimmed`() {
+        assertEquals(
+            sitePermissionFeature.trimOriginHttpsSchemeAndPort("https://https-scheme-https-port.test:443"),
+            "https-scheme-https-port.test",
+        )
+
+        assertEquals(
+            sitePermissionFeature.trimOriginHttpsSchemeAndPort("https://https-scheme-random-port.test:8080"),
+            "https-scheme-random-port.test:8080",
+        )
+
+        assertEquals(
+            sitePermissionFeature.trimOriginHttpsSchemeAndPort("https://https-scheme-nonhttps-port.test:80"),
+            "https-scheme-nonhttps-port.test:80",
+        )
+
+        assertEquals(
+            sitePermissionFeature.trimOriginHttpsSchemeAndPort("http://http-scheme-http-port.test:80"),
+            "http://http-scheme-http-port.test",
+        )
+    }
+
     @Test
     fun `GIVEN a tab load THEN stale permission indicators should be clear up and temporary permissions`() {
         sitePermissionFeature.start()
@@ -234,6 +263,74 @@ class SitePermissionsFeatureTest {
         verify(sitePermissionFeature).consumeAppPermissionRequest(mockAppPermissionRequest)
     }
 
+    @Test
+    fun `GIVEN an appPermissionRequest for both fine and coarse location WHEN either permission is granted THEN permission request is granted`() {
+        // given
+        doReturn(mockAppPermissionRequest).`when`(sitePermissionFeature)
+            .findRequestedAppPermission(any())
+
+        // when
+        sitePermissionFeature.onPermissionsResult(
+            arrayOf(ACCESS_FINE_LOCATION, ACCESS_COARSE_LOCATION),
+            arrayOf(PERMISSION_DENIED, PERMISSION_GRANTED).toIntArray(),
+        )
+
+        // then
+        verify(mockAppPermissionRequest).grant()
+        verify(sitePermissionFeature).consumeAppPermissionRequest(mockAppPermissionRequest)
+    }
+
+    @Test
+    fun `GIVEN an appPermissionRequest for fine and coarse location and another permission WHEN either location permission is granted and the third is denied THEN permission request is not granted`() {
+        // given
+        doReturn(mockAppPermissionRequest).`when`(sitePermissionFeature)
+            .findRequestedAppPermission(any())
+
+        // when
+        sitePermissionFeature.onPermissionsResult(
+            arrayOf(ACCESS_FINE_LOCATION, ACCESS_COARSE_LOCATION, "random permission"),
+            arrayOf(PERMISSION_DENIED, PERMISSION_GRANTED, PERMISSION_DENIED).toIntArray(),
+        )
+
+        // then
+        verify(mockAppPermissionRequest).reject()
+        verify(sitePermissionFeature).consumeAppPermissionRequest(mockAppPermissionRequest)
+    }
+
+    @Test
+    fun `GIVEN an appPermissionRequest for fine and coarse location and another permission WHEN either location permission is granted and the third is granted THEN permission request is granted`() {
+        // given
+        doReturn(mockAppPermissionRequest).`when`(sitePermissionFeature)
+            .findRequestedAppPermission(any())
+
+        // when
+        sitePermissionFeature.onPermissionsResult(
+            arrayOf(ACCESS_FINE_LOCATION, ACCESS_COARSE_LOCATION, "random permission"),
+            arrayOf(PERMISSION_DENIED, PERMISSION_GRANTED, PERMISSION_GRANTED).toIntArray(),
+        )
+
+        // then
+        verify(mockAppPermissionRequest).grant()
+        verify(sitePermissionFeature).consumeAppPermissionRequest(mockAppPermissionRequest)
+    }
+
+    @Test
+    fun `GIVEN an appPermissionRequest for two other permissions WHEN either permission is granted THEN permission request is not granted`() {
+        // given
+        doReturn(mockAppPermissionRequest).`when`(sitePermissionFeature)
+            .findRequestedAppPermission(any())
+
+        // when
+        sitePermissionFeature.onPermissionsResult(
+            arrayOf("random permission 1", "random permission 2"),
+            arrayOf(PERMISSION_DENIED, PERMISSION_GRANTED).toIntArray(),
+        )
+
+        // then
+        verify(mockAppPermissionRequest).reject()
+        verify(sitePermissionFeature).consumeAppPermissionRequest(mockAppPermissionRequest)
+    }
+
     @Test
     fun `GIVEN an appPermissionRequest with blocked permissions and !onShouldShowRequestPermissionRationale WHEN onPermissionsResult() THEN reject(), storeSitePermissions, consume are called`() {
         // given
@@ -258,6 +355,7 @@ class SitePermissionsFeatureTest {
     @Test
     fun `GIVEN shouldStore true WHEN onContentPermissionGranted() THEN storeSitePermissions() called`() {
         // given
+        doReturn(listOf(ContentNotification())).`when`(mockPermissionRequest).permissions
         doNothing().`when`(sitePermissionFeature)
             .storeSitePermissions(any(), any(), any(), any())
 
@@ -286,6 +384,7 @@ class SitePermissionsFeatureTest {
     @Test
     fun `GIVEN permissionRequest WHEN onPositiveButtonPress() THEN consumePermissionRequest, onContentPermissionGranted are called`() {
         // given
+        doReturn(listOf(ContentNotification())).`when`(mockPermissionRequest).permissions
         doNothing().`when`(sitePermissionFeature).consumePermissionRequest(any(), any())
         doNothing().`when`(sitePermissionFeature)
             .onContentPermissionGranted(mockPermissionRequest, true)
@@ -299,8 +398,11 @@ class SitePermissionsFeatureTest {
         // then
         verify(sitePermissionFeature)
             .consumePermissionRequest(mockPermissionRequest, SESSION_ID)
-        verify(sitePermissionFeature)
-            .onContentPermissionGranted(mockPermissionRequest, true)
+        verify(sitePermissionFeature).onContentPermissionGranted(
+            eq(mockPermissionRequest),
+            eq(true),
+            any(),
+        )
     }
 
     @Test
@@ -441,7 +543,11 @@ class SitePermissionsFeatureTest {
         doReturn(mock<SelectOrAddUseCase>()).`when`(sitePermissionFeature).selectOrAddUseCase
 
         // when
-        sitePermissionFeature.onLearnMorePress(PERMISSION_ID, SESSION_ID)
+        sitePermissionFeature.onLearnMorePress(
+            permissionId = PERMISSION_ID,
+            sessionId = SESSION_ID,
+            learnMoreLink = "https://mozilla.org",
+        )
 
         // then
         verify(sitePermissionFeature)
@@ -449,12 +555,47 @@ class SitePermissionsFeatureTest {
         verify(sitePermissionFeature)
             .onContentPermissionDeny(permissionRequest, false)
         verify(sitePermissionFeature.selectOrAddUseCase).invoke(
-            url = STORAGE_ACCESS_DOCUMENTATION_URL,
+            url = "https://mozilla.org",
             private = false,
             source = SessionState.Source.Internal.TextSelection,
         )
     }
 
+    @Test
+    fun `GIVEN permissionRequest WHEN onLearnMorePress() THEN SelectOrAddUseCase is not called if learn more link is empty`() {
+        // given
+        val permission: ContentCrossOriginStorageAccess = mock()
+        val permissionRequest: PermissionRequest = mock {
+            whenever(permissions).thenReturn(listOf(permission))
+        }
+        doNothing().`when`(sitePermissionFeature).consumePermissionRequest(any(), any())
+        doNothing().`when`(sitePermissionFeature)
+            .onContentPermissionDeny(mockPermissionRequest, true)
+        doReturn(permissionRequest).`when`(sitePermissionFeature).findRequestedPermission(
+            anyString(),
+        )
+        doReturn(mock<SelectOrAddUseCase>()).`when`(sitePermissionFeature).selectOrAddUseCase
+
+        // given the learn more link provider returns an empty string
+        sitePermissionFeature.learnMoreUrlProvider = FakeSitePermissionsLearnMoreUrlProvider(expectedLink = "")
+
+        // when
+        sitePermissionFeature.onLearnMorePress(
+            permissionId = PERMISSION_ID,
+            sessionId = SESSION_ID,
+            learnMoreLink = "",
+        )
+
+        // then verify that SelectOrAddUseCase is never called
+        verify(sitePermissionFeature.selectOrAddUseCase, never()).invoke(
+            url = anyString(),
+            private = anyBoolean(),
+            source = any(),
+            flags = any(),
+            ignoreFragment = anyBoolean(),
+        )
+    }
+
     @Test
     fun `GIVEN a new permissionRequest WHEN storeSitePermissions() THEN save(permissionRequest) is called`() = runTestOnMain {
         // given
@@ -821,6 +962,46 @@ class SitePermissionsFeatureTest {
         verify(mockStore).dispatch(NotificationChangedAction(tab1.id, true))
     }
 
+    @Test
+    fun `GIVEN local device access request WHEN calling updatePermissionToolbarIndicator THEN dispatch `() {
+        val tab1 = createTab("https://www.mozilla.org", id = "1")
+        val request: PermissionRequest = mock {
+            whenever(permissions).thenReturn(listOf(ContentLocalDeviceAccess(id = "permission")))
+        }
+
+        doReturn(tab1).`when`(sitePermissionFeature).getCurrentTabState()
+        doReturn(SitePermissionsRules.Action.BLOCKED).`when`(mockSitePermissionRules).localDeviceAccess
+
+        sitePermissionFeature.updatePermissionToolbarIndicator(request, BLOCKED, false)
+        verify(mockStore, never()).dispatch(any<LocalDeviceAccessChangedAction>())
+
+        sitePermissionFeature.updatePermissionToolbarIndicator(request, BLOCKED, true)
+        verify(mockStore).dispatch(LocalDeviceAccessChangedAction(tab1.id, false))
+
+        sitePermissionFeature.updatePermissionToolbarIndicator(request, ALLOWED, true)
+        verify(mockStore).dispatch(LocalDeviceAccessChangedAction(tab1.id, true))
+    }
+
+    @Test
+    fun `GIVEN local network access request WHEN calling updatePermissionToolbarIndicator THEN dispatch `() {
+        val tab1 = createTab("https://www.mozilla.org", id = "1")
+        val request: PermissionRequest = mock {
+            whenever(permissions).thenReturn(listOf(ContentLocalNetworkAccess(id = "permission")))
+        }
+
+        doReturn(tab1).`when`(sitePermissionFeature).getCurrentTabState()
+        doReturn(SitePermissionsRules.Action.BLOCKED).`when`(mockSitePermissionRules).localNetworkAccess
+
+        sitePermissionFeature.updatePermissionToolbarIndicator(request, BLOCKED, false)
+        verify(mockStore, never()).dispatch(any<LocalNetworkAccessChangedAction>())
+
+        sitePermissionFeature.updatePermissionToolbarIndicator(request, BLOCKED, true)
+        verify(mockStore).dispatch(LocalNetworkAccessChangedAction(tab1.id, false))
+
+        sitePermissionFeature.updatePermissionToolbarIndicator(request, ALLOWED, true)
+        verify(mockStore).dispatch(LocalNetworkAccessChangedAction(tab1.id, true))
+    }
+
     @Test
     fun `GIVEN camera request WHEN calling updatePermissionToolbarIndicator THEN dispatch CameraChangedAction`() {
         val tab1 = createTab("https://www.mozilla.org", id = "1")
@@ -1064,30 +1245,35 @@ class SitePermissionsFeatureTest {
     @Test
     fun `GIVEN a ContentStorageAccess request WHEN handlingSingleContentPermissions is called THEN create a specific prompt`() {
         // given
-        val host = "https://mozilla.org"
+        val origin = "https://mozilla.org"
         val permission = ContentCrossOriginStorageAccess(id = "permission")
         val permissionRequest: PermissionRequest = mock {
             whenever(permissions).thenReturn(listOf(permission))
             whenever(id).thenReturn("id")
         }
+        sitePermissionFeature.learnMoreUrlProvider = FakeSitePermissionsLearnMoreUrlProvider(
+            expectedLink = "https://content-storage-url.com",
+        )
 
         // when
-        sitePermissionFeature.handlingSingleContentPermissions(permissionRequest, permission, host)
+        sitePermissionFeature.handlingSingleContentPermissions(permissionRequest, permission, origin)
 
         // then
         verify(sitePermissionFeature).createContentCrossOriginStorageAccessPermissionPrompt(
             context = testContext,
-            host,
+            origin,
             permissionRequest,
-            false,
-            true,
+            showDoNotAskAgainCheckBox = false,
+            shouldSelectRememberChoice = true,
+            learnMoreLink = "https://content-storage-url.com",
         )
     }
 
     @Test
     fun `GIVEN a ContentStorageAccess request WHEN createContentStorageAccessPermissionPrompt is called THEN create a specific SitePermissionsDialogFragment`() {
         // given
-        val host = "https://mozilla.org"
+        val origin = "https://mozilla.org:443"
+        val host = "mozilla.org"
         val permission = ContentCrossOriginStorageAccess(id = "permission")
         val permissionRequest: PermissionRequest = mock {
             whenever(permissions).thenReturn(listOf(permission))
@@ -1095,12 +1281,16 @@ class SitePermissionsFeatureTest {
         }
 
         // when
+        sitePermissionFeature.learnMoreUrlProvider = FakeSitePermissionsLearnMoreUrlProvider(
+            expectedLink = "https://content-storage-url.com",
+        )
         val dialog = sitePermissionFeature.createContentCrossOriginStorageAccessPermissionPrompt(
             testContext,
-            host,
+            origin,
             permissionRequest,
-            false,
-            true,
+            showDoNotAskAgainCheckBox = false,
+            shouldSelectRememberChoice = true,
+            learnMoreLink = "https://content-storage-url.com",
         )
 
         // then
@@ -1108,8 +1298,8 @@ class SitePermissionsFeatureTest {
         assertEquals(
             testContext.getString(
                 R.string.mozac_feature_sitepermissions_storage_access_title,
-                host.stripDefaultPort(),
-                selectedTab.content.url.stripDefaultPort(),
+                host,
+                URL(selectedTab.content.url).host,
             ),
             dialog.title,
         )
@@ -1122,7 +1312,7 @@ class SitePermissionsFeatureTest {
         assertEquals(
             testContext.getString(
                 R.string.mozac_feature_sitepermissions_storage_access_message,
-                host.stripDefaultPort(),
+                host,
             ),
             dialog.message,
         )
@@ -1130,14 +1320,14 @@ class SitePermissionsFeatureTest {
             testContext.getString(R.string.mozac_feature_sitepermissions_storage_access_not_allow),
             dialog.negativeButtonText,
         )
-        assertEquals(true, dialog.shouldShowLearnMoreLink)
+        assertEquals("https://content-storage-url.com", dialog.learnMoreLink)
     }
 
     @Test
     fun `GIVEN permissionRequest and containsVideoAndAudioSources true WHEN createPrompt THEN createSinglePermissionPrompt is called`() {
         // given
         val permissionRequest: PermissionRequest = object : PermissionRequest {
-            override val uri: String?
+            override val uri: String
                 get() = "http://www.mozilla.org"
             override val id: String
                 get() = PERMISSION_ID
@@ -1155,18 +1345,23 @@ class SitePermissionsFeatureTest {
             override fun containsVideoAndAudioSources() = true
 
             override fun reject() = Unit
+
+            override fun merge(permissionRequest: PermissionRequest) = Unit
         }
         val sitePermissionsDialogFragment = SitePermissionsDialogFragment()
         doReturn(sitePermissionsDialogFragment).`when`(sitePermissionFeature)
             .createSinglePermissionPrompt(
-                any(),
-                ArgumentMatchers.anyString(),
-                any(),
-                ArgumentMatchers.anyInt(),
-                ArgumentMatchers.anyInt(),
-                ArgumentMatchers.anyBoolean(),
-                ArgumentMatchers.anyBoolean(),
-                ArgumentMatchers.anyBoolean(),
+                context = any(),
+                origin = ArgumentMatchers.anyString(),
+                permissionRequest = any(),
+                titleId = ArgumentMatchers.anyInt(),
+                iconId = ArgumentMatchers.anyInt(),
+                showDoNotAskAgainCheckBox = ArgumentMatchers.anyBoolean(),
+                doNotAskAgainCheckBoxLabel = ArgumentMatchers.anyInt(),
+                shouldSelectRememberChoice = ArgumentMatchers.anyBoolean(),
+                isNotificationRequest = ArgumentMatchers.anyBoolean(),
+                negativeButtonResId = ArgumentMatchers.anyInt(),
+                learnMoreLink = anyString(),
             )
 
         // when
@@ -1174,14 +1369,17 @@ class SitePermissionsFeatureTest {
 
         // then
         verify(sitePermissionFeature).createSinglePermissionPrompt(
-            any(),
-            ArgumentMatchers.anyString(),
-            any(),
-            ArgumentMatchers.anyInt(),
-            ArgumentMatchers.anyInt(),
-            ArgumentMatchers.anyBoolean(),
-            ArgumentMatchers.anyBoolean(),
-            ArgumentMatchers.anyBoolean(),
+            context = any(),
+            origin = ArgumentMatchers.anyString(),
+            permissionRequest = any(),
+            titleId = ArgumentMatchers.anyInt(),
+            iconId = ArgumentMatchers.anyInt(),
+            showDoNotAskAgainCheckBox = ArgumentMatchers.anyBoolean(),
+            doNotAskAgainCheckBoxLabel = ArgumentMatchers.isNull(),
+            shouldSelectRememberChoice = ArgumentMatchers.anyBoolean(),
+            isNotificationRequest = ArgumentMatchers.anyBoolean(),
+            negativeButtonResId = ArgumentMatchers.isNull(),
+            learnMoreLink = ArgumentMatchers.isNull(),
         )
     }
 
@@ -1220,18 +1418,23 @@ class SitePermissionsFeatureTest {
                 override fun containsVideoAndAudioSources() = true
 
                 override fun reject() = Unit
+
+                override fun merge(permissionRequest: PermissionRequest) = Unit
             }
             val sitePermissionsDialogFragment = SitePermissionsDialogFragment()
             doReturn(sitePermissionsDialogFragment).`when`(sitePermissionFeature)
                 .createSinglePermissionPrompt(
-                    any(),
-                    ArgumentMatchers.anyString(),
-                    any(),
-                    ArgumentMatchers.anyInt(),
-                    ArgumentMatchers.anyInt(),
-                    ArgumentMatchers.anyBoolean(),
-                    ArgumentMatchers.anyBoolean(),
-                    ArgumentMatchers.anyBoolean(),
+                    context = any(),
+                    origin = ArgumentMatchers.anyString(),
+                    permissionRequest = any(),
+                    titleId = ArgumentMatchers.anyInt(),
+                    iconId = ArgumentMatchers.anyInt(),
+                    showDoNotAskAgainCheckBox = ArgumentMatchers.anyBoolean(),
+                    doNotAskAgainCheckBoxLabel = ArgumentMatchers.anyInt(),
+                    shouldSelectRememberChoice = ArgumentMatchers.anyBoolean(),
+                    isNotificationRequest = ArgumentMatchers.anyBoolean(),
+                    negativeButtonResId = ArgumentMatchers.anyInt(),
+                    learnMoreLink = ArgumentMatchers.anyString(),
                 )
 
             sitePermissionFeature.createPrompt(permissionRequest, URL)
@@ -1257,6 +1460,8 @@ class SitePermissionsFeatureTest {
             ContentAutoPlayAudible(),
             ContentAutoPlayInaudible(),
             ContentMediaKeySystemAccess(),
+            ContentLocalDeviceAccess(),
+            ContentLocalNetworkAccess(),
         )
 
         sitePermissionsList.forEach { permission ->
@@ -1275,6 +1480,8 @@ class SitePermissionsFeatureTest {
             doReturn(ALLOWED).`when`(sitePermissionFromStorage).mediaKeySystemAccess
             doReturn(AutoplayStatus.ALLOWED).`when`(sitePermissionFromStorage).autoplayAudible
             doReturn(AutoplayStatus.ALLOWED).`when`(sitePermissionFromStorage).autoplayInaudible
+            doReturn(ALLOWED).`when`(sitePermissionFromStorage).localDeviceAccess
+            doReturn(ALLOWED).`when`(sitePermissionFromStorage).localNetworkAccess
 
             val isAllowed = sitePermissionFromStorage.isGranted(request)
             assertTrue(isAllowed)
@@ -1291,6 +1498,8 @@ class SitePermissionsFeatureTest {
             ContentVideoCamera(),
             ContentVideoCapture(),
             ContentCrossOriginStorageAccess(),
+            ContentLocalDeviceAccess(),
+            ContentLocalNetworkAccess(),
             Generic(),
         )
 
@@ -1307,6 +1516,8 @@ class SitePermissionsFeatureTest {
             doReturn(BLOCKED).`when`(sitePermissionFromStorage).camera
             doReturn(BLOCKED).`when`(sitePermissionFromStorage).microphone
             doReturn(BLOCKED).`when`(sitePermissionFromStorage).crossOriginStorageAccess
+            doReturn(BLOCKED).`when`(sitePermissionFromStorage).localDeviceAccess
+            doReturn(BLOCKED).`when`(sitePermissionFromStorage).localNetworkAccess
 
             try {
                 val isAllowed = sitePermissionFromStorage.isGranted(request)
@@ -1385,6 +1596,8 @@ class SitePermissionsFeatureTest {
             persistentStorage = SitePermissionsRules.Action.BLOCKED,
             crossOriginStorageAccess = SitePermissionsRules.Action.ALLOWED,
             mediaKeySystemAccess = SitePermissionsRules.Action.ASK_TO_ALLOW,
+            localDeviceAccess = SitePermissionsRules.Action.ASK_TO_ALLOW,
+            localNetworkAccess = SitePermissionsRules.Action.ASK_TO_ALLOW,
         )
 
         sitePermissionFeature.sitePermissionsRules = rules
@@ -1400,6 +1613,8 @@ class SitePermissionsFeatureTest {
         assertEquals(ALLOWED, sitePermissions.autoplayInaudible.toStatus())
         assertEquals(BLOCKED, sitePermissions.localStorage)
         assertEquals(ALLOWED, sitePermissions.crossOriginStorageAccess)
+        assertEquals(NO_DECISION, sitePermissions.localDeviceAccess)
+        assertEquals(NO_DECISION, sitePermissions.localNetworkAccess)
         assertEquals(NO_DECISION, sitePermissions.mediaKeySystemAccess)
     }
 
@@ -1428,6 +1643,8 @@ class SitePermissionsFeatureTest {
                 }
 
                 override fun reject() = Unit
+
+                override fun merge(permissionRequest: PermissionRequest) = Unit
             }
 
             mockStorage = mock()
@@ -1447,4 +1664,10 @@ class SitePermissionsFeatureTest {
         doReturn(transaction).`when`(fragmentManager).beginTransaction()
         return fragmentManager
     }
+
+    private class FakeSitePermissionsLearnMoreUrlProvider(
+        var expectedLink: String? = null,
+    ) : SitePermissionsLearnMoreUrlProvider {
+        override fun getUrl(permission: Permission): String? = expectedLink
+    }
 }
