@@ -8,9 +8,9 @@ import androidx.lifecycle.Lifecycle
 import androidx.lifecycle.LifecycleOwner
 import androidx.test.ext.junit.runners.AndroidJUnit4
 import kotlinx.coroutines.CoroutineScope
-import kotlinx.coroutines.ExperimentalCoroutinesApi
 import kotlinx.coroutines.SupervisorJob
 import kotlinx.coroutines.plus
+import mozilla.appservices.fxaclient.CloseTabsResult
 import mozilla.appservices.fxaclient.FxaException
 import mozilla.appservices.fxaclient.IncomingDeviceCommand
 import mozilla.appservices.fxaclient.SendTabPayload
@@ -30,6 +30,7 @@ import mozilla.components.concept.sync.DeviceType
 import mozilla.components.concept.sync.TabData
 import mozilla.components.support.test.any
 import mozilla.components.support.test.argumentCaptor
+import mozilla.components.support.test.expectException
 import mozilla.components.support.test.mock
 import mozilla.components.support.test.robolectric.testContext
 import mozilla.components.support.test.rule.MainCoroutineRule
@@ -44,6 +45,7 @@ import org.junit.Before
 import org.junit.Rule
 import org.junit.Test
 import org.junit.runner.RunWith
+import org.mockito.Mockito.atLeast
 import org.mockito.Mockito.doAnswer
 import org.mockito.Mockito.never
 import org.mockito.Mockito.reset
@@ -57,7 +59,6 @@ import mozilla.appservices.fxaclient.DevicePushSubscription as NativeDevicePushS
 import mozilla.appservices.fxaclient.FxaClient as NativeFirefoxAccount
 import mozilla.appservices.sync15.DeviceType as RustDeviceType
 
-@ExperimentalCoroutinesApi
 @RunWith(AndroidJUnit4::class)
 class FxaDeviceConstellationTest {
     lateinit var account: NativeFirefoxAccount
@@ -115,7 +116,6 @@ class FxaDeviceConstellationTest {
     }
 
     @Test
-    @ExperimentalCoroutinesApi
     fun `updating device name`() = runTestOnMain {
         val currentDevice = testDevice("currentTestDevice", true)
         `when`(account.getDevices()).thenReturn(arrayOf(currentDevice))
@@ -124,7 +124,9 @@ class FxaDeviceConstellationTest {
         try {
             constellation.setDeviceName("new name", testContext)
             fail()
-        } catch (e: IllegalStateException) {}
+        } catch (e: IllegalStateException) {
+            // Ignore exception
+        }
 
         val cache = FxaDeviceSettingsCache(testContext)
         cache.setToCache(DeviceSettings("someId", "test name", RustDeviceType.MOBILE))
@@ -156,7 +158,6 @@ class FxaDeviceConstellationTest {
     }
 
     @Test
-    @ExperimentalCoroutinesApi
     fun `set device push subscription`() = runTestOnMain {
         val subscription = DevicePushSubscription("http://endpoint.com", "pk", "auth key")
         constellation.setDevicePushSubscription(subscription)
@@ -165,7 +166,6 @@ class FxaDeviceConstellationTest {
     }
 
     @Test
-    @ExperimentalCoroutinesApi
     fun `process raw device command`() = runTestOnMain {
         // No commands, no observer.
         `when`(account.handlePushMessage("raw events payload")).thenReturn(mozilla.appservices.fxaclient.AccountEvent.Unknown)
@@ -224,7 +224,7 @@ class FxaDeviceConstellationTest {
     @Test
     fun `send command to device will report exceptions`() = runTestOnMain {
         val exception = FxaException.Other("")
-        val exceptionCaptor = argumentCaptor<SendCommandException>()
+        val exceptionCaptor = argumentCaptor<SendCommandException.Other>()
         doAnswer { throw exception }.`when`(account).sendSingleTab(any(), any(), any())
 
         val success = constellation.sendCommandToDevice(
@@ -253,7 +253,23 @@ class FxaDeviceConstellationTest {
     }
 
     @Test
-    @ExperimentalCoroutinesApi
+    fun `send command to device will propagate close tabs-specific exceptions`() = runTestOnMain {
+        `when`(account.gatherTelemetry()).thenReturn("{}")
+        `when`(account.closeTabs(any(), any())).thenReturn(CloseTabsResult.TabsNotClosed(listOf("https://example.com")))
+
+        val exception = expectException<SendCommandException.TabsNotClosed> {
+            constellation.sendCommandToDevice(
+                "targetID",
+                DeviceCommandOutgoing.CloseTab(listOf("https://example.com", "https://mozilla.org")),
+            )
+        }
+        assertEquals(listOf("https://example.com"), exception.urls)
+        val exceptionCaptor = argumentCaptor<Throwable>()
+        verify(constellation.crashReporter!!, atLeast(0)).submitCaughtException(exceptionCaptor.capture())
+        assertFalse(exceptionCaptor.allValues.any { it is SendCommandException.TabsNotClosed })
+    }
+
+    @Test
     fun `refreshing constellation`() = runTestOnMain {
         // No devices, no observers.
         `when`(account.getDevices()).thenReturn(emptyArray())
@@ -342,7 +358,6 @@ class FxaDeviceConstellationTest {
     }
 
     @Test
-    @ExperimentalCoroutinesApi
     fun `polling for commands triggers observers`() = runTestOnMain {
         // No commands, no observers.
         `when`(account.gatherTelemetry()).thenReturn("{}")
