@@ -5,16 +5,14 @@
 package mozilla.components.service.location
 
 import junit.framework.TestCase.assertNull
-import kotlinx.coroutines.ExperimentalCoroutinesApi
-import kotlinx.coroutines.test.UnconfinedTestDispatcher
+import kotlinx.coroutines.test.StandardTestDispatcher
 import kotlinx.coroutines.test.runTest
 import org.junit.Test
 
 class LocationServiceTest {
     @Test
-    @ExperimentalCoroutinesApi // UnconfinedTestDispatcher
     fun `dummy implementation returns null`() {
-        runTest(UnconfinedTestDispatcher()) {
+        runTest(StandardTestDispatcher()) {
             assertNull(LocationService.dummy().fetchRegion(false))
             assertNull(LocationService.dummy().fetchRegion(true))
             assertNull(LocationService.dummy().fetchRegion(false))
