@@ -28,6 +28,7 @@ import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.concept.engine.EngineSession
 import mozilla.components.concept.engine.webextension.Action
 import mozilla.components.concept.engine.webextension.ActionHandler
+import mozilla.components.concept.engine.webextension.PermissionPromptResponse
 import mozilla.components.concept.engine.webextension.TabHandler
 import mozilla.components.concept.engine.webextension.WebExtension
 import mozilla.components.concept.engine.webextension.WebExtensionDelegate
@@ -37,6 +38,7 @@ import mozilla.components.lib.state.ext.flowScoped
 import mozilla.components.support.base.log.logger.Logger
 import mozilla.components.support.ktx.kotlin.isExtensionUrl
 import mozilla.components.support.ktx.kotlinx.coroutines.flow.filterChanged
+import mozilla.components.support.webextensions.WebExtensionSupport.initialize
 import mozilla.components.support.webextensions.facts.emitWebExtensionsInitializedFact
 import java.util.concurrent.ConcurrentHashMap
 
@@ -44,9 +46,10 @@ import java.util.concurrent.ConcurrentHashMap
  * Function to relay the permission request to the app / user.
  */
 typealias onUpdatePermissionRequest = (
-    current: WebExtension,
-    updated: WebExtension,
+    extension: WebExtension,
     newPermissions: List<String>,
+    newOrigins: List<String>,
+    newDataCollectionPermissions: List<String>,
     onPermissionsGranted: ((Boolean) -> Unit),
 ) -> Unit
 
@@ -173,7 +176,7 @@ object WebExtensionSupport {
         onNewTabOverride: ((WebExtension?, EngineSession, String) -> String)? = null,
         onCloseTabOverride: ((WebExtension?, String) -> Unit)? = null,
         onSelectTabOverride: ((WebExtension?, String) -> Unit)? = null,
-        onUpdatePermissionRequest: onUpdatePermissionRequest? = { _, _, _, _ -> },
+        onUpdatePermissionRequest: onUpdatePermissionRequest? = { _, _, _, _, _ -> },
         onExtensionsLoaded: ((List<WebExtension>) -> Unit)? = null,
     ) {
         this.onUpdatePermissionRequest = onUpdatePermissionRequest
@@ -189,7 +192,12 @@ object WebExtensionSupport {
 
         runtime.registerWebExtensionDelegate(
             object : WebExtensionDelegate {
-                override fun onNewTab(extension: WebExtension, engineSession: EngineSession, active: Boolean, url: String) {
+                override fun onNewTab(
+                    extension: WebExtension,
+                    engineSession: EngineSession,
+                    active: Boolean,
+                    url: String,
+                ) {
                     openTab(store, onNewTabOverride, onSelectTabOverride, extension, engineSession, url, active)
                 }
 
@@ -220,7 +228,13 @@ object WebExtensionSupport {
                             }
                             null
                         } else {
-                            val sessionId = openTab(store, onNewTabOverride, onSelectTabOverride, extension, engineSession)
+                            val sessionId = openTab(
+                                store,
+                                onNewTabOverride,
+                                onSelectTabOverride,
+                                extension,
+                                engineSession,
+                            )
                             store.dispatch(WebExtensionAction.UpdatePopupSessionAction(extension.id, sessionId))
                             engineSession
                         }
@@ -294,28 +308,36 @@ object WebExtensionSupport {
 
                 override fun onInstallPermissionRequest(
                     extension: WebExtension,
-                    onPermissionsGranted: ((Boolean) -> Unit),
+                    permissions: List<String>,
+                    origins: List<String>,
+                    dataCollectionPermissions: List<String>,
+                    onConfirm: (PermissionPromptResponse) -> Unit,
                 ) {
                     store.dispatch(
                         WebExtensionAction.UpdatePromptRequestWebExtensionAction(
                             WebExtensionPromptRequest.AfterInstallation.Permissions.Required(
                                 extension,
-                                onPermissionsGranted,
+                                permissions,
+                                origins,
+                                dataCollectionPermissions,
+                                onConfirm,
                             ),
                         ),
                     )
                 }
 
                 override fun onUpdatePermissionRequest(
-                    current: WebExtension,
-                    updated: WebExtension,
+                    extension: WebExtension,
                     newPermissions: List<String>,
+                    newOrigins: List<String>,
+                    newDataCollectionPermissions: List<String>,
                     onPermissionsGranted: ((Boolean) -> Unit),
                 ) {
                     this@WebExtensionSupport.onUpdatePermissionRequest?.invoke(
-                        current,
-                        updated,
+                        extension,
                         newPermissions,
+                        newOrigins,
+                        newDataCollectionPermissions,
                         onPermissionsGranted,
                     )
                 }
@@ -323,6 +345,8 @@ object WebExtensionSupport {
                 override fun onOptionalPermissionsRequest(
                     extension: WebExtension,
                     permissions: List<String>,
+                    origins: List<String>,
+                    dataCollectionPermissions: List<String>,
                     onPermissionsGranted: ((Boolean) -> Unit),
                 ) {
                     store.dispatch(
@@ -330,6 +354,8 @@ object WebExtensionSupport {
                             WebExtensionPromptRequest.AfterInstallation.Permissions.Optional(
                                 extension,
                                 permissions,
+                                origins,
+                                dataCollectionPermissions,
                                 onPermissionsGranted,
                             ),
                         ),
@@ -360,16 +386,14 @@ object WebExtensionSupport {
      */
     private fun registerInstalledExtensions(store: BrowserStore, runtime: WebExtensionRuntime) {
         runtime.listInstalledWebExtensions(
-            onSuccess = {
-                    extensions ->
+            onSuccess = { extensions ->
                 extensions.forEach { registerInstalledExtension(store, it) }
                 emitWebExtensionsInitializedFact(extensions)
                 closeUnsupportedTabs(store, extensions)
                 initializationResult.complete(Unit)
                 onExtensionsLoaded?.invoke(extensions.filter { !it.isBuiltIn() })
             },
-            onError = {
-                    throwable ->
+            onError = { throwable ->
                 logger.error("Failed to query installed extension", throwable)
                 initializationResult.completeExceptionally(throwable)
             },
@@ -533,6 +557,7 @@ object WebExtensionSupport {
             getMetadata()?.name,
             isEnabled(),
             isAllowedInPrivateBrowsing(),
+            isBuiltIn(),
         )
 
     private fun SessionState.isCustomTab() = this is CustomTabSessionState
