@@ -8,7 +8,6 @@ import android.content.ClipboardManager
 import android.content.Context
 import android.content.Context.INPUT_METHOD_SERVICE
 import android.graphics.Rect
-import android.os.Build
 import android.provider.Settings.Secure.DEFAULT_INPUT_METHOD
 import android.provider.Settings.Secure.getString
 import android.text.Editable
@@ -22,7 +21,6 @@ import android.text.style.BackgroundColorSpan
 import android.text.style.ForegroundColorSpan
 import android.util.AttributeSet
 import android.view.KeyEvent
-import android.view.MotionEvent
 import android.view.View
 import android.view.accessibility.AccessibilityEvent
 import android.view.inputmethod.BaseInputConnection
@@ -276,13 +274,6 @@ open class InlineAutocompleteEditText @JvmOverloads constructor(
         }
 
         removeAutocomplete(text)
-
-        try {
-            restartInput()
-            inputMethodManger?.hideSoftInputFromWindow(windowToken, 0)
-        } catch (ignored: NullPointerException) {
-            // See bug 782096 for details
-        }
     }
 
     override fun setText(text: CharSequence?, type: BufferType) {
@@ -578,7 +569,6 @@ open class InlineAutocompleteEditText @JvmOverloads constructor(
      *
      * Also turns off text prediction for private mode tabs.
      */
-    @SuppressWarnings("ComplexMethod")
     override fun onCreateInputConnection(outAttrs: EditorInfo): InputConnection? {
         val ic = super.onCreateInputConnection(outAttrs) ?: return null
 
@@ -794,13 +784,7 @@ open class InlineAutocompleteEditText @JvmOverloads constructor(
             val min = 0.coerceAtLeast(selectionStart.coerceAtMost(selectionEnd))
             val max = 0.coerceAtLeast(selectionStart.coerceAtLeast(selectionEnd))
 
-            if (id == android.R.id.pasteAsPlainText ||
-                (id == android.R.id.paste && Build.VERSION.SDK_INT >= Build.VERSION_CODES.M)
-            ) {
-                paste(min, max, false)
-            } else {
-                paste(min, max, true)
-            }
+            paste(min, max, false)
 
             return true // action was performed
         }
@@ -808,34 +792,6 @@ open class InlineAutocompleteEditText @JvmOverloads constructor(
         return callOnTextContextMenuItemSuper(id)
     }
 
-    @Suppress("ClickableViewAccessibility")
-    override fun onTouchEvent(event: MotionEvent): Boolean {
-        return if (Build.VERSION.SDK_INT == Build.VERSION_CODES.M &&
-            event.actionMasked == MotionEvent.ACTION_UP
-        ) {
-            // Android 6 occasionally throws a NullPointerException inside Editor.onTouchEvent()
-            // for ACTION_UP when attempting to display (uninitialised) text handles. The Editor
-            // and TextView IME interactions are quite complex, so I don't know how to properly
-            // work around this issue, but we can at least catch the NPE to prevent crashing
-            // the whole app.
-            // (Editor tries to make both selection handles visible, but in certain cases they haven't
-            // been initialised yet, causing the NPE. It doesn't bother to check the selection handle
-            // state, and making some other calls to ensure the handles exist doesn't seem like a
-            // clean solution either since I don't understand most of the selection logic. This implementation
-            // only seems to exist in Android 6, both Android 5 and 7 have different implementations.)
-            try {
-                super.onTouchEvent(event)
-            } catch (ignored: NullPointerException) {
-                // Ignore this (see above) - since we're now in an unknown state let's clear all selection
-                // (which is still better than an arbitrary crash that we can't control):
-                clearFocus()
-                true
-            }
-        } else {
-            super.onTouchEvent(event)
-        }
-    }
-
     @VisibleForTesting
     internal fun callOnTextContextMenuItemSuper(id: Int) = super.onTextContextMenuItem(id)
 
