@@ -17,8 +17,8 @@ import org.mozilla.focus.activity.robots.searchScreen
 import org.mozilla.focus.helpers.FeatureSettingsHelper
 import org.mozilla.focus.helpers.MainActivityFirstrunTestRule
 import org.mozilla.focus.helpers.MockWebServerHelper
+import org.mozilla.focus.helpers.TestAssetHelper.genericAsset
 import org.mozilla.focus.helpers.TestAssetHelper.getEnhancedTrackingProtectionAsset
-import org.mozilla.focus.helpers.TestAssetHelper.getGenericAsset
 import org.mozilla.focus.helpers.TestHelper.exitToBrowser
 import org.mozilla.focus.helpers.TestHelper.exitToTop
 import org.mozilla.focus.helpers.TestHelper.waitingTime
@@ -31,7 +31,7 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     private lateinit var webServer: MockWebServer
     private val featureSettingsHelper = FeatureSettingsHelper()
 
-    @get: Rule
+    @get:Rule
     val mActivityTestRule = MainActivityFirstrunTestRule(showFirstRun = false)
 
     @Before
@@ -76,11 +76,11 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     @SmokeTest
     @Test
     fun blockAdTrackersTest() {
-        val genericPage = getGenericAsset(webServer)
-        val trackingPage = getEnhancedTrackingProtectionAsset(webServer, "adsTrackers")
+        val genericPage = webServer.genericAsset
+        val trackingPage = webServer.getEnhancedTrackingProtectionAsset("adsTrackers")
 
         searchScreen {
-        }.loadPage(genericPage.url) {
+        }.loadPage(genericPage.url.toString()) {
             // loading a generic page to allow GV to fully load on first run
             verifyPageContent(genericPage.content)
         }.openMainMenu {
@@ -97,8 +97,8 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     @SmokeTest
     @Test
     fun allowAdTrackersTest() {
-        val genericPage = getGenericAsset(webServer)
-        val trackingPage = getEnhancedTrackingProtectionAsset(webServer, "adsTrackers")
+        val genericPage = webServer.genericAsset
+        val trackingPage = webServer.getEnhancedTrackingProtectionAsset("adsTrackers")
 
         homeScreen {
         }.openMainMenu {
@@ -127,8 +127,8 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     @SmokeTest
     @Test
     fun blockAnalyticsTrackersTest() {
-        val genericPage = getGenericAsset(webServer)
-        val trackingPage = getEnhancedTrackingProtectionAsset(webServer, "analyticsTrackers")
+        val genericPage = webServer.genericAsset
+        val trackingPage = webServer.getEnhancedTrackingProtectionAsset("analyticsTrackers")
 
         searchScreen {
         }.loadPage(genericPage.url) {
@@ -148,8 +148,8 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     @SmokeTest
     @Test
     fun allowAnalyticsTrackersTest() {
-        val genericPage = getGenericAsset(webServer)
-        val trackingPage = getEnhancedTrackingProtectionAsset(webServer, "analyticsTrackers")
+        val genericPage = webServer.genericAsset
+        val trackingPage = webServer.getEnhancedTrackingProtectionAsset("analyticsTrackers")
 
         homeScreen {
         }.openMainMenu {
@@ -178,8 +178,8 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     @SmokeTest
     @Test
     fun blockSocialTrackersTest() {
-        val genericPage = getGenericAsset(webServer)
-        val trackingPage = getEnhancedTrackingProtectionAsset(webServer, "socialTrackers")
+        val genericPage = webServer.genericAsset
+        val trackingPage = webServer.getEnhancedTrackingProtectionAsset("socialTrackers")
 
         searchScreen {
         }.loadPage(genericPage.url) {
@@ -199,8 +199,8 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     @SmokeTest
     @Test
     fun allowSocialTrackersTest() {
-        val genericPage = getGenericAsset(webServer)
-        val trackingPage = getEnhancedTrackingProtectionAsset(webServer, "socialTrackers")
+        val genericPage = webServer.genericAsset
+        val trackingPage = webServer.getEnhancedTrackingProtectionAsset("socialTrackers")
 
         homeScreen {
         }.openMainMenu {
@@ -225,8 +225,8 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     @SmokeTest
     @Test
     fun allowOtherContentTrackersTest() {
-        val genericPage = getGenericAsset(webServer)
-        val trackingPage = getEnhancedTrackingProtectionAsset(webServer, "otherTrackers")
+        val genericPage = webServer.genericAsset
+        val trackingPage = webServer.getEnhancedTrackingProtectionAsset("otherTrackers")
 
         searchScreen {
         }.loadPage(genericPage.url) {
@@ -247,8 +247,8 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     @SmokeTest
     @Test
     fun blockOtherContentTrackersTest() {
-        val genericPage = getGenericAsset(webServer)
-        val trackingPage = getEnhancedTrackingProtectionAsset(webServer, "otherTrackers")
+        val genericPage = webServer.genericAsset
+        val trackingPage = webServer.getEnhancedTrackingProtectionAsset("otherTrackers")
 
         searchScreen {
         }.loadPage(genericPage.url) {
@@ -273,8 +273,8 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     @SmokeTest
     @Test
     fun addURLToTPExceptionsListTest() {
-        val genericPage = getGenericAsset(webServer)
-        val trackingPage = getEnhancedTrackingProtectionAsset(webServer, "otherTrackers")
+        val genericPage = webServer.genericAsset
+        val trackingPage = webServer.getEnhancedTrackingProtectionAsset("otherTrackers")
 
         searchScreen {
         }.loadPage(genericPage.url) {
@@ -296,8 +296,8 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     @SmokeTest
     @Test
     fun removeOneExceptionURLTest() {
-        val genericPage = getGenericAsset(webServer)
-        val trackingPage = getEnhancedTrackingProtectionAsset(webServer, "otherTrackers")
+        val genericPage = webServer.genericAsset
+        val trackingPage = webServer.getEnhancedTrackingProtectionAsset("otherTrackers")
 
         searchScreen {
         }.loadPage(genericPage.url) {
@@ -325,8 +325,8 @@ class EnhancedTrackingProtectionSettingsTest : TestSetup() {
     @SmokeTest
     @Test
     fun removeAllExceptionURLTest() {
-        val genericPage = getGenericAsset(webServer)
-        val trackingPage = getEnhancedTrackingProtectionAsset(webServer, "otherTrackers")
+        val genericPage = webServer.genericAsset
+        val trackingPage = webServer.getEnhancedTrackingProtectionAsset("otherTrackers")
 
         searchScreen {
         }.loadPage(genericPage.url) {
