@@ -4,6 +4,7 @@
 
 package org.mozilla.focus.activity.robots
 
+import android.net.Uri
 import androidx.test.espresso.Espresso.onView
 import androidx.test.espresso.IdlingRegistry
 import androidx.test.espresso.IdlingResource
@@ -24,7 +25,6 @@ import androidx.test.uiautomator.Until
 import org.hamcrest.Matchers.not
 import org.junit.Assert.assertFalse
 import org.junit.Assert.assertTrue
-import org.mozilla.focus.R
 import org.mozilla.focus.helpers.Constants.RETRY_COUNT
 import org.mozilla.focus.helpers.TestHelper.mDevice
 import org.mozilla.focus.helpers.TestHelper.packageName
@@ -34,6 +34,10 @@ import org.mozilla.focus.helpers.TestHelper.waitingTime
 import org.mozilla.focus.helpers.TestHelper.waitingTimeShort
 import org.mozilla.focus.idlingResources.SessionLoadedIdlingResource
 import java.time.LocalDate
+import mozilla.components.browser.toolbar.R as toolbarR
+import mozilla.components.feature.contextmenu.R as contextmenuR
+import mozilla.components.feature.findinpage.R as findinpageR
+import mozilla.components.ui.tabcounter.R as tabcounterR
 
 class BrowserRobot {
 
@@ -50,15 +54,17 @@ class BrowserRobot {
                 .waitForExists(waitingTime),
         )
 
-    fun verifyPageContent(expectedText: String) {
+    fun verifyPageContent(expectedText: String, alsoClick: Boolean = false) {
         sessionLoadedIdlingResource = SessionLoadedIdlingResource()
 
         runWithIdleRes(sessionLoadedIdlingResource) {
             for (i in 1..RETRY_COUNT) {
                 try {
-                    assertTrue(
-                        webPageItemContainingText(expectedText).waitForExists(pageLoadingTime),
-                    )
+                    val obj = webPageItemContainingText(expectedText)
+                    assertTrue(obj.waitForExists(pageLoadingTime))
+                    if (alsoClick) {
+                        obj.click()
+                    }
                     break
                 } catch (e: AssertionError) {
                     if (i == RETRY_COUNT) {
@@ -113,6 +119,13 @@ class BrowserRobot {
         }
     }
 
+    fun verifyCustomTabUrl(url: String) {
+        val uri = Uri.parse(url)
+        val expectedText = uri.host ?: url // fallback if host is null
+
+        verifyPageURL(expectedText)
+    }
+
     fun verifyPageURL(expectedText: String) {
         browserURLbar.waitForExists(waitingTime)
         sessionLoadedIdlingResource = SessionLoadedIdlingResource()
@@ -129,16 +142,16 @@ class BrowserRobot {
 
     fun clickGetCameraButton() = clickPageObject(webPageItemContainingText("Open camera"))
 
-    fun verifyCameraPermissionPrompt(url: String) {
+    fun verifyCameraPermissionPrompt(host: String) {
         assertTrue(
-            mDevice.findObject(UiSelector().text("Allow $url to use your camera?"))
+            mDevice.findObject(UiSelector().text("Allow $host to use your camera?"))
                 .waitForExists(waitingTime),
         )
     }
 
-    fun verifyLocationPermissionPrompt(url: String) {
+    fun verifyLocationPermissionPrompt(host: String) {
         assertTrue(
-            mDevice.findObject(UiSelector().text("Allow $url to use your location?"))
+            mDevice.findObject(UiSelector().text("Allow $host to use your location?"))
                 .waitForExists(waitingTime),
         )
     }
@@ -167,7 +180,7 @@ class BrowserRobot {
     fun verifyNumberOfTabsOpened(tabsCount: Int) {
         assertTrue(
             mDevice.findObject(
-                UiSelector().description("$tabsCount open tabs. Tap to switch tabs."),
+                UiSelector().description("Private Tabs Open: $tabsCount. Tap to switch tabs."),
             ).waitForExists(waitingTime),
         )
     }
@@ -212,15 +225,14 @@ class BrowserRobot {
 
     fun verifySiteTrackingProtectionIconShown() = assertTrue(securityIcon.waitForExists(waitingTime))
 
-    fun verifySiteSecurityIndicatorShown() = assertTrue(site_security_indicator.waitForExists(waitingTime))
+    fun verifySiteInfoIndicatorShown() = assertTrue(site_info_indicator.waitForExists(waitingTime))
 
     fun verifyLinkContextMenu(linkAddress: String) {
         assertTrue(
-            mDevice.findObject(
-                UiSelector().resourceId("$packageName:id/parentPanel"),
-            ).waitForExists(waitingTime),
+            mDevice.findObject(UiSelector().resourceId("$packageName:id/parentPanel"))
+                .waitForExists(waitingTime),
         )
-        onView(withId(R.id.titleView)).check(matches(withText(linkAddress)))
+        onView(withId(contextmenuR.id.titleView)).check(matches(withText(linkAddress)))
         openLinkInPrivateTab.check(matches(isDisplayed()))
         copyLink.check(matches(isDisplayed()))
         shareLink.check(matches(isDisplayed()))
@@ -228,11 +240,10 @@ class BrowserRobot {
 
     fun verifyImageContextMenu(hasLink: Boolean, linkAddress: String) {
         assertTrue(
-            mDevice.findObject(
-                UiSelector().resourceId("$packageName:id/parentPanel"),
-            ).waitForExists(waitingTime),
+            mDevice.findObject(UiSelector().resourceId("$packageName:id/parentPanel"))
+                .waitForExists(waitingTime),
         )
-        onView(withId(R.id.titleView)).check(matches(withText(linkAddress)))
+        onView(withId(contextmenuR.id.titleView)).check(matches(withText(linkAddress)))
         if (hasLink) {
             openLinkInPrivateTab.check(matches(isDisplayed()))
             downloadLink.check(matches(isDisplayed()))
@@ -385,21 +396,12 @@ class BrowserRobot {
         mDevice.wait(Until.findObject(By.res("$packageName:id/find_in_page_result_text")), waitingTime)
     }
 
-    fun verifyFindNextInPageResult(ratioCounter: String) {
-        mDevice.wait(Until.findObject(By.text(ratioCounter)), waitingTime)
-        val resultsCounter = mDevice.findObject(By.text(ratioCounter))
-        findInPageResult.check(matches(withText((ratioCounter))))
-        findInPageNextButton.perform(click())
-        resultsCounter.wait(Until.textNotEquals(ratioCounter), waitingTime)
-    }
+    fun verifyFindInPageResult(ratioCounter: String) =
+        assertTrue(mDevice.findObject(UiSelector().textContains(ratioCounter)).waitForExists(waitingTime))
 
-    fun verifyFindPrevInPageResult(ratioCounter: String) {
-        mDevice.wait(Until.findObject(By.text(ratioCounter)), waitingTime)
-        val resultsCounter = mDevice.findObject(By.text(ratioCounter))
-        findInPageResult.check(matches(withText((ratioCounter))))
-        findInPagePrevButton.perform(click())
-        resultsCounter.wait(Until.textNotEquals(ratioCounter), waitingTime)
-    }
+    fun clickFindInPageNextButton() = findInPageNextButton.click()
+
+    fun clickFindInPagePrevButton() = findInPagePrevButton.click()
 
     fun closeFindInPage() {
         findInPageCloseButton.perform(click())
@@ -539,7 +541,7 @@ class BrowserRobot {
             if (securityIcon.exists()) {
                 securityIcon.click()
             } else {
-                site_security_indicator.click()
+                site_info_indicator.click()
             }
 
             SiteSecurityInfoSheetRobot().interact()
@@ -608,9 +610,9 @@ private val browserURLbar = mDevice.findObject(
 
 private val eraseBrowsingButton = onView(withContentDescription("Erase browsing history"))
 
-private val tabsCounter = onView(withId(R.id.counter_root))
+private val tabsCounter = onView(withId(tabcounterR.id.counter_root))
 
-private val mainMenu = onView(withId(R.id.mozac_browser_toolbar_menu))
+private val mainMenu = onView(withId(toolbarR.id.mozac_browser_toolbar_menu))
 
 private val shareAppsList =
     mDevice.findObject(UiSelector().resourceId("android:id/resolver_list"))
@@ -621,10 +623,10 @@ private val securityIcon =
             .resourceId("$packageName:id/mozac_browser_toolbar_tracking_protection_indicator"),
     )
 
-private val site_security_indicator =
+private val site_info_indicator =
     mDevice.findObject(
         UiSelector()
-            .resourceId("$packageName:id/mozac_browser_toolbar_security_indicator"),
+            .resourceId("$packageName:id/mozac_browser_toolbar_site_info_indicator"),
     )
 
 // Link long-tap context menu items
@@ -646,15 +648,15 @@ private val copyImageLocation = onView(withText("Copy image location"))
 private val shareImage = onView(withText("Share image"))
 
 // Find in page toolbar
-private val findInPageQuery = onView(withId(R.id.find_in_page_query_text))
+private val findInPageQuery = onView(withId(findinpageR.id.find_in_page_query_text))
 
-private val findInPageResult = onView(withId(R.id.find_in_page_result_text))
+private val findInPageResult = onView(withId(findinpageR.id.find_in_page_result_text))
 
-private val findInPageNextButton = onView(withId(R.id.find_in_page_next_btn))
+private val findInPageNextButton = mDevice.findObject(UiSelector().resourceId("$packageName:id/find_in_page_next_btn"))
 
-private val findInPagePrevButton = onView(withId(R.id.find_in_page_prev_btn))
+private val findInPagePrevButton = mDevice.findObject(UiSelector().resourceId("$packageName:id/find_in_page_prev_btn"))
 
-private val findInPageCloseButton = onView(withId(R.id.find_in_page_close_btn))
+private val findInPageCloseButton = onView(withId(findinpageR.id.find_in_page_close_btn))
 
 private val openLinksInAppsMessage = mDevice.findObject(UiSelector().resourceId("$packageName:id/alertTitle"))
 
