@@ -7,10 +7,10 @@ package org.mozilla.focus.browser.integration
 import android.graphics.Color
 import android.widget.LinearLayout
 import androidx.annotation.VisibleForTesting
+import androidx.appcompat.app.AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM
 import androidx.appcompat.content.res.AppCompatResources
 import androidx.appcompat.widget.AppCompatEditText
-import androidx.compose.material.Text
-import androidx.compose.ui.platform.LocalContext
+import androidx.compose.material3.Text
 import androidx.compose.ui.res.colorResource
 import androidx.compose.ui.res.stringResource
 import androidx.compose.ui.text.style.TextDecoration
@@ -18,11 +18,15 @@ import androidx.compose.ui.unit.dp
 import androidx.core.content.ContextCompat
 import androidx.core.content.res.ResourcesCompat
 import androidx.core.view.children
+import kotlinx.coroutines.CoroutineDispatcher
 import kotlinx.coroutines.CoroutineScope
+import kotlinx.coroutines.Dispatchers
+import kotlinx.coroutines.SupervisorJob
 import kotlinx.coroutines.cancel
 import kotlinx.coroutines.flow.distinctUntilChanged
 import kotlinx.coroutines.flow.distinctUntilChangedBy
 import kotlinx.coroutines.flow.mapNotNull
+import mozilla.components.browser.state.selector.findCustomTab
 import mozilla.components.browser.state.selector.findCustomTabOrSelectedTab
 import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.browser.toolbar.BrowserToolbar
@@ -30,14 +34,18 @@ import mozilla.components.browser.toolbar.display.DisplayToolbar.Indicators
 import mozilla.components.compose.cfr.CFRPopup
 import mozilla.components.compose.cfr.CFRPopupProperties
 import mozilla.components.feature.customtabs.CustomTabsToolbarFeature
+import mozilla.components.feature.customtabs.getConfiguredColorSchemeParams
 import mozilla.components.feature.session.SessionUseCases
 import mozilla.components.feature.tabs.CustomTabsUseCases
 import mozilla.components.feature.tabs.toolbar.TabCounterToolbarButton
 import mozilla.components.feature.toolbar.ToolbarBehaviorController
+import mozilla.components.feature.toolbar.ToolbarFeature
 import mozilla.components.feature.toolbar.ToolbarPresenter
 import mozilla.components.lib.state.ext.flowScoped
 import mozilla.components.support.base.feature.LifecycleAwareFeature
 import mozilla.components.support.ktx.android.view.hideKeyboard
+import mozilla.components.support.utils.ColorUtils.getReadableTextColor
+import mozilla.components.support.utils.ColorUtils.getSecondaryReadableTextColor
 import org.mozilla.focus.GleanMetrics.TabCount
 import org.mozilla.focus.GleanMetrics.TrackingProtection
 import org.mozilla.focus.R
@@ -54,8 +62,11 @@ import org.mozilla.focus.state.AppAction
 import org.mozilla.focus.state.Screen
 import org.mozilla.focus.ui.theme.focusTypography
 import org.mozilla.focus.utils.ClickableSubstringLink
+import androidx.cardview.R as cardViewR
+import mozilla.components.browser.toolbar.R as toolbarR
+import mozilla.components.ui.icons.R as iconsR
 
-@Suppress("LongParameterList", "LargeClass", "TooManyFunctions")
+@Suppress("LongParameterList", "LargeClass")
 class BrowserToolbarIntegration(
     private val store: BrowserStore,
     private val toolbar: BrowserToolbar,
@@ -67,12 +78,36 @@ class BrowserToolbarIntegration(
     private val eraseActionListener: () -> Unit,
     private val tabCounterListener: () -> Unit,
     private val customTabId: String? = null,
-    inTesting: Boolean = false,
+    isOnboardingTab: Boolean = false,
+    renderStyle: ToolbarFeature.RenderStyle = ToolbarFeature.RenderStyle.ColoredUrl,
+    private val coroutineDispatcher: CoroutineDispatcher = Dispatchers.Main,
 ) : LifecycleAwareFeature {
+    val backgroundColor = customTabId
+        ?.let { store.state.findCustomTab(it)?.config }
+        ?.getConfiguredColorSchemeParams(
+            currentNightMode = toolbar.context.resources.configuration.uiMode,
+            preferredNightMode = MODE_NIGHT_FOLLOW_SYSTEM,
+        )
+        ?.toolbarColor
+
     private val presenter = ToolbarPresenter(
         toolbar,
         store,
         customTabId,
+        urlRenderConfiguration = ToolbarFeature.UrlRenderConfiguration(
+            toolbar.context.components.publicSuffixList,
+            if (backgroundColor != null) {
+                getReadableTextColor(backgroundColor)
+            } else {
+                ContextCompat.getColor(toolbar.context, R.color.primaryText)
+            },
+            if (backgroundColor != null) {
+                getSecondaryReadableTextColor(backgroundColor)
+            } else {
+                ContextCompat.getColor(toolbar.context, R.color.secondaryText)
+            },
+            renderStyle,
+        ),
     )
 
     @VisibleForTesting
@@ -93,7 +128,7 @@ class BrowserToolbarIntegration(
     private val eraseAction = BrowserToolbar.Button(
         imageDrawable = AppCompatResources.getDrawable(
             toolbar.context,
-            R.drawable.mozac_ic_delete_24,
+            iconsR.drawable.mozac_ic_delete_24,
         )!!,
         contentDescription = toolbar.context.getString(R.string.content_description_erase),
         iconTintColorResource = R.color.primaryText,
@@ -111,6 +146,7 @@ class BrowserToolbarIntegration(
             tabCounterListener.invoke()
         },
         store = store,
+        showMaskInPrivateMode = false,
     )
 
     @VisibleForTesting
@@ -122,7 +158,7 @@ class BrowserToolbarIntegration(
         toolbar.display.apply {
             colors = colors.copy(
                 hint = ContextCompat.getColor(toolbar.context, R.color.urlBarHintText),
-                securityIconInsecure = Color.TRANSPARENT,
+                siteInfoIconInsecure = Color.TRANSPARENT,
                 text = ContextCompat.getColor(toolbar.context, R.color.primaryText),
                 menu = ContextCompat.getColor(toolbar.context, R.color.primaryText),
             )
@@ -131,7 +167,7 @@ class BrowserToolbarIntegration(
 
             displayIndicatorSeparator = false
 
-            setOnSiteSecurityClickedListener {
+            setOnSiteInfoClickedListener {
                 TrackingProtection.toolbarShieldClicked.add()
                 fragment.initCookieBanner()
                 fragment.showTrackingProtectionPanel()
@@ -147,15 +183,15 @@ class BrowserToolbarIntegration(
             icons = icons.copy(
                 trackingProtectionTrackersBlocked = AppCompatResources.getDrawable(
                     context,
-                    R.drawable.mozac_ic_shield_24,
+                    iconsR.drawable.mozac_ic_shield_checkmark_24,
                 )!!,
                 trackingProtectionNothingBlocked = AppCompatResources.getDrawable(
                     context,
-                    R.drawable.mozac_ic_shield_24,
+                    iconsR.drawable.mozac_ic_shield_checkmark_24,
                 )!!,
                 trackingProtectionException = AppCompatResources.getDrawable(
                     context,
-                    R.drawable.mozac_ic_shield_slash_24,
+                    iconsR.drawable.mozac_ic_shield_slash_24,
                 )!!,
             )
         }
@@ -171,6 +207,7 @@ class BrowserToolbarIntegration(
                 context = fragment.requireContext(),
                 store = store,
                 currentTabId = customTabId,
+                isOnboardingTab = isOnboardingTab,
                 onItemTapped = { controller.handleMenuInteraction(it) },
             )
             customTabsFeature = CustomTabsToolbarFeature(
@@ -182,7 +219,6 @@ class BrowserToolbarIntegration(
                 window = fragment.activity?.window,
                 menuItemIndex = menu.menuBuilder.items.size - 1,
                 closeListener = { fragment.closeCustomTab() },
-                updateTheme = true,
                 forceActionButtonTinting = false,
             )
         }
@@ -201,9 +237,7 @@ class BrowserToolbarIntegration(
 
         if (isCustomTab == false) {
             toolbar.addNavigationAction(eraseAction)
-            if (!inTesting) {
-                setUrlBackground()
-            }
+            setUrlBackground()
         }
     }
 
@@ -218,16 +252,17 @@ class BrowserToolbarIntegration(
     }
 
     private fun setBrowserActionButtons() {
-        tabsCounterScope = store.flowScoped { flow ->
-            flow.distinctUntilChangedBy { state -> state.tabs.size > 1 }
-                .collect { state ->
-                    if (state.tabs.size > 1) {
-                        toolbar.addBrowserAction(tabsAction)
-                    } else {
-                        toolbar.removeBrowserAction(tabsAction)
+        tabsCounterScope =
+            store.flowScoped(coroutineScope = CoroutineScope(coroutineDispatcher + SupervisorJob())) { flow ->
+                flow.distinctUntilChangedBy { state -> state.tabs.size > 1 }
+                    .collect { state ->
+                        if (state.tabs.size > 1) {
+                            toolbar.addBrowserAction(tabsAction)
+                        } else {
+                            toolbar.removeBrowserAction(tabsAction)
+                        }
                     }
-                }
-        }
+            }
     }
 
     override fun start() {
@@ -254,50 +289,55 @@ class BrowserToolbarIntegration(
 
     @VisibleForTesting
     internal fun observeEraseCfr() {
-        eraseTabsCfrScope = fragment.components?.appStore?.flowScoped { flow ->
-            flow.mapNotNull { state -> state.showEraseTabsCfr }
-                .distinctUntilChanged()
-                .collect { showEraseCfr ->
-                    if (showEraseCfr) {
-                        val eraseActionView =
-                            toolbar.findViewById<LinearLayout>(R.id.mozac_browser_toolbar_navigation_actions)
-                                .children
-                                .last()
-                        CFRPopup(
-                            anchor = eraseActionView,
-                            properties = CFRPopupProperties(
-                                popupWidth = 256.dp,
-                                popupAlignment = CFRPopup.PopupAlignment.INDICATOR_CENTERED_IN_ANCHOR,
-                                popupBodyColors = listOf(
-                                    ContextCompat.getColor(
-                                        fragment.requireContext(),
-                                        R.color.cfr_pop_up_shape_end_color,
+        eraseTabsCfrScope =
+            fragment.components?.appStore?.flowScoped(
+                coroutineScope = CoroutineScope(
+                    coroutineDispatcher + SupervisorJob(),
+                ),
+            ) { flow ->
+                flow.mapNotNull { state -> state.showEraseTabsCfr }
+                    .distinctUntilChanged()
+                    .collect { showEraseCfr ->
+                        if (showEraseCfr) {
+                            val eraseActionView =
+                                toolbar.findViewById<LinearLayout>(toolbarR.id.mozac_browser_toolbar_navigation_actions)
+                                    .children
+                                    .last()
+                            CFRPopup(
+                                anchor = eraseActionView,
+                                properties = CFRPopupProperties(
+                                    popupWidth = 256.dp,
+                                    popupAlignment = CFRPopup.PopupAlignment.INDICATOR_CENTERED_IN_ANCHOR,
+                                    popupBodyColors = listOf(
+                                        ContextCompat.getColor(
+                                            fragment.requireContext(),
+                                            R.color.cfr_pop_up_shape_end_color,
+                                        ),
+                                        ContextCompat.getColor(
+                                            fragment.requireContext(),
+                                            R.color.cfr_pop_up_shape_start_color,
+                                        ),
                                     ),
-                                    ContextCompat.getColor(
+                                    dismissButtonColor = ContextCompat.getColor(
                                         fragment.requireContext(),
-                                        R.color.cfr_pop_up_shape_start_color,
+                                        cardViewR.color.cardview_light_background,
                                     ),
+                                    popupVerticalOffset = 0.dp,
                                 ),
-                                dismissButtonColor = ContextCompat.getColor(
-                                    fragment.requireContext(),
-                                    R.color.cardview_light_background,
-                                ),
-                                popupVerticalOffset = 0.dp,
-                            ),
-                            onDismiss = { onDismissEraseTabsCfr() },
-                            text = {
-                                Text(
-                                    style = focusTypography.cfrTextStyle,
-                                    text = fragment.getString(R.string.cfr_for_toolbar_delete_icon2),
-                                    color = colorResource(R.color.cfr_text_color),
-                                )
-                            },
-                        ).apply {
-                            show()
+                                onDismiss = { onDismissEraseTabsCfr() },
+                                text = {
+                                    Text(
+                                        style = focusTypography.cfrTextStyle,
+                                        text = fragment.getString(R.string.cfr_for_toolbar_delete_icon2),
+                                        color = colorResource(R.color.cfr_text_color),
+                                    )
+                                },
+                            ).apply {
+                                show()
+                            }
                         }
                     }
-                }
-        }
+            }
     }
 
     private fun onDismissEraseTabsCfr() {
@@ -306,111 +346,119 @@ class BrowserToolbarIntegration(
 
     @VisibleForTesting
     internal fun observeCookieBannerCfr() {
-        cookieBannerCfrScope = fragment.components?.appStore?.flowScoped { flow ->
-            flow.mapNotNull { state -> state.showCookieBannerCfr }
-                .distinctUntilChanged()
-                .collect { showCookieBannerCfr ->
-                    if (showCookieBannerCfr) {
-                        CFRPopup(
-                            anchor = toolbar.findViewById<AppCompatEditText>(R.id.mozac_browser_toolbar_background),
-                            properties = CFRPopupProperties(
-                                popupWidth = 256.dp,
-                                popupAlignment = CFRPopup.PopupAlignment.BODY_TO_ANCHOR_START,
-                                popupBodyColors = listOf(
-                                    ContextCompat.getColor(
-                                        fragment.requireContext(),
-                                        R.color.cfr_pop_up_shape_end_color,
-                                    ),
-                                    ContextCompat.getColor(
-                                        fragment.requireContext(),
-                                        R.color.cfr_pop_up_shape_start_color,
-                                    ),
+        cookieBannerCfrScope =
+            fragment.components?.appStore?.flowScoped(
+                coroutineScope = CoroutineScope(coroutineDispatcher + SupervisorJob()),
+            ) { flow ->
+                flow.mapNotNull { state -> state.showCookieBannerCfr }
+                    .distinctUntilChanged()
+                    .collect { showCookieBannerCfr ->
+                        if (showCookieBannerCfr) {
+                            CFRPopup(
+                                anchor = toolbar.findViewById<AppCompatEditText>(
+                                    toolbarR.id.mozac_browser_toolbar_background,
                                 ),
-                                dismissButtonColor = ContextCompat.getColor(
-                                    fragment.requireContext(),
-                                    R.color.cardview_light_background,
-                                ),
-                                popupVerticalOffset = 0.dp,
-                                indicatorArrowStartOffset = 10.dp,
-                            ),
-                            onDismiss = { onDismissCookieBannerCfr() },
-                            text = {
-                                val textCookieBannerCfr = stringResource(
-                                    id = R.string.cfr_cookie_banner,
-                                    LocalContext.current.getString(R.string.onboarding_short_app_name),
-                                    LocalContext.current.getString(R.string.cfr_cookie_banner_link),
-                                )
-                                ClickableSubstringLink(
-                                    text = textCookieBannerCfr,
-                                    style = focusTypography.cfrCookieBannerTextStyle,
-                                    linkTextDecoration = TextDecoration.Underline,
-                                    clickableStartIndex = textCookieBannerCfr.indexOf(
-                                        LocalContext.current.getString(
-                                            R.string.cfr_cookie_banner_link,
+                                properties = CFRPopupProperties(
+                                    popupWidth = 256.dp,
+                                    popupAlignment = CFRPopup.PopupAlignment.BODY_TO_ANCHOR_START,
+                                    popupBodyColors = listOf(
+                                        ContextCompat.getColor(
+                                            fragment.requireContext(),
+                                            R.color.cfr_pop_up_shape_end_color,
+                                        ),
+                                        ContextCompat.getColor(
+                                            fragment.requireContext(),
+                                            R.color.cfr_pop_up_shape_start_color,
                                         ),
                                     ),
-                                    clickableEndIndex = textCookieBannerCfr.length,
-                                    onClick = {
-                                        fragment.requireComponents.appStore.dispatch(
-                                            AppAction.OpenSettings(Screen.Settings.Page.CookieBanner),
-                                        )
-                                        onDismissCookieBannerCfr()
-                                    },
-                                )
-                            },
-                        ).apply {
-                            show()
-                            stopObserverCookieBannerCfrChanges()
+                                    dismissButtonColor = ContextCompat.getColor(
+                                        fragment.requireContext(),
+                                        cardViewR.color.cardview_light_background,
+                                    ),
+                                    popupVerticalOffset = 0.dp,
+                                    indicatorArrowStartOffset = 10.dp,
+                                ),
+                                onDismiss = { onDismissCookieBannerCfr() },
+                                text = {
+                                    val appName = stringResource(R.string.onboarding_short_app_name)
+                                    val linkText = stringResource(R.string.cfr_cookie_banner_link)
+                                    val textCookieBannerCfr = stringResource(
+                                        id = R.string.cfr_cookie_banner,
+                                        appName,
+                                        linkText,
+                                    )
+                                    ClickableSubstringLink(
+                                        text = textCookieBannerCfr,
+                                        style = focusTypography.cfrCookieBannerTextStyle,
+                                        linkTextDecoration = TextDecoration.Underline,
+                                        clickableStartIndex = textCookieBannerCfr.indexOf(linkText),
+                                        clickableEndIndex = textCookieBannerCfr.length,
+                                        onClick = {
+                                            fragment.requireComponents.appStore.dispatch(
+                                                AppAction.OpenSettings(Screen.Settings.Page.CookieBanner),
+                                            )
+                                            onDismissCookieBannerCfr()
+                                        },
+                                    )
+                                },
+                            ).apply {
+                                show()
+                                stopObserverCookieBannerCfrChanges()
+                            }
                         }
                     }
-                }
-        }
+            }
     }
 
     @VisibleForTesting
     internal fun observeTrackingProtectionCfr() {
-        trackingProtectionCfrScope = fragment.components?.appStore?.flowScoped { flow ->
-            flow.mapNotNull { state -> state.showTrackingProtectionCfrForTab }
-                .distinctUntilChanged()
-                .collect { showTrackingProtectionCfrForTab ->
-                    if (showTrackingProtectionCfrForTab[store.state.selectedTabId] == true) {
-                        CFRPopup(
-                            anchor = toolbar.findViewById(
-                                R.id.mozac_browser_toolbar_tracking_protection_indicator,
-                            ),
-                            properties = CFRPopupProperties(
-                                popupWidth = 256.dp,
-                                popupAlignment = CFRPopup.PopupAlignment.INDICATOR_CENTERED_IN_ANCHOR,
-                                popupBodyColors = listOf(
-                                    ContextCompat.getColor(
-                                        fragment.requireContext(),
-                                        R.color.cfr_pop_up_shape_end_color,
-                                    ),
-                                    ContextCompat.getColor(
-                                        fragment.requireContext(),
-                                        R.color.cfr_pop_up_shape_start_color,
-                                    ),
+        trackingProtectionCfrScope =
+            fragment.components?.appStore?.flowScoped(
+                coroutineScope = CoroutineScope(
+                    coroutineDispatcher + SupervisorJob(),
+                ),
+            ) { flow ->
+                flow.mapNotNull { state -> state.showTrackingProtectionCfrForTab }
+                    .distinctUntilChanged()
+                    .collect { showTrackingProtectionCfrForTab ->
+                        if (showTrackingProtectionCfrForTab[store.state.selectedTabId] == true) {
+                            CFRPopup(
+                                anchor = toolbar.findViewById(
+                                    toolbarR.id.mozac_browser_toolbar_tracking_protection_indicator,
                                 ),
-                                dismissButtonColor = ContextCompat.getColor(
-                                    fragment.requireContext(),
-                                    R.color.cardview_light_background,
+                                properties = CFRPopupProperties(
+                                    popupWidth = 256.dp,
+                                    popupAlignment = CFRPopup.PopupAlignment.INDICATOR_CENTERED_IN_ANCHOR,
+                                    popupBodyColors = listOf(
+                                        ContextCompat.getColor(
+                                            fragment.requireContext(),
+                                            R.color.cfr_pop_up_shape_end_color,
+                                        ),
+                                        ContextCompat.getColor(
+                                            fragment.requireContext(),
+                                            R.color.cfr_pop_up_shape_start_color,
+                                        ),
+                                    ),
+                                    dismissButtonColor = ContextCompat.getColor(
+                                        fragment.requireContext(),
+                                        cardViewR.color.cardview_light_background,
+                                    ),
+                                    popupVerticalOffset = 0.dp,
                                 ),
-                                popupVerticalOffset = 0.dp,
-                            ),
-                            onDismiss = { onDismissTrackingProtectionCfr() },
-                            text = {
-                                Text(
-                                    style = focusTypography.cfrTextStyle,
-                                    text = fragment.getString(R.string.cfr_for_toolbar_shield_icon2),
-                                    color = colorResource(R.color.cfr_text_color),
-                                )
-                            },
-                        ).apply {
-                            show()
+                                onDismiss = { onDismissTrackingProtectionCfr() },
+                                text = {
+                                    Text(
+                                        style = focusTypography.cfrTextStyle,
+                                        text = fragment.getString(R.string.cfr_for_toolbar_shield_icon2),
+                                        color = colorResource(R.color.cfr_text_color),
+                                    )
+                                },
+                            ).apply {
+                                show()
+                            }
                         }
                     }
-                }
-        }
+            }
     }
 
     private fun onDismissCookieBannerCfr() {
@@ -439,21 +487,30 @@ class BrowserToolbarIntegration(
 
     @VisibleForTesting
     internal fun observerSecurityIndicatorChanges() {
-        securityIndicatorScope = store.flowScoped { flow ->
-            flow.mapNotNull { state -> state.findCustomTabOrSelectedTab(customTabId) }
-                .distinctUntilChangedBy { tab -> tab.content.securityInfo }
-                .collect {
-                    val secure = it.content.securityInfo.secure
-                    val url = it.content.url
-                    if (secure && Indicators.SECURITY in toolbar.display.indicators) {
-                        addTrackingProtectionIndicator()
-                    } else if (!secure && Indicators.SECURITY !in toolbar.display.indicators &&
-                        !url.trim().startsWith("about:")
-                    ) {
-                        addSecurityIndicator()
+        securityIndicatorScope =
+            store.flowScoped(coroutineScope = CoroutineScope(coroutineDispatcher + SupervisorJob())) { flow ->
+                flow.mapNotNull { state -> state.findCustomTabOrSelectedTab(customTabId) }
+                    .distinctUntilChangedBy { tab -> tab.content.securityInfo }
+                    .collect {
+                        val secure = it.content.securityInfo.secure
+                        val url = it.content.url.trim()
+                        when {
+                            secure && Indicators.SECURITY in toolbar.display.indicators -> {
+                                addTrackingProtectionIndicator()
+                            }
+
+                            secure || Indicators.SECURITY in toolbar.display.indicators || url.startsWith(
+                                "about:",
+                            ) -> {
+                                // do nothing
+                            }
+
+                            else -> {
+                                addSecurityIndicator()
+                            }
+                        }
                     }
-                }
-        }
+            }
     }
 
     override fun stop() {
