@@ -7,7 +7,10 @@ package org.mozilla.focus.downloads
 import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.concept.fetch.Client
 import mozilla.components.feature.downloads.AbstractFetchDownloadService
+import mozilla.components.feature.downloads.DefaultPackageNameProvider
+import mozilla.components.feature.downloads.DownloadEstimator
 import mozilla.components.feature.downloads.FileSizeFormatter
+import mozilla.components.feature.downloads.PackageNameProvider
 import mozilla.components.support.base.android.NotificationsDelegate
 import org.mozilla.focus.ext.components
 
@@ -16,4 +19,6 @@ class DownloadService : AbstractFetchDownloadService() {
     override val store: BrowserStore by lazy { components.store }
     override val notificationsDelegate: NotificationsDelegate by lazy { components.notificationsDelegate }
     override val fileSizeFormatter: FileSizeFormatter by lazy { components.fileSizeFormatter }
+    override val downloadEstimator: DownloadEstimator by lazy { components.downloadEstimator }
+    override val packageNameProvider: PackageNameProvider by lazy { DefaultPackageNameProvider(applicationContext) }
 }
