@@ -4,7 +4,6 @@
 
 package org.mozilla.focus.fragment
 
-import android.annotation.SuppressLint
 import android.content.ClipData
 import android.content.ClipboardManager
 import android.content.Context
@@ -20,12 +19,12 @@ import android.webkit.MimeTypeMap
 import android.widget.Toast
 import androidx.activity.result.ActivityResultLauncher
 import androidx.activity.result.contract.ActivityResultContracts
-import androidx.core.content.pm.ShortcutManagerCompat
+import androidx.annotation.VisibleForTesting
 import androidx.core.net.toUri
 import androidx.core.view.isVisible
+import androidx.fragment.app.Fragment
 import androidx.lifecycle.Lifecycle
 import androidx.lifecycle.lifecycleScope
-import androidx.preference.PreferenceManager
 import com.google.android.material.snackbar.Snackbar
 import kotlinx.coroutines.CoroutineScope
 import kotlinx.coroutines.DelicateCoroutinesApi
@@ -39,6 +38,8 @@ import kotlinx.coroutines.plus
 import mozilla.components.browser.state.selector.findTabOrCustomTab
 import mozilla.components.browser.state.selector.privateTabs
 import mozilla.components.browser.state.state.CustomTabConfig
+import mozilla.components.browser.state.state.CustomTabSessionState
+import mozilla.components.browser.state.state.ExternalAppType
 import mozilla.components.browser.state.state.SessionState
 import mozilla.components.browser.state.state.content.DownloadState
 import mozilla.components.browser.state.state.createTab
@@ -48,25 +49,29 @@ import mozilla.components.feature.contextmenu.ContextMenuFeature
 import mozilla.components.feature.downloads.AbstractFetchDownloadService
 import mozilla.components.feature.downloads.DownloadsFeature
 import mozilla.components.feature.downloads.manager.FetchDownloadManager
-import mozilla.components.feature.downloads.temporary.ShareDownloadFeature
+import mozilla.components.feature.downloads.temporary.ShareResourceFeature
 import mozilla.components.feature.media.fullscreen.MediaSessionFullscreenFeature
 import mozilla.components.feature.prompts.PromptFeature
+import mozilla.components.feature.prompts.file.AndroidPhotoPicker
 import mozilla.components.feature.session.PictureInPictureFeature
 import mozilla.components.feature.session.SessionFeature
 import mozilla.components.feature.sitepermissions.SitePermissionsFeature
 import mozilla.components.feature.tabs.WindowFeature
+import mozilla.components.feature.toolbar.ToolbarFeature
 import mozilla.components.feature.top.sites.TopSitesConfig
 import mozilla.components.feature.top.sites.TopSitesFeature
 import mozilla.components.lib.crash.Crash
 import mozilla.components.lib.state.ext.flowScoped
-import mozilla.components.service.glean.private.NoExtras
 import mozilla.components.support.base.feature.UserInteractionHandler
 import mozilla.components.support.base.feature.ViewBoundFeatureWrapper
+import mozilla.components.support.ktx.android.content.createChooserExcludingCurrentApp
+import mozilla.components.support.ktx.android.view.ImeInsetsSynchronizer
 import mozilla.components.support.ktx.android.view.exitImmersiveMode
 import mozilla.components.support.locale.ActivityContextWrapper
 import mozilla.components.support.utils.Browsers
-import mozilla.components.support.utils.StatusBarUtils
 import mozilla.components.support.utils.ext.requestInPlacePermissions
+import mozilla.telemetry.glean.private.NoExtras
+import org.mozilla.focus.Components
 import org.mozilla.focus.GleanMetrics.Browser
 import org.mozilla.focus.GleanMetrics.CookieBanner
 import org.mozilla.focus.GleanMetrics.Downloads
@@ -74,7 +79,7 @@ import org.mozilla.focus.GleanMetrics.OpenWith
 import org.mozilla.focus.GleanMetrics.TabCount
 import org.mozilla.focus.GleanMetrics.TrackingProtection
 import org.mozilla.focus.R
-import org.mozilla.focus.activity.InstallFirefoxActivity
+import org.mozilla.focus.activity.FirefoxInstallationHelper
 import org.mozilla.focus.activity.MainActivity
 import org.mozilla.focus.browser.integration.BrowserMenuController
 import org.mozilla.focus.browser.integration.BrowserToolbarIntegration
@@ -95,6 +100,7 @@ import org.mozilla.focus.ext.components
 import org.mozilla.focus.ext.disableDynamicBehavior
 import org.mozilla.focus.ext.enableDynamicBehavior
 import org.mozilla.focus.ext.ifCustomTab
+import org.mozilla.focus.ext.isAccessibilityEnabled
 import org.mozilla.focus.ext.isCustomTab
 import org.mozilla.focus.ext.requireComponents
 import org.mozilla.focus.ext.settings
@@ -106,12 +112,12 @@ import org.mozilla.focus.session.ui.TabsPopup
 import org.mozilla.focus.settings.permissions.permissionoptions.SitePermissionOptionsStorage
 import org.mozilla.focus.settings.privacy.ConnectionDetailsPanel
 import org.mozilla.focus.settings.privacy.TrackingProtectionPanel
+import org.mozilla.focus.shortcut.HomeScreen
 import org.mozilla.focus.state.AppAction
 import org.mozilla.focus.topsites.DefaultTopSitesStorage.Companion.TOP_SITES_MAX_LIMIT
 import org.mozilla.focus.topsites.DefaultTopSitesView
 import org.mozilla.focus.utils.FocusSnackbar
 import org.mozilla.focus.utils.FocusSnackbarDelegate
-import org.mozilla.focus.utils.IntentUtils
 import org.mozilla.focus.utils.ViewUtils
 import java.net.URLEncoder
 
@@ -135,7 +141,7 @@ class BrowserFragment :
     private val promptFeature = ViewBoundFeatureWrapper<PromptFeature>()
     private val contextMenuFeature = ViewBoundFeatureWrapper<ContextMenuFeature>()
     private val downloadsFeature = ViewBoundFeatureWrapper<DownloadsFeature>()
-    private val shareDownloadFeature = ViewBoundFeatureWrapper<ShareDownloadFeature>()
+    private val shareResourceFeature = ViewBoundFeatureWrapper<ShareResourceFeature>()
     private val windowFeature = ViewBoundFeatureWrapper<WindowFeature>()
     private val appLinksFeature = ViewBoundFeatureWrapper<AppLinksFeature>()
     private val topSitesFeature = ViewBoundFeatureWrapper<TopSitesFeature>()
@@ -167,6 +173,28 @@ class BrowserFragment :
             // Workaround for tab not existing temporarily.
             ?: createTab("about:blank")
 
+    // Registers a photo picker activity launcher in single-select mode.
+    private val singleMediaPicker =
+        AndroidPhotoPicker.singleMediaPicker(
+            { getFragment() },
+            { getPromptFeature() },
+        )
+
+    // Registers a photo picker activity launcher in multi-select mode.
+    private val multipleMediaPicker =
+        AndroidPhotoPicker.multipleMediaPicker(
+            { getFragment() },
+            { getPromptFeature() },
+        )
+
+    private fun getFragment(): Fragment {
+        return this
+    }
+
+    private fun getPromptFeature(): PromptFeature? {
+        return promptFeature.get()
+    }
+
     override fun onCreate(savedInstanceState: Bundle?) {
         super.onCreate(savedInstanceState)
         requestPermissionLauncher =
@@ -188,6 +216,8 @@ class BrowserFragment :
                     grandResults.toIntArray(),
                 )
             }
+
+        HomeScreen.checkIfPinningSupported(requireContext(), lifecycleScope)
     }
 
     /**
@@ -233,7 +263,6 @@ class BrowserFragment :
         }
     }
 
-    @Suppress("LongMethod", "ComplexMethod")
     override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
         _binding = FragmentBrowserBinding.inflate(inflater, container, false)
 
@@ -245,11 +274,27 @@ class BrowserFragment :
         return binding.root
     }
 
-    @SuppressLint("VisibleForTests")
-    @Suppress("ComplexCondition", "LongMethod")
     override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
         val components = requireComponents
 
+        initializeFindInPageFeature(view, components)
+        initializeFullScreenIntegrationFeature(view, components)
+        initializePictureInPictureFeature(components)
+        initializeContextMenuFeature(view, components)
+        initializeSessionFeature(view, components)
+        initializePromptFeature(view, components)
+        initializeDownloadsFeature(view, components)
+        initializeShareResourceFeature(view, components)
+        initializeAppLinksFeature(view, components)
+        initializeTopSitesFeature(view, components)
+        customizeToolbar()
+        initializeUiBasedOnTabType(view, components)
+        initializeMediaSessionFullscreenFeature(view, components)
+        initializeSitePermissionsFeature(view)
+        setupImeInsets(view)
+    }
+
+    private fun initializeFindInPageFeature(view: View, components: Components) {
         findInPageIntegration.set(
             FindInPageIntegration(
                 components.store,
@@ -260,30 +305,34 @@ class BrowserFragment :
             this,
             view,
         )
+    }
 
+    private fun initializeFullScreenIntegrationFeature(view: View, components: Components) {
         fullScreenIntegration.set(
             FullScreenIntegration(
                 requireActivity(),
                 components.store,
                 tab.id,
                 components.sessionUseCases,
-                requireContext().settings,
                 binding.browserToolbar,
-                binding.statusBarBackground,
                 binding.engineView,
-                parentFragmentManager,
+                isAccessibilityEnabled = { requireContext().isAccessibilityEnabled() },
             ),
             this,
             view,
         )
+    }
 
+    private fun initializePictureInPictureFeature(components: Components) {
         pictureInPictureFeature = PictureInPictureFeature(
             store = components.store,
             activity = requireActivity(),
             crashReporting = components.crashReporter,
             tabId = tabId,
         )
+    }
 
+    private fun initializeContextMenuFeature(view: View, components: Components) {
         contextMenuFeature.set(
             ContextMenuFeature(
                 parentFragmentManager,
@@ -305,7 +354,9 @@ class BrowserFragment :
             this,
             view,
         )
+    }
 
+    private fun initializeSessionFeature(view: View, components: Components) {
         sessionFeature.set(
             SessionFeature(
                 components.store,
@@ -317,7 +368,9 @@ class BrowserFragment :
             this,
             view,
         )
+    }
 
+    private fun initializePromptFeature(view: View, components: Components) {
         promptFeature.set(
             PromptFeature(
                 fragment = this,
@@ -339,17 +392,29 @@ class BrowserFragment :
                         )
                     }
                 },
+                androidPhotoPicker = AndroidPhotoPicker(
+                    requireContext(),
+                    singleMediaPicker,
+                    multipleMediaPicker,
+                ),
             ),
             this,
             view,
         )
+    }
 
+    private fun initializeDownloadsFeature(view: View, components: Components) {
         downloadsFeature.set(
             DownloadsFeature(
                 requireContext().applicationContext,
                 components.store,
                 components.downloadsUseCases,
                 fragmentManager = childFragmentManager,
+                promptsStyling = DownloadsFeature.PromptsStyling(
+                    gravity = Gravity.CENTER,
+                    positiveButtonBackgroundColor = R.color.contrastColor,
+                    positiveButtonTextColor = R.color.colorPrimary,
+                ),
                 tabId = tabId,
                 downloadManager = FetchDownloadManager(
                     requireContext().applicationContext,
@@ -377,9 +442,11 @@ class BrowserFragment :
             this,
             view,
         )
+    }
 
-        shareDownloadFeature.set(
-            ShareDownloadFeature(
+    private fun initializeShareResourceFeature(view: View, components: Components) {
+        shareResourceFeature.set(
+            ShareResourceFeature(
                 context = requireContext().applicationContext,
                 httpClient = components.client,
                 store = components.store,
@@ -388,7 +455,9 @@ class BrowserFragment :
             this,
             view,
         )
+    }
 
+    private fun initializeAppLinksFeature(view: View, components: Components) {
         appLinksFeature.set(
             feature = AppLinksFeature(
                 requireContext(),
@@ -410,11 +479,13 @@ class BrowserFragment :
             owner = this,
             view = view,
         )
+    }
 
+    private fun initializeTopSitesFeature(view: View, components: Components) {
         topSitesFeature.set(
             feature = TopSitesFeature(
                 view = DefaultTopSitesView(requireComponents.appStore),
-                storage = requireComponents.topSitesStorage,
+                storage = components.topSitesStorage,
                 config = {
                     TopSitesConfig(
                         totalSites = TOP_SITES_MAX_LIMIT,
@@ -426,19 +497,23 @@ class BrowserFragment :
             owner = this,
             view = view,
         )
+    }
 
-        customizeToolbar()
-
+    private fun initializeUiBasedOnTabType(view: View, components: Components) {
         val customTabConfig = tab.ifCustomTab()?.config
         if (customTabConfig != null) {
-            initialiseCustomTabUi(customTabConfig)
+            initialiseCustomTabUi(
+                customTabConfig,
+                requireContext().isAccessibilityEnabled(),
+            )
 
             // TODO Add custom tabs window feature support
             // We to add support for Custom Tabs here, however in order to send the window request
             // back to us through the intent system, we need to register a unique schema that we
             // can handle. For example, Fenix Nighlyt does this today with `fenix-nightly://`.
+            // See https://bugzilla.mozilla.org/show_bug.cgi?id=1804556
         } else {
-            initialiseNormalBrowserUi()
+            initialiseNormalBrowserUi(requireContext().isAccessibilityEnabled())
 
             windowFeature.set(
                 feature = WindowFeature(
@@ -449,22 +524,26 @@ class BrowserFragment :
                 view = view,
             )
         }
+    }
 
-        // Feature that handles MediaSession state changes
+    private fun initializeMediaSessionFullscreenFeature(view: View, components: Components) {
         fullScreenMediaSessionFeature.set(
-            feature = MediaSessionFullscreenFeature(requireActivity(), requireComponents.store, tryGetCustomTabId()),
+            feature = MediaSessionFullscreenFeature(requireActivity(), components.store, tryGetCustomTabId()),
             owner = this,
             view = view,
         )
-
-        setSitePermissions(view)
     }
 
-    private fun setSitePermissions(rootView: View) {
+    private fun initializeSitePermissionsFeature(rootView: View) {
         sitePermissionsFeature.set(
             feature = SitePermissionsFeature(
                 context = requireContext(),
                 fragmentManager = parentFragmentManager,
+                promptsStyling = SitePermissionsFeature.PromptsStyling(
+                    gravity = Gravity.CENTER,
+                    positiveButtonBackgroundColor = R.color.contrastColor,
+                    positiveButtonTextColor = R.color.colorPrimary,
+                ),
                 onNeedToRequestPermissions = { permissions ->
                     if (SitePermissionOptionsStorage(requireContext()).isSitePermissionNotBlocked(permissions)) {
                         requestPermissionLauncher.launch(permissions)
@@ -488,6 +567,25 @@ class BrowserFragment :
         }
     }
 
+    private fun setupImeInsets(view: View) {
+        ImeInsetsSynchronizer.setup(
+            targetView = view,
+            synchronizeViewWithIME = false,
+            onIMEAnimationStarted = { isKeyboardShowingUp, _ ->
+                if (!isKeyboardShowingUp) {
+                    (view.layoutParams as? ViewGroup.MarginLayoutParams)?.bottomMargin = 0
+                    view.requestLayout()
+                }
+            },
+            onIMEAnimationFinished = { isKeyboardShowingUp, keyboardHeight ->
+                if (isKeyboardShowingUp || keyboardHeight == 0) {
+                    (view.layoutParams as? ViewGroup.MarginLayoutParams)?.bottomMargin = keyboardHeight
+                    view.requestLayout()
+                }
+            },
+        )
+    }
+
     override fun onAccessibilityStateChanged(enabled: Boolean) {
         when (enabled) {
             // using _binding, because this might be called before onCreateView.
@@ -529,7 +627,7 @@ class BrowserFragment :
             requireComponents.topSitesUseCases,
             tabId,
             ::shareCurrentUrl,
-            ::setShouldRequestDesktop,
+            ::toggleDesktopSite,
             ::showAddToHomescreenDialog,
             ::showFindInPageBar,
             ::openSelectBrowser,
@@ -537,19 +635,23 @@ class BrowserFragment :
             ::showShortcutAddedSnackBar,
         )
 
-        if (tab.ifCustomTab()?.config == null) {
+        val customTabSessionState = tab.ifCustomTab()
+        if (customTabSessionState?.config == null) {
             val browserMenu = DefaultBrowserMenu(
                 context = requireContext(),
                 appStore = requireComponents.appStore,
                 store = requireComponents.store,
-                isPinningSupported = ShortcutManagerCompat.isRequestPinShortcutSupported(
-                    requireContext(),
-                ),
                 onItemTapped = { controller.handleMenuInteraction(it) },
             )
             binding.browserToolbar.display.menuBuilder = browserMenu.menuBuilder
         }
 
+        val renderStyle = if (tab.isCustomTab()) {
+            ToolbarFeature.RenderStyle.ColoredDomain
+        } else {
+            ToolbarFeature.RenderStyle.ColoredUrl
+        }
+
         toolbarIntegration.set(
             BrowserToolbarIntegration(
                 requireComponents.store,
@@ -561,29 +663,38 @@ class BrowserFragment :
                 sessionUseCases = requireComponents.sessionUseCases,
                 onUrlLongClicked = ::onUrlLongClicked,
                 eraseActionListener = { erase(shouldEraseAllTabs = true) },
+                isOnboardingTab = isOnboardingTab(customTabSessionState),
                 tabCounterListener = ::tabCounterListener,
+                renderStyle = renderStyle,
             ),
             owner = this,
             view = binding.browserToolbar,
         )
     }
 
+    @VisibleForTesting
+    internal fun isOnboardingTab(sessionState: CustomTabSessionState?) =
+        sessionState?.config?.externalAppType == ExternalAppType.ONBOARDING_CUSTOM_TAB
+
     private fun showShortcutAddedSnackBar() {
         FocusSnackbar.make(requireView())
             .setText(requireContext().getString(R.string.snackbar_added_to_shortcuts))
             .show()
     }
 
-    private fun initialiseNormalBrowserUi() {
-        if (!requireContext().settings.isAccessibilityEnabled()) {
-            binding.browserToolbar.enableDynamicBehavior(requireContext(), binding.engineView)
-        } else {
+    private fun initialiseNormalBrowserUi(accessibilityEnabled: Boolean) {
+        if (accessibilityEnabled) {
             binding.browserToolbar.showAsFixed(requireContext(), binding.engineView)
+        } else {
+            binding.browserToolbar.enableDynamicBehavior(requireContext(), binding.engineView)
         }
     }
 
-    private fun initialiseCustomTabUi(customTabConfig: CustomTabConfig) {
-        if (customTabConfig.enableUrlbarHiding && !requireContext().settings.isAccessibilityEnabled()) {
+    private fun initialiseCustomTabUi(
+        customTabConfig: CustomTabConfig,
+        accessibilityEnabled: Boolean,
+    ) {
+        if (customTabConfig.enableUrlbarHiding && !accessibilityEnabled) {
             binding.browserToolbar.enableDynamicBehavior(requireContext(), binding.engineView)
         } else {
             binding.browserToolbar.showAsFixed(requireContext(), binding.engineView)
@@ -607,6 +718,7 @@ class BrowserFragment :
         requireActivity().exitImmersiveMode()
     }
 
+    @Suppress("OVERRIDE_DEPRECATION")
     override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
         promptFeature.withFeature { it.onActivityResult(requestCode, data, resultCode) }
     }
@@ -657,8 +769,11 @@ class BrowserFragment :
         binding.crashContainer.isVisible = false
     }
 
+    /**
+     * Returns true if the crash reporter fragment is visible.
+     */
     fun crashReporterIsVisible(): Boolean = requireActivity().supportFragmentManager.let {
-        it.findFragmentByTag(CrashReporterFragment.FRAGMENT_TAG)?.isVisible ?: false
+        it.findFragmentByTag(CrashReporterFragment.FRAGMENT_TAG)?.isVisible == true
     }
 
     private fun handleDownloadStopped(
@@ -714,7 +829,10 @@ class BrowserFragment :
         snackbar.setAction(getString(R.string.download_snackbar_open)) { context ->
             val opened = AbstractFetchDownloadService.openFile(
                 applicationContext = context.applicationContext,
-                download = state,
+                packageName = context.applicationContext.packageName,
+                downloadFileName = state.fileName,
+                downloadFilePath = state.filePath,
+                downloadContentType = state.contentType,
             )
 
             if (!opened) {
@@ -772,23 +890,10 @@ class BrowserFragment :
 
         updateEngineColorScheme()
 
-        // Hide status bar background if the parent activity can be casted to MainActivity
-        (requireActivity() as? MainActivity)?.hideStatusBarBackground()
-        StatusBarUtils.getStatusBarHeight(binding.statusBarBackground) { statusBarHeight ->
-            binding.statusBarBackground.layoutParams.height = statusBarHeight
-        }
-
         // Custom tab content should always be visible, even if the app is locked.
         if (tab.isCustomTab()) {
             view?.isVisible = true
         }
-
-        context?.settings?.openLinksInExternalApp?.let { openLinksInExternalApp ->
-            val isCustomTab = tab.isCustomTab()
-            components?.appLinksInterceptor?.updateLaunchInApp {
-                openLinksInExternalApp || isCustomTab
-            }
-        }
     }
 
     private fun updateEngineColorScheme() {
@@ -804,11 +909,12 @@ class BrowserFragment :
         tabsPopup?.dismiss()
         trackingProtectionPanel?.hide()
         siteNotSupportedSnackBarScope?.cancel()
+        requireComponents.sessionUseCases.exitFullscreen()
     }
 
     override fun onHomePressed() = pictureInPictureFeature?.onHomePressed() ?: false
 
-    @Suppress("ComplexMethod", "ReturnCount")
+    @Suppress("ReturnCount")
     override fun onBackPressed(): Boolean {
         if (findInPageIntegration.onBackPressed()) {
             return true
@@ -851,6 +957,15 @@ class BrowserFragment :
         return true
     }
 
+    /**
+     * Erases the current tab or all tabs.
+     *
+     * If [shouldEraseAllTabs] is true, all tabs will be removed and the user will be navigated
+     * to the home screen. Otherwise, only the current tab will be removed and the user will be
+     * navigated to the previously selected tab.
+     *
+     * @param shouldEraseAllTabs Whether to erase all tabs or just the current tab.
+     */
     fun erase(shouldEraseAllTabs: Boolean = false) {
         if (shouldEraseAllTabs) {
             requireComponents.appStore.dispatch(AppAction.NavigateUp(null))
@@ -871,10 +986,9 @@ class BrowserFragment :
             shareIntent.putExtra(Intent.EXTRA_SUBJECT, title)
         }
         startActivity(
-            IntentUtils.getIntentChooser(
+            shareIntent.createChooserExcludingCurrentApp(
                 context = requireContext(),
-                intent = shareIntent,
-                chooserTitle = getString(R.string.share_dialog_title),
+                title = getString(R.string.share_dialog_title),
             ),
         )
     }
@@ -927,7 +1041,7 @@ class BrowserFragment :
         val store = if (browsers.hasFirefoxBrandedBrowserInstalled) {
             null
         } else {
-            InstallFirefoxActivity.resolveAppStore(requireContext())
+            FirefoxInstallationHelper.resolveAppStore(requireContext())
         }
 
         val fragment = OpenWithFragment.newInstance(
@@ -935,8 +1049,7 @@ class BrowserFragment :
             tab.content.url,
             store,
         )
-        @Suppress("DEPRECATION")
-        fragment.show(requireFragmentManager(), OpenWithFragment.FRAGMENT_TAG)
+        fragment.show(getParentFragmentManager(), OpenWithFragment.FRAGMENT_TAG)
 
         OpenWith.listDisplayed.record(OpenWith.ListDisplayedExtra(apps.size))
     }
@@ -946,17 +1059,18 @@ class BrowserFragment :
         requireActivity().finish()
     }
 
-    private fun setShouldRequestDesktop(enabled: Boolean) {
-        if (enabled) {
-            PreferenceManager.getDefaultSharedPreferences(requireContext()).edit()
-                .putBoolean(
-                    requireContext().getString(R.string.has_requested_desktop),
-                    true,
-                ).apply()
-        }
+    /**
+     * Toggles the "Request desktop site" setting for the current tab.
+     *
+     * @param enabled True to request the desktop site, false to request the mobile site.
+     */
+    private fun toggleDesktopSite(enabled: Boolean) {
         requireComponents.sessionUseCases.requestDesktopSite(enabled, tab.id)
     }
 
+    /**
+     * Show tracking protection panel.
+     */
     fun showTrackingProtectionPanel() {
         trackingProtectionPanel = TrackingProtectionPanel(
             context = requireContext(),
@@ -1051,6 +1165,11 @@ class BrowserFragment :
         null
     }
 
+    /**
+     * Handles a tab crash by showing the crash reporter.
+     *
+     * @param crash The crash that occurred.
+     */
     fun handleTabCrash(crash: Crash) {
         showCrashReporter(crash)
     }
@@ -1062,6 +1181,13 @@ class BrowserFragment :
 
         private const val REQUEST_KEY_DOWNLOAD_PERMISSIONS = "downloadFeature"
         private const val REQUEST_KEY_PROMPT_PERMISSIONS = "promptFeature"
+
+        /**
+         * Creates a new [BrowserFragment] instance for the given tab ID.
+         *
+         * @param tabId The ID of the tab to create the fragment for.
+         * @return A new [BrowserFragment] instance.
+         */
         fun createForTab(tabId: String): BrowserFragment {
             val fragment = BrowserFragment()
             fragment.arguments = Bundle().apply {
