@@ -1,6 +1,7 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
 package org.mozilla.focus.menu.browser
 
 import android.content.Context
@@ -20,13 +21,24 @@ import mozilla.components.feature.webcompat.reporter.WebCompatReporterFeature
 import org.mozilla.focus.R
 import org.mozilla.focus.menu.ToolbarMenu
 import org.mozilla.focus.theme.resolveAttribute
+import androidx.appcompat.R as appcompatR
+import mozilla.components.ui.icons.R as iconsR
 
+/**
+ * A menu designed for Custom Tabs.
+ *
+ * @param context The Android context.
+ * @param store The browser store, used to retrieve session information.
+ * @param currentTabId The ID of the currently selected tab.
+ * @param isOnboardingTab Whether the current tab is an onboarding tab, in which case some menu items will be hidden.
+ * @param onItemTapped A callback invoked when a menu item is tapped.
+ */
 class CustomTabMenu(
     private val context: Context,
     private val store: BrowserStore,
     private val currentTabId: String,
     private val isOnboardingTab: Boolean = false,
-    private val onItemTapped: (ToolbarMenu.Item) -> Unit = {},
+    private val onItemTapped: (ToolbarMenu.FocusMenuItem) -> Unit = {},
 ) : ToolbarMenu {
 
     private val selectedSession: CustomTabSessionState?
@@ -41,7 +53,7 @@ class CustomTabMenu(
 
     override val menuToolbar by lazy {
         val back = BrowserMenuItemToolbar.TwoStateButton(
-            primaryImageResource = R.drawable.mozac_ic_back_24,
+            primaryImageResource = iconsR.drawable.mozac_ic_back_24,
             primaryContentDescription = context.getString(R.string.content_description_back),
             primaryImageTintResource = context.theme.resolveAttribute(R.attr.primaryText),
             isInPrimaryState = {
@@ -55,7 +67,7 @@ class CustomTabMenu(
         }
 
         val forward = BrowserMenuItemToolbar.TwoStateButton(
-            primaryImageResource = R.drawable.mozac_ic_forward_24,
+            primaryImageResource = iconsR.drawable.mozac_ic_forward_24,
             primaryContentDescription = context.getString(R.string.content_description_forward),
             primaryImageTintResource = context.theme.resolveAttribute(R.attr.primaryText),
             isInPrimaryState = {
@@ -69,13 +81,13 @@ class CustomTabMenu(
         }
 
         val refresh = BrowserMenuItemToolbar.TwoStateButton(
-            primaryImageResource = R.drawable.mozac_ic_arrow_clockwise_24,
+            primaryImageResource = iconsR.drawable.mozac_ic_arrow_clockwise_24,
             primaryContentDescription = context.getString(R.string.content_description_reload),
             primaryImageTintResource = context.theme.resolveAttribute(R.attr.primaryText),
             isInPrimaryState = {
                 selectedSession?.content?.loading == false
             },
-            secondaryImageResource = R.drawable.mozac_ic_stop,
+            secondaryImageResource = iconsR.drawable.mozac_ic_stop,
             secondaryContentDescription = context.getString(R.string.content_description_stop),
             secondaryImageTintResource = context.theme.resolveAttribute(R.attr.primaryText),
             disableInSecondaryState = false,
@@ -93,13 +105,13 @@ class CustomTabMenu(
     private val menuItems by lazy {
         val findInPage = BrowserMenuImageText(
             label = context.getString(R.string.find_in_page),
-            imageResource = R.drawable.mozac_ic_search_24,
+            imageResource = iconsR.drawable.mozac_ic_search_24,
         ) {
             onItemTapped.invoke(ToolbarMenu.CustomTabItem.FindInPage)
         }
 
         val desktopMode = BrowserMenuImageSwitch(
-            imageResource = R.drawable.mozac_ic_device_desktop_24,
+            imageResource = iconsR.drawable.mozac_ic_device_desktop_24,
             label = context.getString(R.string.preference_performance_request_desktop_site2),
             initialState = {
                 selectedSession?.content?.desktopMode != false
@@ -115,7 +127,7 @@ class CustomTabMenu(
 
         val addToHomescreen = BrowserMenuImageText(
             label = context.getString(R.string.menu_add_to_home_screen),
-            imageResource = R.drawable.mozac_ic_add_to_homescreen_24,
+            imageResource = iconsR.drawable.mozac_ic_add_to_homescreen_24,
         ) {
             onItemTapped.invoke(ToolbarMenu.CustomTabItem.AddToHomeScreen)
         }
@@ -124,7 +136,7 @@ class CustomTabMenu(
             menuToolbar,
             BrowserMenuDivider(),
             findInPage,
-            desktopMode,
+            desktopMode.apply { visible = { selectedSession?.content?.isPdf == false } },
             reportSiteIssue,
             BrowserMenuDivider(),
             addToHomescreen,
@@ -154,7 +166,7 @@ class CustomTabMenu(
             ),
             textSize = CAPTION_TEXT_SIZE,
             textColorResource = context.theme.resolveAttribute(R.attr.secondaryText),
-            backgroundColorResource = context.theme.resolveAttribute(R.attr.colorPrimary),
+            backgroundColorResource = context.theme.resolveAttribute(appcompatR.attr.colorPrimary),
             textStyle = Typeface.NORMAL,
         )
 
