@@ -4,8 +4,8 @@
 
 package org.mozilla.focus.navigation
 
-import android.os.Build
 import android.os.Bundle
+import androidx.fragment.app.commit
 import org.mozilla.focus.R
 import org.mozilla.focus.activity.MainActivity
 import org.mozilla.focus.autocomplete.AutocompleteAddFragment
@@ -19,7 +19,7 @@ import org.mozilla.focus.exceptions.ExceptionsRemoveFragment
 import org.mozilla.focus.ext.components
 import org.mozilla.focus.ext.settings
 import org.mozilla.focus.fragment.BrowserFragment
-import org.mozilla.focus.fragment.FirstrunFragment
+import org.mozilla.focus.fragment.CrashListFragment
 import org.mozilla.focus.fragment.UrlInputFragment
 import org.mozilla.focus.fragment.about.AboutFragment
 import org.mozilla.focus.fragment.onboarding.OnboardingFirstFragment
@@ -27,6 +27,8 @@ import org.mozilla.focus.fragment.onboarding.OnboardingSecondFragment
 import org.mozilla.focus.fragment.onboarding.OnboardingStep
 import org.mozilla.focus.fragment.onboarding.OnboardingStorage
 import org.mozilla.focus.locale.screen.LanguageFragment
+import org.mozilla.focus.navigation.MainActivityNavigation.SessionWidgetPromoThresholds.FIFTH_CLEAR_SESSION_COUNT
+import org.mozilla.focus.navigation.MainActivityNavigation.SessionWidgetPromoThresholds.FIRST_CLEAR_SESSION_COUNT
 import org.mozilla.focus.nimbus.FocusNimbus
 import org.mozilla.focus.nimbus.Onboarding
 import org.mozilla.focus.searchwidget.SearchWidgetUtils
@@ -44,17 +46,14 @@ import org.mozilla.focus.settings.permissions.SitePermissionsFragment
 import org.mozilla.focus.settings.permissions.permissionoptions.SitePermission
 import org.mozilla.focus.settings.permissions.permissionoptions.SitePermissionOptionsFragment
 import org.mozilla.focus.settings.privacy.PrivacySecuritySettingsFragment
-import org.mozilla.focus.settings.privacy.studies.StudiesFragment
 import org.mozilla.focus.state.AppAction
 import org.mozilla.focus.state.Screen
 import org.mozilla.focus.utils.ViewUtils
-import kotlin.collections.forEach as withEach
 
 /**
  * Class performing the actual navigation in [MainActivity] by performing fragment transactions if
  * needed.
  */
-@Suppress("TooManyFunctions")
 class MainActivityNavigation(
     private val activity: MainActivity,
 ) {
@@ -81,7 +80,7 @@ class MainActivityNavigation(
         // and the session was removed while the app was in the background (e.g. via the
         // notification). In this case we do not want to show the content and remove the
         // browser fragment immediately.
-        val shouldAnimate = isShowingBrowser && browserFragment!!.isResumed
+        val shouldAnimate = isShowingBrowser && browserFragment.isResumed
 
         if (shouldAnimate) {
             transaction.setCustomAnimations(0, R.anim.erase_animation)
@@ -118,7 +117,6 @@ class MainActivityNavigation(
      * Display the widget promo at first data clearing action and if it wasn't added after 5th Focus session
      * or display branded snackbar when widget promo is not shown.
      */
-    @Suppress("MagicNumber")
     private fun showPromoteSearchWidgetDialogOrBrandedSnackbar() {
         val onboardingFeature = FocusNimbus.features.onboarding
         val onboardingConfig = onboardingFeature.value()
@@ -128,7 +126,7 @@ class MainActivityNavigation(
 
         if (shouldShowPromoteSearchWidgetDialog(onboardingConfig) &&
             (
-                clearBrowsingSessions == 0 || clearBrowsingSessions == 4
+                clearBrowsingSessions == FIRST_CLEAR_SESSION_COUNT || clearBrowsingSessions == FIFTH_CLEAR_SESSION_COUNT
                 )
         ) {
             onboardingFeature.recordExposure()
@@ -195,22 +193,19 @@ class MainActivityNavigation(
     }
 
     /**
-     * Show first run onBoarding.
+     * Show onBoarding.
      */
     fun firstRun() {
-        val onboardingFragment = if (activity.settings.isNewOnboardingEnable) {
-            FocusNimbus.features.onboarding.recordExposure()
-            val onBoardingStorage = OnboardingStorage(activity)
-            when (onBoardingStorage.getCurrentOnboardingStep()) {
-                OnboardingStep.ON_BOARDING_FIRST_SCREEN -> {
-                    OnboardingFirstFragment()
-                }
-                OnboardingStep.ON_BOARDING_SECOND_SCREEN -> {
-                    OnboardingSecondFragment()
-                }
+        FocusNimbus.features.onboarding.recordExposure()
+        val onBoardingStorage = OnboardingStorage(activity)
+        val onboardingFragment = when (onBoardingStorage.getCurrentOnboardingStep()) {
+            OnboardingStep.ON_BOARDING_FIRST_SCREEN -> {
+                OnboardingFirstFragment()
+            }
+
+            OnboardingStep.ON_BOARDING_SECOND_SCREEN -> {
+                OnboardingSecondFragment()
             }
-        } else {
-            FirstrunFragment.create()
         }
 
         activity.supportFragmentManager
@@ -226,6 +221,16 @@ class MainActivityNavigation(
             .commit()
     }
 
+    /**
+     * Show content of about:crashes
+     */
+    fun showCrashList() {
+        activity.supportFragmentManager
+            .beginTransaction()
+            .replace(R.id.container, CrashListFragment(true), CrashListFragment()::class.java.simpleName)
+            .commit()
+    }
+
     /**
      * Lock app.
      *
@@ -233,10 +238,6 @@ class MainActivityNavigation(
      * be redirected to a certain screen. It comes from the external intent.
      */
     fun lock(bundle: Bundle? = null) {
-        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
-            throw IllegalStateException("Trying to lock unsupported device")
-        }
-
         val fragmentManager = activity.supportFragmentManager
 
         val biometricAuthenticationFragment =
@@ -246,28 +247,21 @@ class MainActivityNavigation(
             return
         }
 
-        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N && activity.isInPictureInPictureMode) {
+        if (activity.isInPictureInPictureMode) {
             return
         }
 
-        val transaction = fragmentManager
-            .beginTransaction()
-
-        fragmentManager.fragments.withEach { fragment ->
-            transaction.remove(fragment)
-        }
-
-        fragmentManager
-            .beginTransaction()
-            .replace(
+        fragmentManager.commit {
+            fragmentManager.fragments.forEach { remove(it) }
+            replace(
                 R.id.container,
                 BiometricAuthenticationFragment.createWithDestinationData(bundle),
                 BiometricAuthenticationFragment.FRAGMENT_TAG,
             )
-            .commit()
+        }
     }
 
-    @Suppress("ComplexMethod")
+    @Suppress("CyclomaticComplexMethod")
     fun settings(page: Screen.Settings.Page) {
         val fragment = when (page) {
             Screen.Settings.Page.Start -> SettingsFragment()
@@ -279,7 +273,6 @@ class MainActivityNavigation(
             Screen.Settings.Page.PrivacyExceptions -> ExceptionsListFragment()
             Screen.Settings.Page.PrivacyExceptionsRemove -> ExceptionsRemoveFragment()
             Screen.Settings.Page.SitePermissions -> SitePermissionsFragment()
-            Screen.Settings.Page.Studies -> StudiesFragment()
             Screen.Settings.Page.SecretSettings -> SecretSettingsFragment()
             Screen.Settings.Page.SearchList -> InstalledSearchEnginesSettingsFragment()
             Screen.Settings.Page.SearchRemove -> RemoveSearchEnginesSettingsFragment()
@@ -292,6 +285,7 @@ class MainActivityNavigation(
             Screen.Settings.Page.Licenses -> AboutLibrariesFragment()
             Screen.Settings.Page.Locale -> LanguageFragment()
             Screen.Settings.Page.CookieBanner -> CookieBannerFragment()
+            Screen.Settings.Page.CrashList -> CrashListFragment()
         }
 
         val tag = "settings_" + fragment::class.java.simpleName
@@ -316,4 +310,9 @@ class MainActivityNavigation(
             )
             .commit()
     }
+
+    private object SessionWidgetPromoThresholds {
+        const val FIRST_CLEAR_SESSION_COUNT = 0
+        const val FIFTH_CLEAR_SESSION_COUNT = 4
+    }
 }
