@@ -91,6 +91,10 @@ this.test = class extends ExtensionAPI {
           return Preferences.get(prefs);
         },
 
+        /* Clears a given user preference. */
+        async clearUserPref(pref) {
+          Services.prefs.clearUserPref(pref);
+        },
         /* Gets link color for a given selector. */
         async getLinkColor(tabId, selector) {
           return getActorForTab(tabId, "TestSupport").sendQuery(
@@ -267,6 +271,26 @@ this.test = class extends ExtensionAPI {
             { handlingUserInput }
           );
         },
+
+        async getWebExtensionsSchemaPermissionNames(typeNames) {
+          const { Schemas } = ChromeUtils.importESModule(
+            "resource://gre/modules/Schemas.sys.mjs"
+          );
+          return Schemas.getPermissionNames(typeNames);
+        },
+
+        async teardownAlertsService() {
+          const alertsService = Cc["@mozilla.org/alerts-service;1"].getService(
+            Ci.nsIAlertsService
+          );
+          alertsService.teardown();
+        },
+
+        async notifyUserGestureActivation(tabId) {
+          return getActorForTab(tabId, "TestSupport").sendQuery(
+            "NotifyUserGestureActivation"
+          );
+        },
       },
     };
   }
