@@ -9,7 +9,11 @@ package org.mozilla.geckoview.test
 
 import androidx.test.ext.junit.runners.AndroidJUnit4
 import androidx.test.filters.MediumTest
-import org.hamcrest.Matchers.* // ktlint-disable no-wildcard-imports
+import org.hamcrest.Matchers.containsString
+import org.hamcrest.Matchers.equalTo
+import org.hamcrest.Matchers.not
+import org.hamcrest.Matchers.notNullValue
+import org.hamcrest.Matchers.startsWith
 import org.junit.Test
 import org.junit.runner.RunWith
 import org.mozilla.geckoview.ContentBlocking
@@ -18,6 +22,7 @@ import org.mozilla.geckoview.ContentBlocking.CookieBannerMode
 import org.mozilla.geckoview.ContentBlockingController
 import org.mozilla.geckoview.GeckoSession
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.AssertCalled
+
 @RunWith(AndroidJUnit4::class)
 @MediumTest
 class ContentBlockingControllerTest : BaseSessionTest() {
@@ -28,15 +33,17 @@ class ContentBlockingControllerTest : BaseSessionTest() {
 
         val google = contentBlocking.safeBrowsingProviders.first { it.name == "google" }
         val google4 = contentBlocking.safeBrowsingProviders.first { it.name == "google4" }
+        val google5 = contentBlocking.safeBrowsingProviders.first { it.name == "google5" }
 
         // Let's make sure the initial value of safeBrowsingProviders is correct
         assertThat(
             "Expected number of default providers",
             contentBlocking.safeBrowsingProviders.size,
-            equalTo(2),
+            equalTo(3),
         )
         assertThat("Google legacy provider is present", google, notNullValue())
         assertThat("Google provider is present", google4, notNullValue())
+        assertThat("Google5 provider is present", google5, notNullValue())
 
         // Checks that the default provider values make sense
         assertThat(
@@ -64,12 +71,26 @@ class ContentBlockingControllerTest : BaseSessionTest() {
             google4.dataSharingUrl,
             startsWith("https://safebrowsing.googleapis.com/"),
         )
+        assertThat(
+            "Default provider values are sensible",
+            google5.getHashUrl,
+            containsString("/safebrowsing5-dummy/"),
+        )
+        assertThat(
+            "Default provider values are sensible",
+            google5.updateUrl,
+            containsString("/safebrowsing5-dummy/"),
+        )
 
         // Checks that the pref value is also consistent with the runtime settings
         val originalPrefs = sessionRule.getPrefs(
             "browser.safebrowsing.provider.google4.updateURL",
             "browser.safebrowsing.provider.google4.gethashURL",
             "browser.safebrowsing.provider.google4.lists",
+            "browser.safebrowsing.provider.google5.updateURL",
+            "browser.safebrowsing.provider.google5.gethashURL",
+            "browser.safebrowsing.provider.google5.lists",
+            "browser.safebrowsing.provider.google5.enabled",
         )
 
         assertThat(
@@ -87,6 +108,26 @@ class ContentBlockingControllerTest : BaseSessionTest() {
             originalPrefs[2] as String,
             equalTo(google4.lists.joinToString(",")),
         )
+        assertThat(
+            "Initial prefs value is correct",
+            originalPrefs[3] as String,
+            equalTo(google5.updateUrl),
+        )
+        assertThat(
+            "Initial prefs value is correct",
+            originalPrefs[4] as String,
+            equalTo(google5.getHashUrl),
+        )
+        assertThat(
+            "Initial prefs value is correct",
+            originalPrefs[5] as String,
+            equalTo(google5.lists.joinToString(",")),
+        )
+        assertThat(
+            "Initial prefs value is correct",
+            originalPrefs[6] as Boolean,
+            equalTo(google5.enabled),
+        )
 
         // Makes sure we can override a default value
         val override = ContentBlocking.SafeBrowsingProvider
@@ -95,6 +136,13 @@ class ContentBlockingControllerTest : BaseSessionTest() {
             .getHashUrl("http://test-get-hash-url.com")
             .build()
 
+        val overrideV5 = ContentBlocking.SafeBrowsingProvider
+            .from(ContentBlocking.GOOGLE_SAFE_BROWSING_V5_PROVIDER)
+            .updateUrl("http://test-update-url-v5.com")
+            .getHashUrl("http://test-get-hash-url-v5.com")
+            .enabled(true)
+            .build()
+
         // ... and that we can add a custom provider
         val custom = ContentBlocking.SafeBrowsingProvider
             .withName("custom-provider")
@@ -114,6 +162,22 @@ class ContentBlockingControllerTest : BaseSessionTest() {
             equalTo("http://test-get-hash-url.com"),
         )
 
+        assertThat(
+            "Override value is correct",
+            overrideV5.updateUrl,
+            equalTo("http://test-update-url-v5.com"),
+        )
+        assertThat(
+            "Override value is correct",
+            overrideV5.getHashUrl,
+            equalTo("http://test-get-hash-url-v5.com"),
+        )
+        assertThat(
+            "Override value is correct",
+            overrideV5.enabled,
+            equalTo(true),
+        )
+
         assertThat(
             "Custom provider value is correct",
             custom.updateUrl,
@@ -130,11 +194,14 @@ class ContentBlockingControllerTest : BaseSessionTest() {
             equalTo(arrayOf("a", "b", "c")),
         )
 
-        contentBlocking.setSafeBrowsingProviders(override, custom)
+        contentBlocking.setSafeBrowsingProviders(override, overrideV5, custom)
 
         val prefs = sessionRule.getPrefs(
             "browser.safebrowsing.provider.google4.updateURL",
             "browser.safebrowsing.provider.google4.gethashURL",
+            "browser.safebrowsing.provider.google5.updateURL",
+            "browser.safebrowsing.provider.google5.gethashURL",
+            "browser.safebrowsing.provider.google5.enabled",
             "browser.safebrowsing.provider.custom-provider.updateURL",
             "browser.safebrowsing.provider.custom-provider.gethashURL",
             "browser.safebrowsing.provider.custom-provider.lists",
@@ -153,27 +220,46 @@ class ContentBlockingControllerTest : BaseSessionTest() {
         assertThat(
             "Pref value is set correctly",
             prefs[2] as String,
-            equalTo("http://test-custom-update-url.com"),
+            equalTo("http://test-update-url-v5.com"),
         )
         assertThat(
             "Pref value is set correctly",
             prefs[3] as String,
+            equalTo("http://test-get-hash-url-v5.com"),
+        )
+        assertThat(
+            "Pref value is set correctly",
+            prefs[4] as Boolean,
+            equalTo(true),
+        )
+        assertThat(
+            "Pref value is set correctly",
+            prefs[5] as String,
+            equalTo("http://test-custom-update-url.com"),
+        )
+        assertThat(
+            "Pref value is set correctly",
+            prefs[6] as String,
             equalTo("http://test-custom-get-hash-url.com"),
         )
         assertThat(
             "Pref value is set correctly",
-            prefs[4] as String,
+            prefs[7] as String,
             equalTo("a,b,c"),
         )
 
         // Restore defaults
-        contentBlocking.setSafeBrowsingProviders(google, google4)
+        contentBlocking.setSafeBrowsingProviders(google, google4, google5)
 
         // Checks that after restoring the providers the prefs get updated
         val restoredPrefs = sessionRule.getPrefs(
             "browser.safebrowsing.provider.google4.updateURL",
             "browser.safebrowsing.provider.google4.gethashURL",
             "browser.safebrowsing.provider.google4.lists",
+            "browser.safebrowsing.provider.google5.updateURL",
+            "browser.safebrowsing.provider.google5.gethashURL",
+            "browser.safebrowsing.provider.google5.lists",
+            "browser.safebrowsing.provider.google5.enabled",
         )
 
         assertThat(
@@ -191,6 +277,26 @@ class ContentBlockingControllerTest : BaseSessionTest() {
             restoredPrefs[2] as String,
             equalTo(originalPrefs[2]),
         )
+        assertThat(
+            "Restored prefs value is correct",
+            restoredPrefs[3] as String,
+            equalTo(originalPrefs[3]),
+        )
+        assertThat(
+            "Restored prefs value is correct",
+            restoredPrefs[4] as String,
+            equalTo(originalPrefs[4]),
+        )
+        assertThat(
+            "Restored prefs value is correct",
+            restoredPrefs[5] as String,
+            equalTo(originalPrefs[5]),
+        )
+        assertThat(
+            "Restored prefs value is correct",
+            restoredPrefs[6] as Boolean,
+            equalTo(originalPrefs[6]),
+        )
     }
 
     @Test
@@ -599,4 +705,123 @@ class ContentBlockingControllerTest : BaseSessionTest() {
             equalTo(true),
         )
     }
+
+    @Test
+    fun bounceTrackingProtectionModeSettings() {
+        // Check default value
+        val contentBlocking = sessionRule.runtime.settings.contentBlocking
+
+        assertThat(
+            "Expect correct default value which is off",
+            contentBlocking.bounceTrackingProtectionMode,
+            equalTo(ContentBlocking.BounceTrackingProtectionMode.BOUNCE_TRACKING_PROTECTION_MODE_DISABLED),
+        )
+
+        // Checks that the pref value is also consistent with the runtime settings
+        val originalPrefs = sessionRule.getPrefs(
+            "privacy.bounceTrackingProtection.mode",
+        )
+
+        assertThat(
+            "Initial value is correct",
+            originalPrefs[0] as Int,
+            equalTo(contentBlocking.bounceTrackingProtectionMode),
+        )
+
+        contentBlocking.bounceTrackingProtectionMode = ContentBlocking.BounceTrackingProtectionMode.BOUNCE_TRACKING_PROTECTION_MODE_ENABLED
+
+        val actualPrefs = sessionRule.getPrefs(
+            "privacy.bounceTrackingProtection.mode",
+        )
+
+        assertThat(
+            "The value is updated",
+            actualPrefs[0] as Int,
+            equalTo(contentBlocking.bounceTrackingProtectionMode),
+        )
+
+        // Set a new pref value, with a different setter method.
+        contentBlocking.setBounceTrackingProtectionMode(ContentBlocking.BounceTrackingProtectionMode.BOUNCE_TRACKING_PROTECTION_MODE_ENABLED_STANDBY)
+
+        val actualPrefs2 = sessionRule.getPrefs(
+            "privacy.bounceTrackingProtection.mode",
+        )
+
+        assertThat(
+            "The value is updated",
+            actualPrefs2[0] as Int,
+            equalTo(contentBlocking.bounceTrackingProtectionMode),
+        )
+
+        // Test that the getter returns the correct value.
+        assertThat(
+            "The getter returns the correct value",
+            contentBlocking.getBounceTrackingProtectionMode(),
+            equalTo(ContentBlocking.BounceTrackingProtectionMode.BOUNCE_TRACKING_PROTECTION_MODE_ENABLED_STANDBY),
+        )
+    }
+
+    @Test
+    fun allowListTrackingProtectionSettings() {
+        // Check default value
+        val contentBlocking = sessionRule.runtime.settings.contentBlocking
+
+        assertThat(
+            "Expect correct default for allowListBaselineTrackingProtection value which is true",
+            contentBlocking.allowListBaselineTrackingProtection,
+            equalTo(true),
+        )
+
+        assertThat(
+            "Expect correct default for allowListConvenienceTrackingProtection value which is true",
+            contentBlocking.allowListConvenienceTrackingProtection,
+            equalTo(true),
+        )
+
+        // Checks that the pref value is also consistent with the runtime settings
+        val originalAllowListBaseline = sessionRule.getPrefs(
+            "privacy.trackingprotection.allow_list.baseline.enabled",
+        )
+
+        val originalAllowListConvenience = sessionRule.getPrefs(
+            "privacy.trackingprotection.allow_list.convenience.enabled",
+        )
+
+        assertThat(
+            "Initial allow list baseline value is correct",
+            originalAllowListBaseline[0],
+            equalTo(contentBlocking.allowListBaselineTrackingProtection),
+        )
+
+        assertThat(
+            "Initial initial allow list convenience value is correct",
+            originalAllowListConvenience[0],
+            equalTo(contentBlocking.allowListConvenienceTrackingProtection),
+        )
+
+        contentBlocking.allowListConvenienceTrackingProtection = false
+
+        val actualPrefs = sessionRule.getPrefs(
+            "privacy.trackingprotection.allow_list.convenience.enabled",
+        )
+
+        assertThat(
+            "Convenience is updated",
+            actualPrefs[0] as Boolean,
+            equalTo(false),
+        )
+
+        // Set a new pref value, with a different setter method.
+        contentBlocking.setAllowListConvenienceTrackingProtection(true)
+
+        val actualPrefs2 = sessionRule.getPrefs(
+            "privacy.trackingprotection.allow_list.convenience.enabled",
+        )
+
+        assertThat(
+            "Convenience is updated",
+            actualPrefs2[0] as Boolean,
+            equalTo(true),
+        )
+    }
 }
