@@ -6,7 +6,12 @@ package org.mozilla.geckoview.test
 
 import androidx.test.ext.junit.runners.AndroidJUnit4
 import androidx.test.filters.MediumTest
-import org.hamcrest.Matchers.* // ktlint-disable no-wildcard-imports
+import junit.framework.TestCase.assertFalse
+import junit.framework.TestCase.assertNull
+import junit.framework.TestCase.assertTrue
+import org.hamcrest.Matchers.endsWith
+import org.hamcrest.Matchers.equalTo
+import org.hamcrest.Matchers.nullValue
 import org.junit.Assume.assumeThat
 import org.junit.Ignore
 import org.junit.Test
@@ -250,14 +255,6 @@ class HistoryDelegateTest : BaseSessionTest() {
     }
 
     @Test fun onHistoryStateChangeSavingState() {
-        // TODO: Bug 1884334
-        val geckoPrefs = sessionRule.getPrefs(
-            "fission.disableSessionHistoryInParent",
-        )
-        assumeThat(geckoPrefs[0] as Boolean, equalTo(true))
-        // TODO: Bug 1837551
-        assumeThat(sessionRule.env.isFission, equalTo(false))
-
         // This is a smaller version of the above test, in the hopes to minimize race conditions
         mainSession.loadTestPath(HELLO_HTML_PATH)
 
@@ -305,4 +302,117 @@ class HistoryDelegateTest : BaseSessionTest() {
             }
         })
     }
+
+    @Test fun fissionDisabledWithShipDisabled() {
+        // Check if this is a fission disabled run.
+        assumeThat(sessionRule.env.isFission, equalTo(false))
+
+        // Check preference with Gecko, this can be changed externally through configs.
+        val shipPref = sessionRule.getPrefs(
+            "fission.disableSessionHistoryInParent",
+        )
+
+        // Check if this is a SHIP disabled run.
+        assumeThat(
+            shipPref[0] as Boolean,
+            equalTo(true),
+        )
+
+        // Check preference with GeckoView
+        assertNull(
+            "Default will have no value since we are relying on Gecko.",
+            sessionRule.runtime.settings.disableShip,
+        )
+
+        // Verify SHIP is not running
+        assertFalse(
+            "SHIP is not running.",
+            sessionRule.isSessionHistoryInParentRunning,
+        )
+    }
+
+    @Test fun fissionDisabledWithShipEnabled() {
+        // Check if this is a fission disabled run.
+        assumeThat(sessionRule.env.isFission, equalTo(false))
+
+        // Check preference with Gecko, this can be changed externally through configs.
+        val shipPref = sessionRule.getPrefs(
+            "fission.disableSessionHistoryInParent",
+        )
+
+        // Check if this is a SHIP enabled run.
+        assumeThat(
+            shipPref[0] as Boolean,
+            equalTo(false),
+        )
+
+        // Check preference with GeckoView
+        assertNull(
+            "Default will have no value since we are relying on Gecko.",
+            sessionRule.runtime.settings.disableShip,
+        )
+
+        // Verify SHIP is running
+        assertTrue(
+            "SHIP is running.",
+            sessionRule.isSessionHistoryInParentRunning,
+        )
+    }
+
+    @Test fun fissionEnabledWithShipEnabled() {
+        // Check if this is a fission enabled run.
+        assumeThat(sessionRule.env.isFission, equalTo(true))
+
+        // Check preference with Gecko, this can be changed externally through configs.
+        val shipPref = sessionRule.getPrefs(
+            "fission.disableSessionHistoryInParent",
+        )
+
+        // Check if this is a SHIP enabled run.
+        assumeThat(
+            shipPref[0] as Boolean,
+            equalTo(false),
+        )
+
+        // Check preference with GeckoView
+        assertNull(
+            "Default will have no value since we are relying on Gecko.",
+            sessionRule.runtime.settings.disableShip,
+        )
+
+        // Verify SHIP is running
+        assertTrue(
+            "SHIP is running.",
+            sessionRule.isSessionHistoryInParentRunning,
+        )
+    }
+
+    @Test fun flushSessionStateTriggersHistoryChange() {
+        mainSession.loadTestPath(HELLO_HTML_PATH)
+        mainSession.loadTestPath(HELLO2_HTML_PATH)
+
+        sessionRule.delegateUntilTestEnd(object : HistoryDelegate {
+            @AssertCalled(count = 1)
+            override fun onHistoryStateChange(session: GeckoSession, state: GeckoSession.HistoryDelegate.HistoryList) {
+                assertThat(
+                    "History should have two entries",
+                    state.size,
+                    equalTo(2),
+                )
+                assertThat(
+                    "URLs should match",
+                    state[state.currentIndex].uri,
+                    endsWith(HELLO2_HTML_PATH),
+                )
+                assertThat(
+                    "History index should be 1",
+                    state.currentIndex,
+                    equalTo(1),
+                )
+            }
+        })
+
+        sessionRule.waitForPageStops(2)
+        mainSession.flushSessionState()
+    }
 }
