@@ -1,3 +1,6 @@
+/* Any copyright is dedicated to the Public Domain.
+   http://creativecommons.org/publicdomain/zero/1.0/ */
+
 package org.mozilla.geckoview.test
 
 import androidx.test.ext.junit.runners.AndroidJUnit4
@@ -8,11 +11,16 @@ import org.junit.Before
 import org.junit.Test
 import org.junit.runner.RunWith
 import org.mozilla.gecko.util.ThreadUtils
-import org.mozilla.geckoview.* // ktlint-disable no-wildcard-imports
+import org.mozilla.geckoview.AllowOrDeny
+import org.mozilla.geckoview.GeckoResult
 import org.mozilla.geckoview.GeckoRuntime.ServiceWorkerDelegate
+import org.mozilla.geckoview.GeckoSession
 import org.mozilla.geckoview.GeckoSession.ContentDelegate
 import org.mozilla.geckoview.GeckoSession.NavigationDelegate
+import org.mozilla.geckoview.GeckoSession.NavigationDelegate.LoadRequest
 import org.mozilla.geckoview.GeckoSession.PermissionDelegate
+import org.mozilla.geckoview.WebNotification
+import org.mozilla.geckoview.WebNotificationDelegate
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.AssertCalled
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.NullDelegate
@@ -35,8 +43,8 @@ class OpenWindowTest : BaseSessionTest() {
         })
     }
 
-    private fun openPageClickNotification() {
-        mainSession.loadTestPath(OPEN_WINDOW_PATH)
+    private fun openPageClickNotification(teardownAlertsService: Boolean = false, urlParam: String = "") {
+        mainSession.loadUri(OPEN_WINDOW_PATH + urlParam)
         sessionRule.waitForPageStop()
         val result = mainSession.waitForJS("Notification.requestPermission()")
         assertThat(
@@ -52,11 +60,17 @@ class OpenWindowTest : BaseSessionTest() {
             @GeckoSessionTestRule.AssertCalled
             override fun onShowNotification(notification: WebNotification) {
                 notificationShown = notification
+                notification.show()
                 notificationResult.complete(null)
             }
         })
         mainSession.evaluateJS("showNotification()")
         sessionRule.waitForResult(notificationResult)
+
+        if (teardownAlertsService) {
+            mainSession.teardownAlertsService()
+        }
+
         notificationShown!!.click()
     }
 
@@ -66,7 +80,7 @@ class OpenWindowTest : BaseSessionTest() {
         sessionRule.delegateUntilTestEnd(object : ContentDelegate, NavigationDelegate {
             override fun onLocationChange(session: GeckoSession, url: String?, perms: MutableList<PermissionDelegate.ContentPermission>, hasUserGesture: Boolean) {
                 // we should not open the target url
-                assertThat("URL should notmatch", url, not(createTestUrl(OPEN_WINDOW_TARGET_PATH)))
+                assertThat("URL should notmatch", url, not(OPEN_WINDOW_TARGET_PATH))
             }
         })
         openPageClickNotification()
@@ -78,7 +92,7 @@ class OpenWindowTest : BaseSessionTest() {
         sessionRule.delegateUntilTestEnd(object : ContentDelegate, NavigationDelegate {
             override fun onLocationChange(session: GeckoSession, url: String?, perms: MutableList<PermissionDelegate.ContentPermission>, hasUserGesture: Boolean) {
                 // we should not open the target url
-                assertThat("URL should notmatch", url, not(createTestUrl(OPEN_WINDOW_TARGET_PATH)))
+                assertThat("URL should notmatch", url, not(OPEN_WINDOW_TARGET_PATH))
             }
         })
         openPageClickNotification()
@@ -105,7 +119,7 @@ class OpenWindowTest : BaseSessionTest() {
             @AssertCalled(count = 1, order = [1])
             override fun onLocationChange(session: GeckoSession, url: String?, perms: MutableList<PermissionDelegate.ContentPermission>, hasUserGesture: Boolean) {
                 assertThat("Should be on the main session", session, equalTo(mainSession))
-                assertThat("URL should match", url, equalTo(createTestUrl(OPEN_WINDOW_TARGET_PATH)))
+                assertThat("URL should match", url, equalTo(OPEN_WINDOW_TARGET_PATH))
             }
 
             @AssertCalled(count = 1, order = [2])
@@ -132,7 +146,7 @@ class OpenWindowTest : BaseSessionTest() {
             @AssertCalled(count = 1, order = [1])
             override fun onLocationChange(session: GeckoSession, url: String?, perms: MutableList<PermissionDelegate.ContentPermission>, hasUserGesture: Boolean) {
                 assertThat("Should be on the target session", session, equalTo(targetSession))
-                assertThat("URL should match", url, equalTo(createTestUrl(OPEN_WINDOW_TARGET_PATH)))
+                assertThat("URL should match", url, equalTo(OPEN_WINDOW_TARGET_PATH))
             }
 
             @AssertCalled(count = 1, order = [2])
@@ -142,4 +156,77 @@ class OpenWindowTest : BaseSessionTest() {
             }
         })
     }
+
+    @Test
+    fun openWindowNewClosedSession() {
+        var targetSession: GeckoSession? = null
+        sessionRule.delegateUntilTestEnd(object : ServiceWorkerDelegate {
+            @AssertCalled(count = 1)
+            override fun onOpenWindow(url: String): GeckoResult<GeckoSession> {
+                ThreadUtils.assertOnUiThread()
+                assertThat("URL should match", url, equalTo(OPEN_WINDOW_TARGET_PATH))
+                targetSession = sessionRule.createClosedSession()
+                return GeckoResult.fromValue(targetSession)
+            }
+        })
+        openPageClickNotification()
+        sessionRule.waitUntilCalled(object : ContentDelegate, NavigationDelegate {
+            @AssertCalled(count = 2, order = [1, 2])
+            override fun onLocationChange(session: GeckoSession, url: String?, perms: MutableList<PermissionDelegate.ContentPermission>, hasUserGesture: Boolean) {
+                assertThat("Should be on the target session", session, equalTo(targetSession))
+                assertThat("URL should match", url, equalTo(forEachCall("about:blank", OPEN_WINDOW_TARGET_PATH)))
+            }
+
+            @AssertCalled(count = 1, order = [3])
+            override fun onLoadRequest(session: GeckoSession, request: LoadRequest): GeckoResult<AllowOrDeny>? {
+                assertThat("Should be on the target session", session, equalTo(targetSession))
+                assertThat("URL should match", request.uri, equalTo(OPEN_WINDOW_TARGET_PATH))
+                return null
+            }
+
+            @AssertCalled(count = 1, order = [4])
+            override fun onTitleChange(session: GeckoSession, title: String?) {
+                assertThat("Should be on the target session", session, equalTo(targetSession))
+                assertThat("Title should be correct", title, equalTo("Open Window test target"))
+            }
+        })
+    }
+
+    @Test
+    fun openWindowAfterAlertServiceTeardown() {
+        sessionRule.delegateUntilTestEnd(object : ServiceWorkerDelegate {
+            @AssertCalled(count = 1)
+            override fun onOpenWindow(url: String): GeckoResult<GeckoSession> {
+                ThreadUtils.assertOnUiThread()
+                return GeckoResult.fromValue(mainSession)
+            }
+        })
+        openPageClickNotification(teardownAlertsService = true)
+        sessionRule.waitUntilCalled(object : ContentDelegate, NavigationDelegate {
+            @AssertCalled(count = 1)
+            override fun onLocationChange(session: GeckoSession, url: String?, perms: MutableList<PermissionDelegate.ContentPermission>, hasUserGesture: Boolean) {
+                assertThat("Should be on the main session", session, equalTo(mainSession))
+                assertThat("URL should match", url, equalTo(OPEN_WINDOW_TARGET_PATH))
+            }
+        })
+    }
+
+    @Test
+    fun openWindowWithActionAfterAlertServiceTeardown() {
+        sessionRule.delegateUntilTestEnd(object : ServiceWorkerDelegate {
+            @AssertCalled(count = 1)
+            override fun onOpenWindow(url: String): GeckoResult<GeckoSession> {
+                ThreadUtils.assertOnUiThread()
+                return GeckoResult.fromValue(mainSession)
+            }
+        })
+        openPageClickNotification(teardownAlertsService = true, urlParam = "?action=foo")
+        sessionRule.waitUntilCalled(object : ContentDelegate, NavigationDelegate {
+            @AssertCalled(count = 1)
+            override fun onLocationChange(session: GeckoSession, url: String?, perms: MutableList<PermissionDelegate.ContentPermission>, hasUserGesture: Boolean) {
+                assertThat("Should be on the main session", session, equalTo(mainSession))
+                assertThat("URL should match", url, equalTo("$OPEN_WINDOW_TARGET_PATH?action=foo"))
+            }
+        })
+    }
 }
