@@ -4,14 +4,16 @@
 
 package org.mozilla.geckoview.test
 
+import android.accessibilityservice.AccessibilityService
 import android.app.UiAutomation.FLAG_DONT_SUPPRESS_ACCESSIBILITY_SERVICES
 import android.content.Context
-import android.graphics.Bitmap
 import android.graphics.Color
 import android.graphics.Color.rgb
 import android.os.Handler
 import android.os.Looper
 import android.view.accessibility.AccessibilityEvent.TYPE_VIEW_SCROLLED
+import androidx.core.graphics.get
+import androidx.core.graphics.scale
 import androidx.test.ext.junit.rules.ActivityScenarioRule
 import androidx.test.ext.junit.runners.AndroidJUnit4
 import androidx.test.filters.LargeTest
@@ -26,7 +28,9 @@ import org.junit.rules.RuleChain
 import org.junit.runner.RunWith
 import org.mozilla.geckoview.Autofill
 import org.mozilla.geckoview.GeckoResult
+import org.mozilla.geckoview.GeckoResult.fromException
 import org.mozilla.geckoview.GeckoSession
+import org.mozilla.geckoview.GeckoSession.GeckoPrintException
 import org.mozilla.geckoview.GeckoSession.PrintDelegate
 import org.mozilla.geckoview.GeckoView.ActivityContextDelegate
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule
@@ -92,7 +96,6 @@ class PrintDelegateTest : BaseSessionTest() {
     @NullDelegate(Autofill.Delegate::class)
     @Test
     fun windowDotPrintAvailableTest() {
-        sessionRule.setPrefsUntilTestEnd(mapOf("dom.enable_window_print" to true))
         activityRule.scenario.onActivity {
             mainSession.loadTestPath(COLOR_ORANGE_BACKGROUND_HTML_PATH)
             mainSession.waitForPageStop()
@@ -113,8 +116,8 @@ class PrintDelegateTest : BaseSessionTest() {
                 // Delaying the screenshot to give time for preview to load
                 Handler(Looper.getMainLooper()).postDelayed({
                     val bitmap = uiAutomation.takeScreenshot()
-                    val scaled = Bitmap.createScaledBitmap(bitmap, scaledWidth, scaledHeight, false)
-                    pixelResult.complete(scaled.getPixel(scaledWidth / 2, scaledHeight / 2))
+                    val scaled = bitmap.scale(scaledWidth, scaledHeight, filter = false)
+                    pixelResult.complete(scaled[scaledWidth / 2, scaledHeight / 2])
                 }, 1500)
             }
         }
@@ -130,9 +133,9 @@ class PrintDelegateTest : BaseSessionTest() {
             mainSession.waitForPageStop()
             // Setting to the default delegate (test rules changed it)
             mainSession.printDelegate = activity.view.printDelegate
+            val centerPixel = printCenterPixelColor()
             mainSession.printPageContent()
             val orange = rgb(255, 113, 57)
-            val centerPixel = printCenterPixelColor()
             assertTrue(
                 "Android print opened and rendered.",
                 sessionRule.waitForResult(centerPixel) == orange,
@@ -142,16 +145,79 @@ class PrintDelegateTest : BaseSessionTest() {
 
     @NullDelegate(Autofill.Delegate::class)
     @Test
-    fun basicWindowDotPrintTest() {
-        sessionRule.setPrefsUntilTestEnd(mapOf("dom.enable_window_print" to true))
+    fun printFtFontRendered() {
+        activityRule.scenario.onActivity { activity ->
+            mainSession.loadTestPath(FT_FONT_HTML_PATH)
+            mainSession.waitForPageStop()
+            mainSession.printDelegate = activity.view.printDelegate
+            val centerPixel = printCenterPixelColor()
+            mainSession.printPageContent()
+            val orange = rgb(255, 113, 57)
+            assertTrue(
+                "Android print opened and rendered.",
+                sessionRule.waitForResult(centerPixel) == orange,
+            )
+        }
+    }
+
+    @NullDelegate(Autofill.Delegate::class)
+    @Test
+    fun printSuccessWithStatus() {
         activityRule.scenario.onActivity { activity ->
             // CSS rules render this blue on screen and orange on print
             mainSession.loadTestPath(PRINT_CONTENT_CHANGE)
             mainSession.waitForPageStop()
             // Setting to the default delegate (test rules changed it)
             mainSession.printDelegate = activity.view.printDelegate
-            mainSession.evaluateJS("window.print();")
             val centerPixel = printCenterPixelColor()
+            val result = mainSession.didPrintPageContent()
+            val orange = rgb(255, 113, 57)
+            assertTrue(
+                "Android print opened and rendered.",
+                sessionRule.waitForResult(centerPixel) == orange,
+            )
+            uiAutomation.performGlobalAction(AccessibilityService.GLOBAL_ACTION_BACK)
+            assertTrue(
+                "Printing should conclude when back is pressed.",
+                sessionRule.waitForResult(result),
+            )
+        }
+    }
+
+    @Test
+    fun printFailWithStatus() {
+        activityRule.scenario.onActivity {
+            // CSS rules render this blue on screen and orange on print
+            mainSession.loadTestPath(PRINT_CONTENT_CHANGE)
+            mainSession.waitForPageStop()
+            mainSession.printDelegate = null
+            val result = mainSession.didPrintPageContent().accept {
+                assertTrue("Should not be able to print.", false)
+            }.exceptionally(
+                GeckoResult.OnExceptionListener<Throwable> { error: Throwable ->
+                    assertTrue("Should receive a missing print delegate exception.", (error as GeckoPrintException).code == GeckoPrintException.ERROR_NO_PRINT_DELEGATE)
+                    fromException(error)
+                },
+            )
+            try {
+                sessionRule.waitForResult(result)
+            } catch (e: Exception) {
+                assertTrue("Should have an exception", true)
+            }
+        }
+    }
+
+    @NullDelegate(Autofill.Delegate::class)
+    @Test
+    fun basicWindowDotPrintTest() {
+        activityRule.scenario.onActivity { activity ->
+            // CSS rules render this blue on screen and orange on print
+            mainSession.loadTestPath(PRINT_CONTENT_CHANGE)
+            mainSession.waitForPageStop()
+            // Setting to the default delegate (test rules changed it)
+            mainSession.printDelegate = activity.view.printDelegate
+            val centerPixel = printCenterPixelColor()
+            mainSession.evaluateJS("window.print();")
             val orange = rgb(255, 113, 57)
             assertTrue(
                 "Android print opened and rendered.",
@@ -163,15 +229,14 @@ class PrintDelegateTest : BaseSessionTest() {
     @NullDelegate(Autofill.Delegate::class)
     @Test
     fun statusWindowDotPrintTest() {
-        sessionRule.setPrefsUntilTestEnd(mapOf("dom.enable_window_print" to true))
         activityRule.scenario.onActivity { activity ->
             // CSS rules render this blue on screen and orange on print
             mainSession.loadTestPath(PRINT_CONTENT_CHANGE)
             mainSession.waitForPageStop()
             // Setting to the default delegate (test rules changed it)
             mainSession.printDelegate = activity.view.printDelegate
-            mainSession.evaluateJS("window.print()")
             val centerPixel = printCenterPixelColor()
+            mainSession.evaluateJS("window.print()")
             val orange = rgb(255, 113, 57)
             assertTrue(
                 "Android print opened and rendered.",
@@ -195,7 +260,6 @@ class PrintDelegateTest : BaseSessionTest() {
     @NullDelegate(Autofill.Delegate::class)
     @Test
     fun staticContextWindowDotPrintTest() {
-        sessionRule.setPrefsUntilTestEnd(mapOf("dom.enable_window_print" to true))
         activityRule.scenario.onActivity { activity ->
             // CSS rules render this blue on screen and orange on print
             // Print button removes content after printing to test if it froze a static page for printing
@@ -203,8 +267,8 @@ class PrintDelegateTest : BaseSessionTest() {
             mainSession.waitForPageStop()
             // Setting to the default delegate (test rules changed it)
             mainSession.printDelegate = activity.view.printDelegate
-            mainSession.evaluateJS("document.getElementById('print-button').click();")
             val centerPixel = printCenterPixelColor()
+            mainSession.evaluateJS("document.getElementById('print-button').click();")
             val orange = rgb(255, 113, 57)
             assertTrue(
                 "Android print opened and rendered static page.",
@@ -216,7 +280,6 @@ class PrintDelegateTest : BaseSessionTest() {
     @NullDelegate(Autofill.Delegate::class)
     @Test
     fun iframeWindowDotPrintTest() {
-        sessionRule.setPrefsUntilTestEnd(mapOf("dom.enable_window_print" to true))
         activityRule.scenario.onActivity { activity ->
             // Main frame CSS rules render red on screen and green on print
             // iframe CSS rules render blue on screen and orange on print
@@ -225,8 +288,8 @@ class PrintDelegateTest : BaseSessionTest() {
             // Setting to the default delegate (test rules changed it)
             mainSession.printDelegate = activity.view.printDelegate
             // iframe window.print button
-            mainSession.evaluateJS("document.getElementById('iframe').contentDocument.getElementById('print-button').click();")
             val centerPixelIframe = printCenterPixelColor()
+            mainSession.evaluateJS("document.getElementById('iframe').contentDocument.getElementById('print-button').click();")
             val orange = rgb(255, 113, 57)
             sessionRule.waitForResult(centerPixelIframe).let { it ->
                 assertTrue("The iframe should not print green. (Printed containing page instead of iframe.)", it != Color.GREEN)
@@ -238,7 +301,6 @@ class PrintDelegateTest : BaseSessionTest() {
     @NullDelegate(Autofill.Delegate::class)
     @Test
     fun contentIframeWindowDotPrintTest() {
-        sessionRule.setPrefsUntilTestEnd(mapOf("dom.enable_window_print" to true))
         activityRule.scenario.onActivity { activity ->
             // Main frame CSS rules render red on screen and green on print
             // iframe CSS rules render blue on screen and orange on print
@@ -247,9 +309,48 @@ class PrintDelegateTest : BaseSessionTest() {
             // Setting to the default delegate (test rules changed it)
             mainSession.printDelegate = activity.view.printDelegate
             // Main page window.print button
-            mainSession.evaluateJS("document.getElementById('print-button-page').click();")
             val centerPixelContent = printCenterPixelColor()
+            mainSession.evaluateJS("document.getElementById('print-button-page').click();")
             assertTrue("Printed the main content correctly.", sessionRule.waitForResult(centerPixelContent) == Color.GREEN)
         }
     }
+
+    @NullDelegate(Autofill.Delegate::class)
+    @Test
+    fun contentPDFWindowDotPrintTest() {
+        activityRule.scenario.onActivity { activity ->
+            // CSS rules render this blue on screen and orange on print
+            mainSession.loadTestPath(ORANGE_PDF_PATH)
+            mainSession.waitForPageStop()
+            // Setting to the default delegate (test rules changed it)
+            mainSession.printDelegate = activity.view.printDelegate
+            val centerPixel = printCenterPixelColor()
+            mainSession.printPageContent()
+            val orange = rgb(255, 113, 57)
+            assertTrue(
+                "Android print opened and rendered.",
+                sessionRule.waitForResult(centerPixel) == orange,
+            )
+        }
+    }
+
+    @NullDelegate(Autofill.Delegate::class)
+    @Test
+    fun availableCanonicalBrowsingContext() {
+        activityRule.scenario.onActivity { activity ->
+            // CSS rules render this blue on screen and orange on print
+            mainSession.loadTestPath(ORANGE_PDF_PATH)
+            mainSession.waitForPageStop()
+            // Setting to the default delegate (test rules changed it)
+            mainSession.printDelegate = activity.view.printDelegate
+            mainSession.setFocused(false)
+            val centerPixel = printCenterPixelColor()
+            mainSession.printPageContent()
+            val orange = rgb(255, 113, 57)
+            assertTrue(
+                "Android print opened and rendered.",
+                sessionRule.waitForResult(centerPixel) == orange,
+            )
+        }
+    }
 }
