@@ -9,18 +9,23 @@ import android.content.ClipboardManager
 import android.content.Context
 import android.graphics.Point
 import android.graphics.RectF
-import android.net.Uri
 import android.os.Build
 import android.util.Base64
+import androidx.core.net.toUri
 import androidx.test.ext.junit.rules.ActivityScenarioRule
 import androidx.test.filters.MediumTest
-import androidx.test.filters.SdkSuppress
 import androidx.test.platform.app.InstrumentationRegistry
 import org.hamcrest.Matcher
-import org.hamcrest.Matchers.* // ktlint-disable no-wildcard-imports
+import org.hamcrest.Matchers.arrayContainingInAnyOrder
+import org.hamcrest.Matchers.both
+import org.hamcrest.Matchers.containsString
+import org.hamcrest.Matchers.equalTo
+import org.hamcrest.Matchers.hasToString
+import org.hamcrest.Matchers.not
 import org.json.JSONArray
 import org.junit.Assume.assumeThat
 import org.junit.Before
+import org.junit.Ignore
 import org.junit.Rule
 import org.junit.Test
 import org.junit.rules.RuleChain
@@ -33,7 +38,21 @@ import org.mozilla.geckoview.GeckoResult
 import org.mozilla.geckoview.GeckoSession
 import org.mozilla.geckoview.GeckoSession.PromptDelegate
 import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate
-import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.* // ktlint-disable no-wildcard-imports
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.ACTION_COLLAPSE_TO_END
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.ACTION_COLLAPSE_TO_START
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.ACTION_COPY
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.ACTION_CUT
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.ACTION_DELETE
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.ACTION_HIDE
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.ACTION_PASTE
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.ACTION_PASTE_AS_PLAIN_TEXT
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.ACTION_SELECT_ALL
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.ACTION_UNSELECT
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.ClipboardPermission
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.FLAG_IS_COLLAPSED
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.FLAG_IS_EDITABLE
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.HIDE_REASON_NO_SELECTION
+import org.mozilla.geckoview.GeckoSession.SelectionActionDelegate.Selection
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.AssertCalled
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.NullDelegate
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.WithDisplay
@@ -54,7 +73,7 @@ class SelectionActionDelegateTest : BaseSessionTest() {
     companion object {
         @get:Parameters(name = "{0}")
         @JvmStatic
-        val parameters: List<Array<out Any>> = listOf(
+        val parameters: List<Array<Any>> = listOf(
             arrayOf("#text", ContentType.DIV, "lorem", false),
             arrayOf("#input", ContentType.EDITABLE_ELEMENT, "ipsum", true),
             arrayOf("#textarea", ContentType.EDITABLE_ELEMENT, "dolor", true),
@@ -149,7 +168,6 @@ class SelectionActionDelegateTest : BaseSessionTest() {
         }
     }
 
-    @SdkSuppress(minSdkVersion = Build.VERSION_CODES.O)
     @Test
     fun request_html() {
         if (editable) {
@@ -252,7 +270,6 @@ class SelectionActionDelegateTest : BaseSessionTest() {
         }
     }
 
-    @SdkSuppress(minSdkVersion = Build.VERSION_CODES.O)
     @Test
     fun pasteAsPlainText() = assumingEditable(true) {
         assumeThat("Paste as plain text works on content editable", type, not(equalTo(ContentType.EDITABLE_ELEMENT)))
@@ -303,6 +320,7 @@ class SelectionActionDelegateTest : BaseSessionTest() {
         testThat(selectedContent, withResponse(ACTION_UNSELECT), clearsSelection())
     }
 
+    @Ignore("https://bugzilla.mozilla.org/show_bug.cgi?id=1988041")
     @Test fun multipleActions() = assumingEditable(false) {
         withClipboard("") {
             testThat(
@@ -710,7 +728,7 @@ class SelectionActionDelegateTest : BaseSessionTest() {
         val oldClip = clipboard.primaryClip
         try {
             TestContentProvider.setTestData(this.getTestBytes(contentPath), mime)
-            val clipData = ClipData("image", arrayOf(mime), ClipData.Item(Uri.parse("content://org.mozilla.geckoview.test.provider/gif")))
+            val clipData = ClipData("image", arrayOf(mime), ClipData.Item("content://org.mozilla.geckoview.test.provider/gif".toUri()))
             clipboard.setPrimaryClip(clipData)
 
             sessionRule.addExternalDelegateUntilTestEnd(
