@@ -21,7 +21,6 @@ import android.view.inputmethod.InputConnection
 import android.view.inputmethod.InputContentInfo
 import androidx.core.net.toUri
 import androidx.test.filters.MediumTest
-import androidx.test.filters.SdkSuppress
 import androidx.test.platform.app.InstrumentationRegistry
 import org.hamcrest.Matchers.equalTo
 import org.hamcrest.Matchers.not
@@ -298,6 +297,7 @@ class TextInputDelegateTest : BaseSessionTest() {
         })
     }
 
+    @Ignore("https://bugzilla.mozilla.org/show_bug.cgi?id=1988041")
     @Test fun restartInput_temporaryFocus() {
         // Our user action trick doesn't work for design-mode, so we can't test that here.
         assumeThat("Not in designmode", id, not(equalTo("#designmode")))
@@ -632,6 +632,7 @@ class TextInputDelegateTest : BaseSessionTest() {
         )
     }
 
+    @Ignore("https://bugzilla.mozilla.org/show_bug.cgi?id=1988041")
     // Test deleteSurroundingText
     @WithDisplay(width = 512, height = 512)
     // Child process updates require having a display.
@@ -927,7 +928,6 @@ class TextInputDelegateTest : BaseSessionTest() {
         )
     }
 
-    @SdkSuppress(minSdkVersion = Build.VERSION_CODES.N_MR1)
     @WithDisplay(width = 512, height = 512)
     // Child process updates require having a display.
     @Test
@@ -1052,6 +1052,7 @@ class TextInputDelegateTest : BaseSessionTest() {
     }
 
     // Bug 1275371 - shift+backspace should not forward delete on Android.
+    @Ignore("https://bugzilla.mozilla.org/show_bug.cgi?id=1988041")
     @WithDisplay(width = 512, height = 512)
     // Child process updates require having a display.
     @Test
