@@ -3,11 +3,13 @@ package org.mozilla.geckoview.test
 import android.os.Parcel
 import androidx.test.ext.junit.runners.AndroidJUnit4
 import androidx.test.filters.MediumTest
-import org.hamcrest.Matchers.* // ktlint-disable no-wildcard-imports
+import org.hamcrest.Matchers.endsWith
+import org.hamcrest.Matchers.equalTo
+import org.hamcrest.Matchers.not
 import org.junit.Assert.assertNotNull
 import org.junit.Assert.assertTrue
+import org.junit.Assume.assumeThat
 import org.junit.Before
-import org.junit.Ignore
 import org.junit.Test
 import org.junit.runner.RunWith
 import org.mozilla.geckoview.GeckoResult
@@ -56,6 +58,8 @@ class WebNotificationTest : BaseSessionTest() {
                 assertThat("Silent should match", notification.silent, equalTo(true))
                 assertThat("Vibrate should match", notification.vibrate, equalTo(intArrayOf()))
                 assertThat("Source should match", notification.source, equalTo(createTestUrl(HELLO_HTML_PATH)))
+                assertThat("Origin should match", notification.origin, equalTo(GeckoSessionTestRule.TEST_ENDPOINT))
+                notification.show()
                 notificationResult.complete(null)
             }
         })
@@ -72,7 +76,6 @@ class WebNotificationTest : BaseSessionTest() {
     fun assertNotificationData(notification: WebNotification, requireInteraction: Boolean) {
         assertThat("Title should match", notification.title, equalTo("The Title"))
         assertThat("Body should match", notification.text, equalTo("The Text"))
-        assertThat("Tag should match", notification.tag, endsWith("Tag"))
         assertThat("ImageUrl should match", notification.imageUrl, endsWith("icon.png"))
         assertThat("Language should match", notification.lang, equalTo("en-US"))
         assertThat("Direction should match", notification.textDirection, equalTo("ltr"))
@@ -84,6 +87,13 @@ class WebNotificationTest : BaseSessionTest() {
         assertThat("Vibrate should match", notification.vibrate, equalTo(intArrayOf(1, 2, 3, 4)))
         assertThat("Silent should match", notification.silent, equalTo(false))
         assertThat("Source should match", notification.source, equalTo(createTestUrl(HELLO_HTML_PATH)))
+
+        val origin = if (notification.privateBrowsing) {
+            GeckoSessionTestRule.TEST_ENDPOINT + "^privateBrowsingId=1"
+        } else {
+            GeckoSessionTestRule.TEST_ENDPOINT
+        }
+        assertThat("Origin should match", notification.origin, equalTo(origin))
     }
 
     @GeckoSessionTestRule.Setting.List(
@@ -92,7 +102,6 @@ class WebNotificationTest : BaseSessionTest() {
             value = "true",
         ),
     )
-    @Ignore // Bug 1843046 - Disabled because private notifications are temporarily disabled.
     @Test
     fun onShowNotification() {
         sessionRule.setPrefsUntilTestEnd(mapOf("dom.webnotifications.vibrate.enabled" to true))
@@ -105,6 +114,7 @@ class WebNotificationTest : BaseSessionTest() {
             override fun onShowNotification(notification: WebNotification) {
                 assertNotificationData(notification, requireInteraction)
                 assertThat("privateBrowsing should match", notification.privateBrowsing, equalTo(true))
+                notification.show()
                 notificationResult.complete(null)
             }
         })
@@ -120,6 +130,60 @@ class WebNotificationTest : BaseSessionTest() {
         sessionRule.waitForResult(notificationResult)
     }
 
+    @Test
+    fun onShowNotificationSameTag() {
+        val tags = mutableListOf<String>()
+        val notificationResult = GeckoResult<Void>()
+
+        sessionRule.delegateDuringNextWait(object : WebNotificationDelegate {
+            @GeckoSessionTestRule.AssertCalled
+            override fun onShowNotification(notification: WebNotification) {
+                tags += notification.tag
+                if (tags.size == 2) {
+                    notificationResult.complete(null)
+                }
+            }
+        })
+
+        mainSession.evaluateJS(
+            """
+            new Notification('The Title', { tag: 'Tag' });
+            new Notification('The Title', { tag: 'Tag' });
+            """.trimIndent(),
+        )
+
+        sessionRule.waitForResult(notificationResult)
+
+        assertThat("tags should be same", tags[0], equalTo(tags[1]))
+    }
+
+    @Test
+    fun onShowNotificationDifferentTag() {
+        val tags = mutableListOf<String>()
+        val notificationResult = GeckoResult<Void>()
+
+        sessionRule.delegateDuringNextWait(object : WebNotificationDelegate {
+            @GeckoSessionTestRule.AssertCalled
+            override fun onShowNotification(notification: WebNotification) {
+                tags += notification.tag
+                if (tags.size == 2) {
+                    notificationResult.complete(null)
+                }
+            }
+        })
+
+        mainSession.evaluateJS(
+            """
+            new Notification('The Title', { tag: 'Tag1' });
+            new Notification('The Title', { tag: 'Tag2' });
+            """.trimIndent(),
+        )
+
+        sessionRule.waitForResult(notificationResult)
+
+        assertThat("tags should be different", tags[0], not(equalTo(tags[1])))
+    }
+
     @Test fun onCloseNotification() {
         val closeCalled = GeckoResult<Void>()
 
@@ -149,6 +213,7 @@ class WebNotificationTest : BaseSessionTest() {
         sessionRule.delegateDuringNextWait(object : WebNotificationDelegate {
             @GeckoSessionTestRule.AssertCalled
             override fun onShowNotification(notification: WebNotification) {
+                notification.show()
                 notificationResult.complete(notification)
             }
         })
@@ -185,6 +250,7 @@ class WebNotificationTest : BaseSessionTest() {
         val deserialized = WebNotification.CREATOR.createFromParcel(parcel)
         assertNotificationData(deserialized, requireInteraction)
         assertThat("privateBrowsing should match", deserialized.privateBrowsing, equalTo(false))
+        assertThat("tag should match", deserialized.tag, equalTo(notification.tag))
 
         deserialized!!.click()
         assertThat("Promise should have been resolved.", promiseResult.value as Double, equalTo(1.0))
@@ -196,7 +262,6 @@ class WebNotificationTest : BaseSessionTest() {
             value = "true",
         ),
     )
-    @Ignore // Bug 1843046 - Disabled because private notifications are temporarily disabled.
     @Test
     fun clickPrivateNotificationParceled() {
         sessionRule.setPrefsUntilTestEnd(mapOf("dom.webnotifications.vibrate.enabled" to true))
@@ -207,6 +272,7 @@ class WebNotificationTest : BaseSessionTest() {
         sessionRule.delegateDuringNextWait(object : WebNotificationDelegate {
             @GeckoSessionTestRule.AssertCalled
             override fun onShowNotification(notification: WebNotification) {
+                notification.show()
                 notificationResult.complete(notification)
             }
         })
@@ -256,6 +322,7 @@ class WebNotificationTest : BaseSessionTest() {
             @GeckoSessionTestRule.AssertCalled
             override fun onShowNotification(notification: WebNotification) {
                 notificationShown = notification
+                notification.show()
                 notificationResult.complete(null)
             }
         })
@@ -285,6 +352,7 @@ class WebNotificationTest : BaseSessionTest() {
             @GeckoSessionTestRule.AssertCalled
             override fun onShowNotification(notification: WebNotification) {
                 notificationShown = notification
+                notification.show()
                 notificationResult.complete(null)
             }
         })
@@ -312,6 +380,7 @@ class WebNotificationTest : BaseSessionTest() {
         sessionRule.delegateDuringNextWait(object : WebNotificationDelegate {
             @GeckoSessionTestRule.AssertCalled
             override fun onShowNotification(notification: WebNotification) {
+                notification.show()
                 notificationResult.complete(notification)
             }
         })
@@ -350,6 +419,7 @@ class WebNotificationTest : BaseSessionTest() {
         sessionRule.delegateDuringNextWait(object : WebNotificationDelegate {
             @GeckoSessionTestRule.AssertCalled
             override fun onShowNotification(notification: WebNotification) {
+                notification.show()
                 notificationResult.complete(notification)
             }
         })
@@ -390,4 +460,59 @@ class WebNotificationTest : BaseSessionTest() {
 
         assertThat("Promise should have been resolved.", promiseResult.value as Double, equalTo(1.0))
     }
+
+    private fun openNotificationAction(): GeckoResult<WebNotification> {
+        mainSession.loadTestPath(CLICK_ACTION_PATH)
+        mainSession.waitForPageStop()
+
+        val notificationResult = GeckoResult<WebNotification>()
+
+        sessionRule.delegateDuringNextWait(object : WebNotificationDelegate {
+            @GeckoSessionTestRule.AssertCalled
+            override fun onShowNotification(notification: WebNotification) {
+                notificationResult.complete(notification)
+            }
+        })
+
+        return notificationResult
+    }
+
+    @Test fun openWithActionsAndClickAction() {
+        assumeThat(sessionRule.env.isNightly, equalTo(true))
+
+        val notificationResult = openNotificationAction()
+        val promiseResult = mainSession.evaluatePromiseJS("showNotification()")
+        sessionRule.waitForResult(notificationResult)
+
+        notificationResult.accept { it!!.click("action1") }
+
+        assertThat("Promise should have been resolved.", promiseResult.value as String, equalTo("action1"))
+    }
+
+    @Test fun openWithActionsAndClickTitle() {
+        assumeThat(sessionRule.env.isNightly, equalTo(true))
+
+        val notificationResult = openNotificationAction()
+        val promiseResult = mainSession.evaluatePromiseJS("showNotification()")
+        sessionRule.waitForResult(notificationResult)
+
+        notificationResult.accept { it!!.click() }
+
+        assertThat("Promise should have been resolved.", promiseResult.value as String, equalTo(""))
+    }
+
+    @Test fun openWithActionsAndClickUnknownAction() {
+        assumeThat(sessionRule.env.isNightly, equalTo(true))
+
+        val notificationResult = openNotificationAction()
+        val promiseResult = mainSession.evaluatePromiseJS("showNotification()")
+        sessionRule.waitForResult(notificationResult)
+
+        // This may happen when a tagged notification is replaced with a different set of actions
+        // but at the same time the user clicks the existing notification action just before the
+        // system replaces it.
+        notificationResult.accept { it!!.click("action-unknown") }
+
+        assertThat("Promise should have been resolved.", promiseResult.value as String, equalTo(""))
+    }
 }
