@@ -4,18 +4,14 @@
 
 package org.mozilla.gecko.media;
 
-import android.annotation.SuppressLint;
-import android.annotation.TargetApi;
 import android.media.DeniedByServerException;
 import android.media.MediaCrypto;
 import android.media.MediaDrm;
 import android.media.NotProvisionedException;
 import android.os.AsyncTask;
-import android.os.Build;
 import android.os.Handler;
 import android.os.HandlerThread;
 import android.util.Log;
-import androidx.annotation.RequiresApi;
 import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStreamReader;
@@ -31,8 +27,6 @@ import java.util.HashSet;
 import java.util.UUID;
 import org.mozilla.gecko.util.ProxySelector;
 
-@TargetApi(Build.VERSION_CODES.LOLLIPOP)
-@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
 public class GeckoMediaDrmBridgeV21 implements GeckoMediaDrm {
   protected final String LOGTAG;
   private static final String INVALID_SESSION_ID = "Invalid";
@@ -109,7 +103,6 @@ public class GeckoMediaDrmBridgeV21 implements GeckoMediaDrm {
     }
   }
 
-  @SuppressLint("WrongConstant")
   private void configureVendorSpecificProperty() {
     assertTrue(mDrm != null);
     if (mDrm == null) {
@@ -322,7 +315,6 @@ public class GeckoMediaDrmBridgeV21 implements GeckoMediaDrm {
     return mCrypto;
   }
 
-  @SuppressLint("WrongConstant")
   @Override
   public void setServerCertificate(final byte[] cert) {
     if (DEBUG) Log.d(LOGTAG, "setServerCertificate()");
@@ -678,9 +670,7 @@ public class GeckoMediaDrmBridgeV21 implements GeckoMediaDrm {
     // The EME spec says the messageType is only for optimization and optional.
     // Send 'License_request' as default when it's not available.
     int requestType = LICENSE_REQUEST_INITIAL;
-    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
-      requestType = request.getRequestType();
-    }
+    requestType = request.getRequestType();
     onSessionMessage(session, requestType, request.getData());
   }
 
@@ -765,7 +755,6 @@ public class GeckoMediaDrmBridgeV21 implements GeckoMediaDrm {
   private String getCDMUserAgent() {
     // This user agent is found and hard-coded in Android(L) source code and
     // Chromium project. Not sure if it's gonna change in the future.
-    final String ua = "Widevine CDM v1.0";
-    return ua;
+    return "Widevine CDM v1.0";
   }
 }
