@@ -8,12 +8,15 @@ package org.mozilla.geckoview;
 
 import android.graphics.Bitmap;
 import android.graphics.Rect;
+import android.os.Build;
 import android.view.Surface;
 import android.view.SurfaceControl;
+import android.view.WindowInsets;
 import androidx.annotation.AnyThread;
 import androidx.annotation.NonNull;
 import androidx.annotation.Nullable;
 import androidx.annotation.UiThread;
+import androidx.core.view.WindowInsetsCompat;
 import org.mozilla.gecko.util.ThreadUtils;
 
 /**
@@ -25,6 +28,11 @@ import org.mozilla.gecko.util.ThreadUtils;
 public class GeckoDisplay {
   private final GeckoSession mSession;
 
+  /**
+   * Protected constructor for GeckoDisplay.
+   *
+   * @param session The GeckoSession this display is associated with
+   */
   protected GeckoDisplay(final GeckoSession session) {
     mSession = session;
   }
@@ -527,4 +535,42 @@ public class GeckoDisplay {
   public @NonNull ScreenshotBuilder screenshot() {
     return new ScreenshotBuilder(mSession);
   }
+
+  private static int getKeyboardHeight(@NonNull final WindowInsetsCompat insetsCompat) {
+    if (Build.VERSION.SDK_INT >= 30) {
+      final WindowInsets insets = insetsCompat.toWindowInsets();
+      if (insets == null) {
+        return 0;
+      }
+
+      // https://developer.android.com/reference/android/view/WindowInsets.Type#ime()
+      final int imeHeight = insets.getInsets(WindowInsets.Type.ime()).bottom;
+      if (imeHeight == 0) {
+        return 0;
+      }
+      return imeHeight - insets.getInsets(WindowInsets.Type.navigationBars()).bottom;
+    }
+
+    if (Build.VERSION.SDK_INT >= 29) {
+      return insetsCompat.getInsets(WindowInsetsCompat.Type.ime()).bottom;
+    }
+
+    return insetsCompat.getSystemWindowInsets().bottom;
+  }
+
+  /**
+   * Called when the root window insets changed.
+   *
+   * @param insets the WindowInsetsCompat
+   */
+  @UiThread
+  public void windowInsetsChanged(@NonNull final WindowInsetsCompat insets) {
+    ThreadUtils.assertOnUiThread();
+
+    final int keyboardHeight = getKeyboardHeight(insets);
+
+    if (mSession != null) {
+      mSession.onKeyboardHeight(keyboardHeight);
+    }
+  }
 }
