@@ -115,6 +115,7 @@ public class WebNotification implements Parcelable {
    */
   public final @NonNull int[] vibrate;
 
+  /** Array of actions available for this notification. */
   public final @NonNull WebNotificationAction[] actions;
 
   /**
@@ -155,6 +156,16 @@ public class WebNotification implements Parcelable {
     this.origin = origin;
   }
 
+  /**
+   * This should be called when the app starts showing the notification. This is important, as it
+   * tells the result of the notification request to Web Content.
+   */
+  @UiThread
+  public void show() {
+    ThreadUtils.assertOnUiThread();
+    GeckoAppShell.onNotificationShow(tag, mCookie, origin);
+  }
+
   /**
    * This should be called when the user taps or clicks a notification. Note that this does not
    * automatically dismiss the notification as far as Web Content is concerned. For that, see {@link
@@ -264,6 +275,7 @@ public class WebNotification implements Parcelable {
     origin = Objects.requireNonNull(in.readString());
   }
 
+  /** Parcelable creator for WebNotification instances. */
   public static final Creator<WebNotification> CREATOR =
       new Creator<>() {
         @Override
