@@ -8,8 +8,8 @@ import os
 import platform
 import shutil
 import sys
-import tarfile
 import time
+import zipfile
 
 import mozpack.path as mozpath
 from mach.decorators import Command, CommandArgument, SubCommand
@@ -163,11 +163,11 @@ def create_maven_archive(topobjdir):
     # files which cannot be significantly compressed; attempting to compress
     # the archive is usually expensive in time and results in minimal
     # reduction in size.
-    # Even though the archive is not compressed, use the .xz file extension
-    # so that the taskcluster worker also skips compression.
-    with tarfile.open(os.path.join(gradle_folder, "target.maven.tar.xz"), "w") as tar:
+    with zipfile.ZipFile(
+        os.path.join(gradle_folder, "target.maven.zip"), "w"
+    ) as target_zip:
         for abs_path in get_maven_archive_paths(maven_folder):
-            tar.add(
+            target_zip.write(
                 abs_path,
                 arcname=os.path.join(
                     "geckoview", os.path.relpath(abs_path, maven_folder)
@@ -659,8 +659,7 @@ def emulator(
             logging.WARN,
             "emulator",
             {},
-            "Emulator binary not found.\n"
-            "Install the Android SDK and make sure 'emulator' is in your PATH.",
+            "Emulator binary not found. Try |mach bootstrap|\n",
         )
         return 2
 
@@ -775,7 +774,7 @@ def emulator_reset(command_context):
             logging.ERROR,
             "emulator",
             {},
-            f"Unable to proceed – 'sdkmanager' not found at {sdk_manager_tool_path}. "
+            f"Unable to proceed - 'sdkmanager' not found at {sdk_manager_tool_path}. "
             f"Please run './mach bootstrap' to reinstall your Android SDK.",
         )
         return 1
